@@ -2,7 +2,7 @@
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -98,7 +98,13 @@ char  *window_title = "exim monitor";
 
 
 /* These ones are used by Exim modules included in Eximon. Not all are
-actually relevant to the operation of Eximon. */
+actually relevant to the operation of Eximon. If SPOOL_DIRECTORY is not
+defined (Exim was compiled with it unset), just define it empty. The script
+that fires up the monitor fishes the value out by using -bP anyway. */
+
+#ifndef SPOOL_DIRECTORY
+#define SPOOL_DIRECTORY ""
+#endif
 
 char  *authenticated_id       = NULL;
 char  *authenticated_sender   = NULL;
@@ -116,6 +122,7 @@ BOOL   deliver_firsttime      = FALSE;
 BOOL   deliver_freeze         = FALSE;
 int    deliver_frozen_at      = 0;
 BOOL   deliver_manual_thaw    = FALSE;
+BOOL   dont_deliver           = FALSE;
 char  *dsn_envid              = 0;
 int    dsn_ret                = 0;
 
@@ -166,15 +173,19 @@ char  *sender_helo_name       = NULL;
 char  *sender_host_address    = NULL;
 char  *sender_host_authenticated = NULL;
 char  *sender_host_name       = NULL;
+int    sender_host_port       = 0;
 char  *sender_ident           = NULL;
 BOOL   sender_local           = FALSE;
+BOOL   sender_set_untrusted   = FALSE;
 volatile BOOL sigalrm_seen;
 
 BOOL   split_spool_directory  = FALSE;
-char  *spool_directory        = SPOOL_DIRECTORY
-			  "\0<--------------Space to patch spool_directory->";
+char  *spool_directory        = SPOOL_DIRECTORY;
 int    store_pool             = POOL_MAIN;
 
+char  *tls_cipher             = NULL;
+char  *tls_peerdn             = NULL;
+
 tree_node *tree_duplicates    = NULL;
 tree_node *tree_nonrecipients = NULL;
 tree_node *tree_unusable      = NULL;
