@@ -9,6 +9,12 @@
 /* This is the general header file for all the modules that comprise
 the exim monitor program. */
 
+/* If this macro is defined, Eximon will anonymize all email addresses. This
+feature is just so that screen shots can be obtained for documentation
+purposes! */
+
+/* #define ANONYMIZE */
+
 /* System compilation parameters */
 
 #define queue_index_size  10      /* Size of index into queue */
@@ -81,7 +87,11 @@ anything. */
 
 #include "pcre/pcre.h"
 
-/* Includes from the main source of Exim */
+/* Includes from the main source of Exim. We need to have MAXPACKET defined for
+the benefit of structs.h. One of these days I should tidy up this interface so
+that this kind of kludge isn't needed. */
+
+#define MAXPACKET 1024
 
 #include "macros.h"
 #include "config.h"
