@@ -13,6 +13,23 @@
 queue and displaying the data therefrom. */
 
 
+/* If we are anonymizing for screen shots, define a function to anonymize
+addresses. Otherwise, define a macro that does nothing. */
+
+#ifdef ANONYMIZE
+static char *anon(char *s)
+{
+static char anon_result[256];
+char *ss = anon_result;
+for (; *s != 0; s++) *ss++ = (*s == '@' || *s == '.')? *s : 'x';
+*ss = 0;
+return anon_result;
+}
+#else
+#define anon(x) x
+#endif
+
+
 /*************************************************
 *                 Static variables               *
 *************************************************/
@@ -188,14 +205,23 @@ if (rc != spool_read_OK)
     }
   }
 
-/* Now set up the remaining data */
+/* Now set up the remaining data. */
 
 q->frozen = deliver_freeze;
 
-if (user_null_sender && sender_address != NULL && sender_address[0] != 0)
+if (sender_set_untrusted)
   {
-  q->sender = store_malloc((int)strlen(sender_address) + 6);
-  sprintf(q->sender, "<> (%s)", sender_address);
+  if (sender_address[0] == 0)
+    {
+    q->sender = store_malloc((int)strlen(originator_login) + 6);
+    sprintf(q->sender, "<> (%s)", originator_login);
+    }
+  else
+    {
+    q->sender = store_malloc((int)strlen(sender_address) +
+      (int)strlen(originator_login) + 4);
+    sprintf(q->sender, "%s (%s)", sender_address, originator_login);
+    }
   }
 else
   {
@@ -742,22 +768,26 @@ while (p != NULL)
     if (sk == NULL) break;
     }
 
+  /* Don't use more than one call of anon() in one statement - it uses
+  a fixed static buffer. */
 
   if (ddd != NULL || dd == NULL)
     {
-    text_showf(queue_widget, "%c%2d%c %s %s %-8s %s%s%s",
+    text_showf(queue_widget, "%c%2d%c %s %s %-8s ",
       (p->frozen)? '*' : ' ',
       t, u,
       string_format_size(p->size, big_buffer),
       p->name,
       (p->sender == NULL)? "       " :
-        (p->sender[0] == 0)? "<>     " : p->sender,
+        (p->sender[0] == 0)? "<>     " : anon(p->sender));
+
+    text_showf(queue_widget, "%s%s%s",
       (dd == NULL || dd->address[0] == '*')? "" : "<",
-      (dd == NULL)? "" : dd->address,
+      (dd == NULL)? "" : anon(dd->address),
       (dd == NULL || dd->address[0] == '*')? "" : ">");
 
     if (dd != NULL && dd->parent != NULL && dd->parent->address[0] != '*')
-      text_showf(queue_widget, " parent <%s>", dd->parent->address);
+      text_showf(queue_widget, " parent <%s>", anon(dd->parent->address));
 
     text_show(queue_widget, "\n");
 
@@ -765,9 +795,9 @@ while (p != NULL)
     while (dd != NULL && count++ < queue_max_addresses)
       {
       text_showf(queue_widget, "                                     <%s>",
-        dd->address);
+        anon(dd->address));
       if (dd->parent != NULL && dd->parent->address[0] != '*')
-        text_showf(queue_widget, " parent <%s>", dd->parent->address);
+        text_showf(queue_widget, " parent <%s>", anon(dd->parent->address));
       text_show(queue_widget, "\n");
       dd = dd->next;
       }
