@@ -5,36 +5,25 @@
 /* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 
-#include "pcre/pcre.h"
-#include <sys/types.h>
+extern char *version_string;
+extern char *version_date;
 
-#include "macros.h"
-#include "config.h"
-#include "structs.h"
-#include "globals.h"
-
-/* Don't include functions.h, because it needs types that aren't available.
-Instead, just give a prototype for the one function we need. */
-
-extern void *store_get_3(int, char *, int);
-
-void version_init(void)
+void
+version_init(void)
 {
 int i = 0;
 char today[20];
 
-version_string = "1.62";
+version_string = "1.65";
 
 strcpy(today, __DATE__);
 if (today[4] == ' ') i = 1;
 today[3] = today[6] = '-';
 
-version_date = (char *)store_get(32);
+version_date = (char *)malloc(32);
 version_date[0] = 0;
 strncat(version_date, today+4+i, 3-i);
 strncat(version_date, today, 4);
