@@ -169,12 +169,8 @@ if (smtp_input)
   {
   if (!smtp_batched_input)
     {
-    DEBUG(3)
-      debug_printf("421 %s %s - closing connection.\n",
-      primary_hostname, msg);
-    fprintf(smtp_out, "421 %s %s - closing connection.\r\n",
-      primary_hostname, msg);
-    fflush(smtp_out);
+    smtp_printf("421 %s %s - closing connection.\r\n", primary_hostname, msg);
+    mac_smtp_fflush();
     }
 
   /* Control does not return from moan_smtp_batch(). */
@@ -184,7 +180,7 @@ if (smtp_input)
 
 /* Exit from the program (non-BSMTP cases) */
 
-exit(EXIT_FAILURE);
+exim_exit(EXIT_FAILURE);
 }
 
 
@@ -244,8 +240,8 @@ if (smtp_input)
   {
   msg = "Service not available - SIGTERM or SIGINT received";
   log_write(0, LOG_MAIN, "SMTP connection%s%s closed after %s",
-    sender_fullhost? " from " : "",
-    sender_fullhost? sender_fullhost : "",
+    (sender_fullhost != NULL)? " from " : "",
+    (sender_fullhost != NULL)? sender_fullhost : "",
     (sig == SIGTERM)? "SIGTERM" : "SIGINT");
   }
 else
@@ -605,9 +601,9 @@ The general actions of this function are:
 
   . If there is no from: header, generate one.
 
-  . Otherwise, if the sender is local, check that from: is correct, and
-    if not, generate a Sender header unless message comes from root, or this
-    feature is disabled.
+  . If the sender is local, check that from: is correct, and if not, generate
+    a Sender header unless message comes from a trusted caller, or this feature
+    is disabled.
 
   . Ensure the presence of "to" or "cc" or "bcc".
 
@@ -861,19 +857,31 @@ for (;;)
          ))
 
   /* This is not the end of the line. If this is SMTP input and this is
-  the first character in the line and it is a "." character, ignore it
-  unless the following character is CR or LF. This implements the dot-
-  doubling rule, though header lines starting with dots aren't exactly
-  common. They are legal in RFC 822, though. */
+  the first character in the line and it is a "." character, ignore it.
+  This implements the dot-doubling rule, though header lines starting with
+  dots aren't exactly common. They are legal in RFC 822, though. If the
+  following is CRLF or LF, this is the line that that terminates the
+  entire message. We set message_ended to indicate this has happened (to
+  prevent further reading), and break out of the loop, leaving a blank line,
+  which will serve to terminate the header. */
 
     {
-    if (ptr == 0 && ch == '.' && smtp_input)
+    if (ptr == 0 && ch == '.' && (smtp_input || dot_ends))
       {
       ch = (accept_getc)(fin);
-      if (ch == '\r' || ch == '\n')
+      if (ch == '\r')
         {
-        (accept_ungetc)(ch, fin);
-        ch = '.';
+        ch = (accept_getc)(fin);
+        if (ch != '\n')
+          {
+          accept_ungetc(ch, fin);
+          ch = '\r';             /* bare CR is data */
+          }
+        }
+      if (ch == '\n')
+        {
+        message_ended = END_DOT;
+        break;
         }
       }
 
@@ -918,7 +926,7 @@ for (;;)
           }
         else fprintf(stderr, "exim: message header longer than %d characters "
           "received: message not accepted\n", HEADER_MAXSIZE);
-        exit(error_rc);
+        exim_exit(error_rc);
         }
       }
 
@@ -943,16 +951,13 @@ for (;;)
   message_size++;
 
   /* A blank line signals the end of the headers; release the unwanted
-  space and set next to NULL to indicate this. For SMTP input, or if the
-  "dot_ends" flag is set, a line containing only "." also signals the end
-  of the headers, since it is the end of the whole input. */
+  space and set next to NULL to indicate this. A terminating line containing
+  just '.' will have been recognized above, and turned into a blank line. */
 
-  if (ptr == 1 ||
-     ((smtp_input || dot_ends) && ptr == 2 && next->text[0] == '.'))
+  if (ptr == 1)
     {
     store_reset(next);
     next = NULL;
-    if (ptr == 2) message_ended = END_DOT;   /* So no more gets read */
     break;
     }
 
@@ -1028,7 +1033,8 @@ for (;;)
        ) &&
        regex_match_and_setup(regex_From, next->text, 0, -1))
     {
-    if ((trusted_caller || filter_test != NULL) && !sender_address_forced)
+    if ((trusted_caller || untrusted_set_sender || filter_test != NULL)
+         && !sender_address_forced)
       {
       char *uucp_sender = expand_string(uucp_from_sender);
       if (uucp_sender == NULL)
@@ -1048,9 +1054,14 @@ for (;;)
           if (domain == 0 && newsender[0] != 0)
             newsender = rewrite_address_qualify(newsender, FALSE);
           sender_address = newsender;
-          authenticated_sender = NULL;
-          user_name = "";
-          sender_local = FALSE;
+
+          if (trusted_caller || filter_test != NULL)
+            {
+            authenticated_sender = NULL;
+            originator_name = "";
+            sender_local = FALSE;
+            }
+
           if (filter_test != NULL)
             printf("Sender taken from \"From\" line\n");
           }
@@ -1058,7 +1069,9 @@ for (;;)
       }
     }
 
-  /* Not a leading "From " line. Check to see if it is a valid header line */
+  /* Not a leading "From " line. Check to see if it is a valid header line.
+  Header names may contain any non-control characters except space and colon,
+  amazingly. */
 
   else
     {
@@ -1228,7 +1241,7 @@ for (h = header_list->next; h != NULL; h = h->next)
     }
   else if (header_checkname(h, header_names[hn_sender].name,
     header_names[hn_sender].len))
-      h->type = sender_local? htype_old : htype_sender;
+      h->type = (sender_local && !trusted_caller)? htype_old : htype_sender;
 
   else if (header_checkname(h, header_names[hn_from].name,
     header_names[hn_from].len))
@@ -1244,10 +1257,10 @@ for (h = header_list->next; h != NULL; h = h->next)
         {
         char *s = h->text + header_names[hn_from].len + 1;
         while (isspace((uschar)*s)) s++;
-        if (strncmpic(s, user_login, h->slen - (s - h->text) - 1) == 0)
+        if (strncmpic(s, originator_login, h->slen - (s - h->text) - 1) == 0)
           {
-          header_add(htype_other, "From: %s <%s@%s>\n", user_name, user_login,
-            qualify_domain_sender);
+          header_add(htype_other, "From: %s <%s@%s>\n", originator_name,
+            originator_login, qualify_domain_sender);
           h->type = htype_old;
           DEBUG(9) debug_printf("rewrote \"From:\" header using gecos\n");
           }
@@ -1301,7 +1314,7 @@ if (extract_recip)
     while (recipients_count-- > 0)
       {
       char *s = rewrite_address(recipients_list[recipients_count].address,
-        TRUE, TRUE);
+        TRUE, TRUE, global_rewrite_rules, rewrite_existflags);
       tree_add_nonrecipient(s, FALSE);
       }
     recipients_list = NULL;
@@ -1333,11 +1346,11 @@ if (extract_recip)
           if (error_handling == ERRORS_STDERR)
             {
             fprintf(stderr, "exim: too many recipients\n");
-            exit(EXIT_FAILURE);
+            exim_exit(EXIT_FAILURE);
             }
           else
-            exit(moan_to_sender(ERRMESS_TOOMANYRECIP, NULL, NULL, stdin, FALSE)?
-              error_rc : EXIT_FAILURE);
+            exim_exit(moan_to_sender(ERRMESS_TOOMANYRECIP, NULL, NULL, stdin,
+              FALSE)? error_rc : EXIT_FAILURE);
           }
 
         /* Temporarily terminate the string at this point, and extract
@@ -1547,7 +1560,8 @@ if (log_received_recipients)
 
 for (i = 0; i < recipients_count; i++)
   recipients_list[i].address =
-    rewrite_address(recipients_list[i].address, TRUE, TRUE);
+    rewrite_address(recipients_list[i].address, TRUE, TRUE,
+      global_rewrite_rules, rewrite_existflags);
 
 
 /* Generate our own Received: header. We must not do this earlier because the
@@ -1578,10 +1592,11 @@ header_list->type = htype_received;
 /* If there are any rewriting rules, apply them to the sender address, unless
 it has already been rewitten as part of verification for SMTP input. */
 
-if (rewrite_rules != NULL && !sender_address_rewritten &&
+if (global_rewrite_rules != NULL && !sender_address_rewritten &&
     sender_address[0] != 0)
   {
-  sender_address = rewrite_address(sender_address, FALSE, TRUE);
+  sender_address = rewrite_address(sender_address, FALSE, TRUE,
+    global_rewrite_rules, rewrite_existflags);
   DEBUG(5) debug_printf("rewritten sender = %s\n", sender_address);
   }
 
@@ -1595,7 +1610,8 @@ headers by the -t command line option. */
 
 for (h = header_list->next; h != NULL; h = h->next)
   {
-  header_line *newh = rewrite_header(h, NULL, NULL);
+  header_line *newh = rewrite_header(h, NULL, NULL, global_rewrite_rules,
+    rewrite_existflags, TRUE);
   if (newh != NULL)
     {
     if (h == from_header) from_header = newh;
@@ -1612,42 +1628,57 @@ updated any of the others, since the code below merely tests for their being
 NULL. Beware, though, if any changes require anything else. */
 
 
-/* If there is no from: header, generate one. If this is a delivery error from
-a non-local source, we can't generate a proper address. */
+/* If there is no from: header, generate one. If there is no sender address,
+but the sender is local or this is a local delivery error, use the originator
+login. This shouldn't happen for genuine bounces, but might happen for
+autoreplies. */
 
 if (from_header == NULL)
   {
+  /* Envelope sender is empty */
+
   if (sender_address[0] == 0)
     {
-    if (!smtp_input || sender_local)
-      header_add(htype_from, "%s: Mail Delivery System <Mailer-Daemon@%s>\n",
-        header_names[hn_from].name, qualify_domain_sender);
-    else
-      header_add(htype_from, "%s: Remote Mail Delivery System <>\n",
-        header_names[hn_from].name);
+    if (sender_local || local_error_message)
+      {
+      header_add(htype_from, "%s: %s%s%s@%s%s\n", header_names[hn_from].name,
+        originator_name,
+        (originator_name[0] == 0)? "" : " <",
+        originator_login,
+        qualify_domain_sender,
+        (originator_name[0] == 0)? "" : ">");
+      from_header = header_last;
+      }
     }
+
+  /* There is a non-null envelope sender */
+
   else
     {
     if (!smtp_input || sender_local)
-      header_add(htype_from, "%s: %s%s%s%s\n", header_names[hn_from].name,
-        user_name, (user_name[0] == 0)? "" : " <", sender_address,
-        (user_name[0] == 0)? "" : ">");
+      header_add(htype_from, "%s: %s%s%s%s\n",
+        header_names[hn_from].name,
+        originator_name,
+        (originator_name[0] == 0)? "" : " <",
+        sender_address,
+        (originator_name[0] == 0)? "" : ">");
     else
       header_add(htype_from, "%s: %s\n", header_names[hn_from].name,
         sender_address);
+    from_header = header_last;
     }
   }
 
-/* Otherwise, if the sender is local, check that From: is correct, and if not,
-generate a Sender: header, unless disabled. Any previously-existing Sender:
-header was removed above. Note that sender_local, as well as being TRUE if the
-caller of exim is not trusted, is also true if a trusted caller did not supply
-a -f argument for non-smtp input. To allow trusted callers to forge From:
-without supplying -f, we have to test explicitly here. If the From: header
-contains more than one address, then the call to parse_extract_address fails,
-and a Sender: header is inserted, as required. */
+/* If the sender is local, check that From: is correct, and if not, generate a
+Sender: header, unless disabled. Any previously-existing Sender: header was
+removed above. Note that sender_local, as well as being TRUE if the caller of
+exim is not trusted, is also true if a trusted caller did not supply a -f
+argument for non-smtp input. To allow trusted callers to forge From: without
+supplying -f, we have to test explicitly here. If the From: header contains
+more than one address, then the call to parse_extract_address fails, and a
+Sender: header is inserted, as required. */
 
-else if (sender_local && !trusted_caller && local_from_check)
+if (sender_local && !trusted_caller && local_from_check)
   {
   BOOL make_sender = TRUE;
   int start, end, domain;
@@ -1655,6 +1686,8 @@ else if (sender_local && !trusted_caller && local_from_check)
   char *from_address =
     parse_extract_address(strchr(from_header->text, ':') + 1, &errmess,
       &start, &end, &domain, FALSE);
+  char *login_sender_address = string_sprintf("%s@%s", originator_login,
+    qualify_domain_sender);
 
   /* Remove permitted prefixes and suffixes from the local part
   of the From: address before doing the comparison with the sender. */
@@ -1674,21 +1707,20 @@ else if (sender_local && !trusted_caller && local_from_check)
       }
     if (at != NULL) *at = '@';
 
-    if (strcmpic(sender_address, from_address) == 0 ||
-      (domain == 0 && strcmpic(from_address, user_login) == 0))
+    if (strcmpic(login_sender_address, from_address) == 0 ||
+      (domain == 0 && strcmpic(from_address, originator_login) == 0))
         make_sender = FALSE;
     }
 
   /* We have to cause the Sender header to be rewritten if there are
-  appropriate rewriting rules, because even though sender_address has
-  already been rewritten, the rewriting rules that apply to Sender: may
-  not apply to it. */
+  appropriate rewriting rules. */
 
   if (make_sender)
     {
     header_add(htype_sender, "%s: %s <%s>\n",
-      header_names[hn_sender].name, user_name, sender_address);
-    (void) rewrite_header(header_last, NULL, NULL);
+      header_names[hn_sender].name, originator_name, login_sender_address);
+    (void) rewrite_header(header_last, NULL, NULL, global_rewrite_rules,
+      rewrite_existflags, TRUE);
     }
   }
 
@@ -1752,10 +1784,7 @@ if (recipients_count > 0 && to_header == NULL && bcc_header == NULL &&
     header_add(htype_to, "%s: %s\n", header_names[hn_to].name, temp);
 
     /**** We used to rewrite as well, but that is surely wrong, since
-    the recipients have already been rewritten above.
-
-    (void) rewrite_header(header_last, NULL, NULL);
-    ****/
+    the recipients have already been rewritten above. ****/
     }
   }
 
@@ -1809,10 +1838,8 @@ if (smtp_input && !sender_local && (sender_verify || sender_try_verify))
     the headers), set no message current, and then skip to the end of this
     function. */
 
-    log_write(3, LOG_MAIN|LOG_REJECT, "rejected%s%s: %s",
-      (sender_fullhost == NULL)? "" : " from ",
-      (sender_fullhost == NULL)? "" : host_and_ident(),
-      errmess);
+    log_write(3, LOG_MAIN|LOG_REJECT, "rejected%s: %s",
+      host_and_ident(" from ", NULL), errmess);
     message_id[0] = 0;
     goto TIDYUP;
     }
@@ -1899,14 +1926,12 @@ if (headers_check_syntax)
             if (smtp_input)
               {
               accept_swallow_smtp(fin);
-              smtp_reply = string_sprintf("550 Syntax error in '%s' header: %s",
-                hname, errmess);
+              smtp_reply = string_sprintf("550 Syntax error in '%s' header: "
+                "%.256s", hname, errmess);
 
-              log_write(3, LOG_MAIN|LOG_REJECT, "rejected from%s%s <%s>: "
+              log_write(3, LOG_MAIN|LOG_REJECT, "rejected from%s <%s>: "
                 "syntax error in '%s' header: %s",
-                  (sender_fullhost == NULL)? "" : " ",
-                  (sender_fullhost == NULL)? "" : host_and_ident(),
-                  sender_address, hname, errmess);
+                  host_and_ident(" ", NULL), sender_address, hname, errmess);
 
               message_id[0] = 0;            /* Indicate no message accepted */
               goto TIDYUP;                  /* Skip to end of function */
@@ -1916,17 +1941,15 @@ if (headers_check_syntax)
               fprintf(stderr, "exim: syntax error in '%s' header: %s\n",
                 hname, errmess);
               fclose(data_file);
-              exit(EXIT_FAILURE);
+              exim_exit(EXIT_FAILURE);
               }
             }
 
           /* Just log warning message */
 
-          else log_write(3, LOG_REJECT, "warning: from%s%s <%s>: "
+          else log_write(3, LOG_REJECT, "warning: from%s <%s>: "
                  "syntax error in '%s' header: %s",
-                 (sender_fullhost == NULL)? "" : " ",
-                 (sender_fullhost == NULL)? "" : host_and_ident(),
-                 sender_address, hname, errmess);
+                   host_and_ident(" ", NULL), sender_address, hname, errmess);
           }
 
         /* Advance to the next address */
@@ -2010,11 +2033,6 @@ if (next != NULL)
   {
   char *s = next->text;
   int len = next->slen;
-  if (smtp_input && next->text[0] == '.')
-    {
-    s++;
-    len--;
-    }
   fwrite(s, 1, len, data_file);
   body_linecount++;                 /* Assumes only 1 line */
   }
@@ -2070,34 +2088,29 @@ if (!ferror(data_file) && !(accept_feof)(fin) && message_ended != END_DOT)
   if (message_ended == END_SIZE)
     {
     unlink(spool_name);                /* Lose the data file when closed */
+    if (smtp_input) accept_swallow_smtp(fin);  /* Swallow incoming SMTP */
+
+    log_write(2, LOG_MAIN|LOG_REJECT, "rejected from <%s>%s%s%s%s: "
+      "message too large: %d recipient%s size=%d max=%d",
+      sender_address,
+      (sender_fullhost == NULL)? "" : " H=",
+      (sender_fullhost == NULL)? "" : sender_fullhost,
+      (sender_ident == NULL)? "" : " U=",
+      (sender_ident == NULL)? "" : sender_ident,
+      recipients_count,
+      (recipients_count == 1)? "" : "s",
+      message_size,
+      thismessage_size_limit);
+
     if (smtp_input)
       {
-      accept_swallow_smtp(fin);
       smtp_reply = "552 Message too large";
-      log_write(2, LOG_MAIN|LOG_REJECT, "rejected from <%s>%s%s%s%s: "
-        "message too large: %d recipient%s size=%d max=%d",
-        sender_address,
-        (sender_fullhost == NULL)? "" : " H=",
-        (sender_fullhost == NULL)? "" : sender_fullhost,
-        (sender_ident == NULL)? "" : " U=",
-        (sender_ident == NULL)? "" : sender_ident,
-        recipients_count,
-        (recipients_count == 1)? "" : "s",
-        message_size,
-        thismessage_size_limit);
       message_id[0] = 0;               /* Indicate no message accepted */
       goto TIDYUP;                     /* Skip to end of function */
       }
     else
       {
       fseek(data_file, (long int)DATA_START_OFFSET, SEEK_SET);
-      log_write(2, LOG_MAIN|LOG_REJECT, "rejected local message from <%s>: "
-        "too large: %d recipient%s size=%d max=%d",
-        sender_address,
-        recipients_count,
-        (recipients_count == 1)? "" : "s",
-        message_size,
-        thismessage_size_limit);
       if (error_handling == ERRORS_SENDER)
         {
         DEBUG(1) debug_printf("Data too large while reading input for "
@@ -2110,7 +2123,7 @@ if (!ferror(data_file) && !(accept_feof)(fin) && message_ended != END_DOT)
         }
       else fprintf(stderr, "exim: message too large\n");
       fclose(data_file);
-      exit(error_rc);
+      exim_exit(error_rc);
       }
     }
   }
@@ -2170,7 +2183,7 @@ if (fflush(data_file) == EOF || ferror(data_file) ||
       }
     else fprintf(stderr, "exim: %s\n", msg);
     fclose(data_file);
-    exit(error_rc);
+    exim_exit(error_rc);
     }
   }
 
@@ -2249,7 +2262,7 @@ if (bad_addresses != NULL || recipients_count == 0)
     {
     unlink(spool_name);
     fclose(data_file);
-    exit(error_rc);
+    exim_exit(error_rc);
     }
   }
 
@@ -2298,7 +2311,7 @@ if ((msg_size = spool_write_header(message_id, FALSE)) < 0)
       }
     else fprintf(stderr, "exim: %s\n", msg);
     fclose(data_file);
-    exit(error_rc);
+    exim_exit(error_rc);
     }
   }
 
@@ -2310,9 +2323,9 @@ actually an addr-spec, we can use the parse routine to canonicize it. */
 
 else
   {
-  char *errmess, *subject, *old_id, *log_auth;
+  char *errmess, *old_id, *s;
   struct stat statbuf;
-  int start, end, domain;
+  int start, end, domain, size, ptr;
 
   /* Reset signal handlers to ignore signals that previously would have caused
   the message to be abandoned. */
@@ -2335,6 +2348,45 @@ else
 
   HOST_CHECKING_WRITE_LOG:
 
+  /* Generate a "message received" log entry. We do this by building up a
+  dynamic string as required. Since we commonly want to add two items at a
+  time, use a macro to simplify the coding. */
+
+  size = 256;
+  ptr = 0;
+  s = store_get(size);
+
+  s = mac_cat2(s, &size, &ptr, "<= ",
+    (sender_address[0] == 0)? "<>" : sender_address);
+  if (message_reference != NULL)
+    s = mac_cat2(s, &size, &ptr, " R=", message_reference);
+  if (sender_fullhost != NULL)
+    s = mac_cat2(s, &size, &ptr, " H=", sender_fullhost);
+  if (sender_ident != NULL)
+    s = mac_cat2(s, &size, &ptr, " U=", sender_ident);
+  if (received_protocol != NULL)
+    s = mac_cat2(s, &size, &ptr, " P=", received_protocol);
+
+  #ifdef SUPPORT_TLS
+  if (tls_log_cipher && tls_cipher != NULL)
+    s = mac_cat2(s, &size, &ptr, " X=", tls_cipher);
+  if (tls_log_peerdn && tls_peerdn != NULL)
+    {
+    s = mac_cat2(s, &size, &ptr, " DN=\"", tls_peerdn);
+    s = string_cat(s, &size, &ptr, "\"", 1);
+    }
+  #endif
+
+  if (sender_host_authenticated != NULL)
+    {
+    s = mac_cat2(s, &size, &ptr, " A=", sender_host_authenticated);
+    if (authenticated_id != NULL)
+      s = mac_cat2(s, &size, &ptr, ":", authenticated_id);
+    }
+
+  sprintf(big_buffer, "%d", msg_size);
+  s = mac_cat2(s, &size, &ptr, " S=", big_buffer);
+
   /* If an addr-spec in a message-id contains a quoted string, it can contain
   any characters except " \ and CR and so in particular it can contain NL!
   Therefore, make sure we use a printing-characters only version for the log. */
@@ -2345,7 +2397,8 @@ else
     {
     old_id = parse_extract_address(strchr(old_id, ':') + 1,
       &errmess, &start, &end, &domain, FALSE);
-    if (old_id != NULL) old_id = string_printing(old_id);
+    if (old_id != NULL)
+      s = mac_cat2(s, &size, &ptr, " id=", string_printing(old_id));
     }
 
   /* If subject logging is turned on, create suitable printing-character
@@ -2354,14 +2407,11 @@ else
   if (log_subject && subject_header != NULL)
     {
     int i;
-    char *p;
-    char *s = subject_header->text + 7;
+    char *p = big_buffer;
+    char *ss = subject_header->text + 7;
 
-    while (*s != ':') s++;
-    while (isspace((uschar)*(++s)));
-
-    strcpy(big_buffer, " T=\"");
-    p = big_buffer + (int)strlen(big_buffer);
+    while (*ss != ':') ss++;
+    while (isspace((uschar)*(++ss)));
 
     /* Backslash-quote any double quotes or backslashes so as to make a
     a C-like string, and leave off the terminating newline. However, if
@@ -2369,56 +2419,35 @@ else
     the newline, so we must first test for this case. Then turn any
     non-printers into escape sequences. */
 
-    if (*s != 0) for (i = 0; i < 100 && s[i+1] != 0; i++)
+    *p++ = '\"';
+    if (*ss != 0) for (i = 0; i < 100 && ss[i+1] != 0; i++)
       {
-      if (s[i] == '\"' || s[i] == '\\') *p++ = '\\';
-      *p++ = s[i];
+      if (ss[i] == '\"' || ss[i] == '\\') *p++ = '\\';
+      *p++ = ss[i];
       }
     *p++ = '\"';
     *p = 0;
-    subject = string_printing(big_buffer);
+    ss = string_printing(big_buffer);
+    s = mac_cat2(s, &size, &ptr, " T=", ss);
     }
-  else subject = "";
 
-  /* Generate a "message received" log entry. First set up log data for the
-  authenticated id - we want to log it when it is the result of an SMTP
-  AUTH command, but not for messages from local callers. While writing to the
-  log, set a flag to cause a call to accept_bomb_out() if the log cannot be
-  opened. Do logging of messages accepted despite RBL matches here. */
+  /* Terminate the string: string_cat() leaves room, but does not put the
+  zero in. */
 
-  if (sender_host_authenticated != NULL)
-    {
-    log_auth = string_sprintf(" A=%s%s%s",
-      sender_host_authenticated,
-      (authenticated_id != NULL)? ":" : "",
-      (authenticated_id != NULL)? authenticated_id : "");
-    }
-  else log_auth = "";    /* Do this here because of the host checking case */
+  s[ptr] = 0;
+
+  /* While writing to the log, set a flag to cause a call to accept_bomb_out()
+  if the log cannot be opened. Do logging of messages accepted despite RBL
+  matches here. */
 
   accept_call_bombout = TRUE;
-
   log_write(0, LOG_MAIN | (log_received_recipients? LOG_RECIPIENTS : 0) |
-                          (log_received_sender? LOG_SENDER : 0),
-    "<= %s%s%s%s%s%s%s%s%s%s S=%d%s%s%s",
-    (sender_address[0] == 0)?    "<>" : sender_address,
-    (message_reference != NULL)? " R=" : "",
-    (message_reference != NULL)? message_reference : "",
-    (sender_fullhost != NULL)?   " H=" : "",
-    (sender_fullhost != NULL)?   sender_fullhost : "",
-    (sender_ident != NULL)?      " U=" : "",
-    (sender_ident != NULL)?      sender_ident : "",
-    (received_protocol != NULL)? " P=" : "",
-    (received_protocol != NULL)? received_protocol : "",
-    log_auth,
-    msg_size,
-    (old_id != NULL)? " id=" : "",
-    (old_id != NULL)? old_id : "",
-    subject);
-
+                          (log_received_sender? LOG_SENDER : 0), "%s", s);
   if (rbl_host && rbl_log_headers)
     log_write(1, LOG_REJECT, "Message accepted from RBL host");
-
   accept_call_bombout = FALSE;
+
+  store_reset(s);   /* The store can be reused */
   }
 
 
@@ -2468,7 +2497,6 @@ if (smtp_input)
           "\n**** SMTP testing: that is not a real message id!\n\n");
       }
     else smtp_printf("%.1024s\r\n", smtp_reply);
-    fflush(smtp_out);
     }
 
   /* For batched SMTP, generate an error message on failure, and do
