@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -79,15 +79,14 @@ Returns:         nothing
 */
 
 static void
-compute_cram_md5(char *secret, char *challenge,
-  unsigned char *digestptr)
+compute_cram_md5(char *secret, char *challenge, uschar *digestptr)
 {
 md5 base;
 int i;
 int len = (int)strlen(secret);
-unsigned char isecret[64];
-unsigned char osecret[64];
-unsigned char md5secret[16];
+uschar isecret[64];
+uschar osecret[64];
+uschar md5secret[16];
 
 /* If the secret is longer than 64 characters, we compute its MD5 digest
 and use that. */
@@ -95,7 +94,7 @@ and use that. */
 if (len > 64)
   {
   md5_start(&base);
-  md5_end(&base, (unsigned char *)secret, len, md5secret);
+  md5_end(&base, (uschar *)secret, len, md5secret);
   secret = (char *)md5secret;
   len = 16;
   }
@@ -118,7 +117,7 @@ for (i = 0; i < 64; i++)
 
 md5_start(&base);
 md5_mid(&base, isecret);
-md5_end(&base, (unsigned char *)challenge, (int)strlen(challenge), md5secret);
+md5_end(&base, (uschar *)challenge, (int)strlen(challenge), md5secret);
 
 /* Compute the outer MD5 digest */
 
@@ -143,13 +142,13 @@ auth_cram_md5_options_block *ob =
 char *challenge = string_sprintf("<%d.%d@%s>", getpid(), time(NULL),
   primary_hostname);
 char *clear, *secret;
-unsigned char digest[16];
+uschar digest[16];
 int i, rc, len;
 
 /* If we are running in the test harness, always send the same challenge,
 an example string taken from the RFC. */
 
-if (strcmp(running_status, "<<<testing>>>") == 0)
+if (running_in_test_harness)
   challenge = "<1896.697170952@postoffice.reston.mci.net>";
 
 /* No data should have been sent with the AUTH command */
@@ -166,8 +165,8 @@ digest, expressed in hex. Extract the user name and put it in $1. Then check
 that the remaining length is 32. */
 
 expand_nstring[1] = clear;
-while (*clear != 0 && !isspace(*clear)) clear++;
-if (!isspace(*clear)) return FAIL;
+while (*clear != 0 && !isspace((uschar)*clear)) clear++;
+if (!isspace((uschar)*clear)) return FAIL;
 *clear++ = 0;
 
 expand_nlength[1] = clear - expand_nstring[1] - 1;
@@ -219,7 +218,8 @@ return OK;
 int
 auth_cram_md5_client(
   auth_instance *ablock,                 /* authenticator block */
-  int sock,                              /* connected socket */
+  smtp_inblock *inblock,                 /* input connection */
+  smtp_outblock *outblock,               /* output connection */
   int timeout,                           /* command timeout */
   char *buffer,                          /* for reading response */
   int buffsize)                          /* size of buffer */
@@ -230,7 +230,7 @@ char *secret = expand_string(ob->client_secret);
 char *name = expand_string(ob->client_name);
 char *challenge, *p;
 int i;
-unsigned char digest[16];
+uschar digest[16];
 
 /* If expansion of either the secret or the user name failed, return FORCEFAIL
 or ERROR, as approriate. */
@@ -248,9 +248,10 @@ if (secret == NULL || name == NULL)
 /* Initiate the authentication exchange and read the challenge, which arrives
 in base 64. */
 
-if (!smtp_write_command(sock, "AUTH %s\r\n", ablock->public_name)) return
-  FAIL_SEND;
-if (!smtp_read_response(sock, buffer, buffsize, '3', timeout)) return FAIL;
+if (!smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name))
+  return FAIL_SEND;
+if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
+  return FAIL;
 
 if (auth_b64decode(buffer + 4, &challenge) < 0)
   {
@@ -281,10 +282,10 @@ in big_buffer, but auth_b64encode() returns its result in working store,
 so calling smtp_write_command(), which uses big_buffer, is OK. */
 
 buffer[0] = 0;
-if (!smtp_write_command(sock, "%s\r\n", auth_b64encode(big_buffer,
+if (!smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(big_buffer,
   p - big_buffer))) return FAIL_SEND;
 
-return smtp_read_response(sock, buffer, buffsize, '2', timeout)?
+return smtp_read_response(inblock, (uschar *)buffer, buffsize, '2', timeout)?
   OK : FAIL;
 }
 
