@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -26,6 +26,7 @@ extern auth_cram_md5_options_block auth_cram_md5_option_defaults;
 
 extern void auth_cram_md5_init(auth_instance *);
 extern int auth_cram_md5_server(auth_instance *, char *);
-extern int auth_cram_md5_client(auth_instance *, int, int, char *, int);
+extern int auth_cram_md5_client(auth_instance *, smtp_inblock *,
+                                smtp_outblock *, int, char *, int);
 
 /* End of cram_md5.h */
