@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -30,7 +30,7 @@ auth_get_data(char **aptr, char *challenge)
 int c;
 int p = 0;
 smtp_printf("334 %s\r\n", auth_b64encode(challenge, (int)strlen(challenge)));
-while ((c = smtp_getc(smtp_in)) != '\n' && c != EOF)
+while ((c = accept_getc(smtp_in)) != '\n' && c != EOF)
   {
   if (p >= big_buffer_size - 1) return BAD64;
   big_buffer[p++] = c;
