@@ -58,7 +58,7 @@ Returns:     nothing
 */
 
 void
-md5_mid(md5 *base, const unsigned char *text)
+md5_mid(md5 *base, const uschar *text)
 {
 register unsigned int a = base->abcd[0];
 register unsigned int b = base->abcd[1];
@@ -228,10 +228,10 @@ Returns:    nothing
 */
 
 void
-md5_end(md5 *base, const unsigned char *text, int length, unsigned char *digest)
+md5_end(md5 *base, const uschar *text, int length, uschar *digest)
 {
 int i;
-unsigned char work[64];
+uschar work[64];
 
 /* Process in chunks of 64 until we have less than 64 bytes left. */
 
@@ -334,8 +334,8 @@ int main(void)
 md5 base;
 int i = 0x01020304;
 char *ctest = (char *)(&i);
-unsigned char buffer[256];
-unsigned char digest[16];
+uschar buffer[256];
+uschar digest[16];
 printf("Checking md5: %s-endian\n", (ctest[0] == 0x04)? "little" : "big");
 
 for (i = 0; i < sizeof(tests)/sizeof(char *); i += 2)
