@@ -175,7 +175,8 @@ return DEFER;
 int
 auth_plaintext_client(
   auth_instance *ablock,                 /* authenticator block */
-  int sock,                              /* connected socket */
+  smtp_inblock *inblock,                 /* connection inblock */
+  smtp_outblock *outblock,               /* connection outblock */
   int timeout,                           /* command timeout */
   char *buffer,                          /* buffer for reading response */
   int buffsize)                          /* size of buffer */
@@ -206,8 +207,8 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
     char *ssave = string_copy(s);
     if (!first)
       {
-      if (smtp_write_command(sock, "*\r\n"))
-        (void) smtp_read_response(sock, buffer, buffsize, '2', timeout);
+      if (smtp_write_command(outblock, FALSE, "*\r\n"))
+        (void) smtp_read_response(inblock, US buffer, buffsize, '2', timeout);
       }
     if (expand_string_forcedfail) return FORCEFAIL;
     string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
@@ -239,12 +240,13 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
   if (first)
     {
     first = FALSE;
-    if (!smtp_write_command(sock, "AUTH %s%s%s\r\n", ablock->public_name,
-      (len == 0)? "" : " ", auth_b64encode(ss, len))) return FAIL_SEND;
+    if (!smtp_write_command(outblock, FALSE, "AUTH %s%s%s\r\n",
+         ablock->public_name, (len == 0)? "" : " ", auth_b64encode(ss, len)))
+      return FAIL_SEND;
     }
   else
     {
-    if (!smtp_write_command(sock, "%s\r\n", auth_b64encode(ss, len)))
+    if (!smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(ss, len)))
       return FAIL_SEND;
     }
 
@@ -252,7 +254,7 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
   has succeeded. There may be more data to send, but is there any point
   in provoking an error here? */
 
-  if (smtp_read_response(sock, buffer, buffsize, '2', timeout)) return OK;
+  if (smtp_read_response(inblock, US buffer, buffsize, '2', timeout)) return OK;
 
   /* Not a success response. If errno != 0 there is some kind of transmission
   error. Otherwise, check the response code in the buffer. If it starts with
@@ -265,8 +267,8 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
 
   if (text == NULL)
     {
-    if (smtp_write_command(sock, "*\r\n"))
-      (void)smtp_read_response(sock, buffer, buffsize, '2', timeout);
+    if (smtp_write_command(outblock, FALSE, "*\r\n"))
+      (void)smtp_read_response(inblock, US buffer, buffsize, '2', timeout);
     string_format(buffer, buffsize, "Too few items in client_send in %s "
       "authenticator", ablock->name);
     return ERROR;
