@@ -26,6 +26,7 @@ extern auth_plaintext_options_block auth_plaintext_option_defaults;
 
 extern void auth_plaintext_init(auth_instance *);
 extern int auth_plaintext_server(auth_instance *, char *);
-extern int auth_plaintext_client(auth_instance *, int, int, char *, int);
+extern int auth_plaintext_client(auth_instance *, smtp_inblock *,
+                                 smtp_outblock *, int, char *, int);
 
 /* End of plaintext.h */
