@@ -68,8 +68,7 @@ pid = fork();
 
 /* Child process: make the reading end of the pipe into the standard input and
 close the writing end. If debugging, pass debug_fd as stderr. Then re-exec
-Exim. Exec failure is signalled with EX_UNAVAILABLE, but we don't expect this
-to occur! */
+Exim. Failure is signalled with EX_UNAVAILABLE, but this shouldn't occur! */
 
 if (pid == 0)
   {
@@ -158,11 +157,13 @@ oldsignal = signal(SIGCHLD, SIG_DFL);
 pid = fork();
 
 /* The child process becomes a process group leader if requested, and then
-organizes the pipes. */
+organizes the pipes. Any unexpected failure is signalled with EX_UNAVAILBLE;
+these are all "should never occur" failures, except perhaps for exec failing
+because the command doesn't exist. */
 
 if (pid == 0)
   {
-  if (make_leader) setpgid(0,0);
+  if (make_leader && setpgid(0,0) < 0) goto CHILD_FAILED;
 
   close(inpfd[pipe_write]);
   force_fd(inpfd[pipe_read], 0);
@@ -182,17 +183,17 @@ if (pid == 0)
   SIGUSR1 is ignored, as the process won't have the privilege to
   write to the process log. */
 
-  if (newgid != NULL) setgid(*newgid);
+  if (newgid != NULL && setgid(*newgid) < 0) goto CHILD_FAILED;
   if (newuid != NULL)
     {
     signal(SIGUSR1, SIG_IGN);
-    setuid(*newuid);
+    if (setuid(*newuid) < 0) goto CHILD_FAILED;
     }
-  umask(newumask);
+  (void)umask(newumask);
 
   /* Set the working directory if required */
 
-  if (wd != NULL) chdir(wd);
+  if (wd != NULL && chdir(wd) < 0) goto CHILD_FAILED;
 
   /* Now do the exec */
 
@@ -203,6 +204,7 @@ if (pid == 0)
   losing the actual errno we got back, because there is no way to return
   this. */
 
+  CHILD_FAILED:
   _exit(EX_UNAVAILABLE);      /* Note: must be _exit(), NOT exit() */
   }
 
