@@ -125,10 +125,12 @@ pid_t pid;
 int dup_accept_socket = dup(accept_socket);
 void *reset_point = store_get(0);
 
-/* Make the address available in ASCII representation. */
+/* Make the address available in ASCII representation, and also fish out
+the remote port. */
 
-sender_host_address = host_ntoa(-1, accepted, NULL);
-DEBUG(1) debug_printf("Connection request from %s\n", sender_host_address);
+sender_host_address = host_ntoa(-1, accepted, NULL, &sender_host_port);
+DEBUG(1) debug_printf("Connection request from %s/%d\n", sender_host_address,
+  sender_host_port);
 
 /* Set up the file for writing replies, then check that duplication of the
 socket worked, and if so, make an input file. */
@@ -255,7 +257,7 @@ if (pid == 0)
        &size) == 0)
     {
     char *buffer = store_malloc(sizeof(ip_address_item));
-    interface_address = host_ntoa(-1, &interface_sockaddr, buffer);
+    interface_address = host_ntoa(-1, &interface_sockaddr, buffer, NULL);
     DEBUG(9) debug_printf("interface address = %s\n", interface_address);
     }
 
@@ -304,7 +306,8 @@ if (pid == 0)
 
   if (!smtp_start_session())
     {
-    fflush(smtp_out);
+    mac_smtp_fflush();
+    search_tidyup();
     _exit(EXIT_SUCCESS);
     }
 
@@ -333,7 +336,8 @@ if (pid == 0)
       }
     else
       {
-      fflush(smtp_out);
+      mac_smtp_fflush();
+      search_tidyup();
       _exit((rc == 0)? EXIT_SUCCESS : EXIT_FAILURE);
       }
 
@@ -419,11 +423,25 @@ if (pid == 0)
 
     else
       {
+      /* Before forking, ensure that the C output buffer is flushed. Otherwise
+      anything that it in it will get duplicated, leading to duplicate copies
+      of the pending output. */
+
+      mac_smtp_fflush();
+
       if ((pid = fork()) == 0)
         {
         fclose(smtp_in);
         fclose(smtp_out);
         smtp_in = NULL;
+
+        /* Don't ever molest the parent's SSL connection, but do clean up
+        the data structures if necessary. */
+
+        #ifdef SUPPORT_TLS
+        tls_close(FALSE);
+        #endif
+
         if (security_level == 3 || security_level == 2)
           {
           int i = 0;
@@ -453,8 +471,11 @@ if (pid == 0)
             strerror(errno));
           }
 
+
+
         /* No need to re-exec */
         (void)deliver_message(message_id, FALSE, FALSE);
+        search_tidyup();
         _exit(EXIT_SUCCESS);
         }
       }
@@ -868,6 +889,11 @@ if (daemon_listen)
     listen(listen_sockets[sk], smtp_connect_backlog);
   }
 
+/* Set up the handler for SIGHUP, which causes a restart of the daemon. */
+
+sighup_seen = FALSE;
+signal(SIGHUP, sighup_handler);
+
 /* Write the pid to a known file for assistance in identification. Make it
 read-only. We do this before giving up root privilege, because on some systems
 it is necessary to be root in order to write into the pid file directory. */
@@ -891,10 +917,7 @@ implying the use of these functions on such systems. */
 if (exim_uid_set)
   {
   if (security_level >= 2)
-    {
-    setgid(exim_gid);
-    setuid(exim_uid);
-    }
+    exim_setugid(exim_uid, exim_gid, "running the daemon");
   else
     {
     mac_setegid(exim_gid);
@@ -917,11 +940,6 @@ if (queue_interval > 0 && queue_run_max > 0)
 sigchld_seen = FALSE;
 signal(SIGCHLD, main_sigchld_handler);
 
-/* Set up the handler for SIGHUP, which causes a restart of the daemon. */
-
-sighup_seen = FALSE;
-signal(SIGHUP, sighup_handler);
-
 /* If we are to run the queue periodically, pretend the alarm has just
 gone off. This will cause the first queue-runner to get kicked off straight
 away, and the alarm to be reset. */
