@@ -66,14 +66,18 @@ Arguments:
   name     The single-component name of one of Exim's database files.
   flags    Standard O_xxx flags indicating the type of open required.
   dbblock  Points to an open_db block to be filled in.
+  lof      If TRUE, write to the log for actual open failures (locking failures
+           are always logged).
 
-Returns:   NULL if the open failed, or the locking failed;
-           otherwise dbblock, which contains the dbm pointer and the fd
-           of the locked lock file.
+Returns:   NULL if the open failed, or the locking failed. After locking
+           failures, errno is zero.
+
+           On success, dbblock is returned. This contains the dbm pointer and
+           the fd of the locked lock file.
 */
 
 open_db *
-dbfn_open(char *name, int flags, open_db *dbblock)
+dbfn_open(char *name, int flags, open_db *dbblock, BOOL lof)
 {
 int rc;
 BOOL read_only = flags == O_RDONLY;
@@ -108,6 +112,7 @@ if (dbblock->lockfd < 0)
     {
     log_write(0, LOG_MAIN, "%s",
       string_open_failed(errno, "database lock file %s", buffer));
+    errno = 0;      /* Indicates locking failure */
     return NULL;
     }
   }
@@ -132,6 +137,7 @@ if (rc < 0)
     ((flags & O_RDONLY) != 0)? "read" : "write", buffer,
     (errno == ETIMEDOUT)? "timed out" : strerror(errno));
   close(dbblock->lockfd);
+  errno = 0;       /* Indicates locking failure */
   return NULL;
   }
 
@@ -150,13 +156,17 @@ file, leaving it open for both reading and writing. */
 if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDONLY)
   EXIM_DBOPEN(buffer, O_RDWR|O_CREAT, DB_MODE, &(dbblock->dbptr));
 
-/* If the open has failed, return NULL */
+/* If the open has failed, return NULL, leaving errno set. If lof is TRUE,
+log the event - also for debugging - but not if the file just doesn't exist. */
 
 if (dbblock->dbptr == NULL)
   {
-  DEBUG(9) debug_printf("%s\n",
-    string_open_failed(errno, "DB file %s", buffer));
+  int save_errno = errno;
+  if ((lof || debug_level >= 9) && errno != ENOENT)
+    log_write(0, LOG_MAIN, "%s",
+      string_open_failed(errno, "DB file %s", buffer));
   close(dbblock->lockfd);
+  errno = save_errno;
   return NULL;
   }
 
@@ -219,14 +229,8 @@ EXIM_DATUM key_datum, result_datum;
 
 DEBUG(9) debug_printf("dbfn_read: key=%s\n", key);
 
-/* Native DB 2.x has additional fields in the datum that
-must be cleared. */
-
-#ifdef EXIM_CLEAR_DATUMSTRUCT
-memset(&key_datum, 0, sizeof(key_datum));
-memset(&result_datum, 0, sizeof(result_datum));
-#endif
-
+EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
+EXIM_DATUM_INIT(result_datum);      /* to be cleared before use. */
 EXIM_DATUM_DATA(key_datum) = key;
 EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
 
@@ -247,6 +251,7 @@ if (setjmp(read_env) == 0)
   yield = store_get(EXIM_DATUM_SIZE(result_datum));
   memcpy(yield, EXIM_DATUM_DATA(result_datum), EXIM_DATUM_SIZE(result_datum));
   if (length != NULL) *length = EXIM_DATUM_SIZE(result_datum);
+  EXIM_DATUM_FREE(result_datum);    /* Some DBM libs require freeing */
   return yield;
   }
 
@@ -290,14 +295,8 @@ gptr->time_stamp = time(NULL);
 
 DEBUG(9) debug_printf("dbfn_write: key=%s\n", key);
 
-/* Native DB 2.x has additional fields in the datum that
-must be cleared. */
-
-#ifdef EXIM_CLEAR_DATUMSTRUCT
-memset(&key_datum, 0, sizeof(key_datum));
-memset(&value_datum, 0, sizeof(value_datum));
-#endif
-
+EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
+EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
 EXIM_DATUM_DATA(key_datum) = key;
 EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
 EXIM_DATUM_DATA(value_datum) = (char *)ptr;
@@ -323,14 +322,7 @@ int
 dbfn_delete(open_db *dbblock, char *key)
 {
 EXIM_DATUM key_datum;
-
-/* Native DB 2.x has additional fields in the datum that
-must be cleared. */
-
-#ifdef EXIM_CLEAR_DATUMSTRUCT
-memset(&key_datum, 0, sizeof(key_datum));
-#endif
-
+EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require clearing */
 EXIM_DATUM_DATA(key_datum) = key;
 EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
 return EXIM_DBDEL(dbblock->dbptr, key_datum);
@@ -365,13 +357,8 @@ value_datum = value_datum;    /* dummy; not all db libraries use this */
 
 if (start) EXIM_DBCREATE_CURSOR(dbblock->dbptr, cursor);
 
-/* Native DB 2.x has additional fields in the datum that
-must be cleared. */
-
-#ifdef EXIM_CLEAR_DATUMSTRUCT
-memset(&key_datum, 0, sizeof(key_datum));
-memset(&value_datum, 0, sizeof(value_datum));
-#endif
+EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
+EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
 
 yield = (EXIM_DBSCAN(dbblock->dbptr, key_datum, value_datum, start, *cursor))?
   EXIM_DATUM_DATA(key_datum) : NULL;
@@ -451,6 +438,8 @@ printf("Berkeley DB: %s\n", DB_VERSION_STRING);
   #endif
 #elif defined(_DBM_RDONLY) || defined(dbm_dirfno)
 printf("probably ndbm\n");
+#elif defined(USE_TDB)
+printf("using tdb\n");
 #else
 printf("probably GDBM\n");
 #endif
@@ -502,17 +491,13 @@ while (fgets(buffer, 256, stdin) != NULL)
       }
 
     start = clock();
-    odb = dbfn_open(s, O_RDWR|O_CREAT, dbblock + i);
+    odb = dbfn_open(s, O_RDWR|O_CREAT, dbblock + i, TRUE);
     stop = clock();
 
-    if (odb == NULL)
-      printf("Failed to open %s: %s\n", s,
-        (errno == ETIMEDOUT)? "timed out" : strerror(errno));
-    else
+    if (odb != NULL)
       {
       current = i;
       printf("opened %d\n", current);
-
       }
     }
 
