@@ -119,16 +119,16 @@ direct_set_ugid(address_item *addr, ugid_block *ugid)
 if (ugid->uid_set)
   {
   addr->uid = ugid->uid;
-  addr->uid_set = TRUE;
+  setflag(addr, af_uid_set);
   }
 
 if (ugid->gid_set)
   {
   addr->gid = ugid->gid;
-  addr->gid_set = TRUE;
+  setflag(addr, af_gid_set);
   }
 
-addr->initgroups = ugid->initgroups;
+if (ugid->initgroups) setflag(addr, af_initgroups);
 }
 
 
@@ -340,7 +340,10 @@ if (dblock->extra_headers != NULL)
       }
     }
 
-  /* Expand succeeded */
+  /* Expand succeeded. Put extra header at the start of the chain because
+  further down it may point to headers from other directors, which may be
+  shared with other addresses. The output function outputs them in reverse
+  order. */
 
   else
     {
@@ -410,7 +413,7 @@ Arguments:
   pattern     the pattern to be matched
   dname       the director name, for error messages
 
-Returns:      OK, FAIL, DEFER, or ERROR
+Returns:      OK, DECLINE, DEFER, or ERROR
 */
 
 int
@@ -419,6 +422,8 @@ direct_match_directory(address_item *addr, char *home_dir, char *pattern,
 {
 BOOL match;
 char *expat = pattern;
+int match_return = OK;
+int nomatch_return = DECLINE;
 
 /* The pattern gets expanded if it contains any expansion items. In this case,
 we mustn't use the regex anchor block that is passed in, because we have to
@@ -434,7 +439,7 @@ if (strchr(expat, '$') != NULL)
       {
       DEBUG(2) debug_printf("%s director expansion of \"%s\" forced "
         "failure\n", dname, pattern);
-      return FAIL;
+      return DECLINE;
       }
     addr->message = string_sprintf("%s director: failed to expand "
       "match_directory string \"%s\": %s", dname, pattern,
@@ -445,9 +450,30 @@ if (strchr(expat, '$') != NULL)
     }
   }
 
+/* If the pattern starts with ! it negates the outcome */
+
+while (*expat == ' ') expat++;
+if (*expat == '!')
+  {
+  match_return = DECLINE;
+  nomatch_return = OK;
+  while (*(++expat) == ' ');
+  }
+
+/* Now do the match */
+
 match = match_check_string(home_dir, expat, -1, TRUE, FALSE, NULL);
 
-if (match) return OK;
+DEBUG(2)
+  {
+  debug_printf("%s director: home directory for %s is \"%s\"\n  "
+    "match against pattern \"%s\" yielded %d\n", dname, addr->local_part,
+    home_dir, pattern, match);
+  if (strcmp(expat, pattern) != 0)
+    debug_printf("  expanded string was \"%s\"\n", expat);
+  }
+
+if (match) return match_return;
 
 if (search_find_defer)
   {
@@ -456,16 +482,7 @@ if (search_find_defer)
   return DEFER;
   }
 
-DEBUG(2)
-  {
-  debug_printf("%s director failed to match home directory for %s\n  "
-    "directory = \"%s\" pattern = \"%s\"\n", dname, addr->local_part,
-    home_dir, pattern);
-  if (strcmp(expat, pattern) != 0)
-    debug_printf("  expanded string was \"%s\"\n", expat);
-  }
-
-return FAIL;
+return nomatch_return;
 }
 
 
@@ -479,10 +496,10 @@ return FAIL;
 /* This is a utility function for use by directors, and called from within
 them. It carries out file existence, sender checking, and general condition
 tests, in that order, because the latter may be using the files mentioned in
-the former. There are two failure yields, partly for historical reasons. FAIL
-causes the directing not to continue if the director has no_more set;
-FAIL_CONDITION does not. Retaining FAIL is for backwards compatibility, but it
-may be that changing them all to FAIL_CONDITION one day might be reasonable.
+the former. There are two failure yields, partly for historical reasons.
+DECLINE causes the directing not to continue if the director has no_more set;
+FAIL_CONDITION does not. Retaining DECLINE is for backwards compatibility, but
+it may be that changing them all to FAIL_CONDITION one day might be reasonable.
 
 Argument:
   dblock    points to the director instance
@@ -490,7 +507,7 @@ Argument:
 
 Returns:    OK               no conditions failed
             DEFER            some condition couldn't be checked
-            FAIL             a condition failed; no_more means no more
+            DECLINE          a condition failed; no_more means no more
             FAIL_CONDITION   a condition failed; ignore no_more
 */
 
@@ -506,7 +523,7 @@ switch (match_exists(dblock->require_files))
   case FAIL:
   DEBUG(9) debug_printf("%s director skipped: file existence failure\n",
     dblock->name);
-  return FAIL;
+  return DECLINE;
 
   case DEFER:
   addr->message = string_sprintf("file existence defer in %s director: %s",
@@ -514,10 +531,15 @@ switch (match_exists(dblock->require_files))
   return DEFER;
   }
 
+/* Arrange not to replace $key in the senders string. This allows users to miss
+off the \ without causing trouble. */
+
 if (dblock->senders != NULL)
   {
-  char *senders = expand_string_panic(dblock->senders, dblock->name,
-    "director");
+  char *senders;
+  lookup_key = "$key";
+  senders = expand_string_panic(dblock->senders, dblock->name, "director");
+  lookup_key = NULL;
   if ((rc = match_sender(senders)) != OK)
     {
     if (rc == DEFER)
@@ -527,7 +549,7 @@ if (dblock->senders != NULL)
       }
     DEBUG(9) debug_printf("%s director: sender match failure\n",
       dblock->name);
-    return FAIL;
+    return DECLINE;
     }
   }
 
@@ -956,21 +978,10 @@ return 0;
 
 /* This function is passed in one address item, for processing by the
 directors. It has been determined that the address is for one of the local
-domains. The action can be one of:
-
-  . adding the address to the chain on addr_local, for local delivery, and
-    returning OK;
-  . adding the address to the chain on addr_remote, for remote delivery, and
-    returning OK;
-  . returning FAIL;
-  . returning DEFER;
-  . adding one or more new addresses, with this one as parent, to the chain
-    on addr_new, for reprocessing ab initio, and returning OK, or
-    OK_CONTINUE.
-
-The verify flag is set if this is being called for verification rather than
-delivery. If the director doesn't have its "verify" flag set, it is skipped.
-Otherwise, the flag is passed to the director in case it needs to know.
+domains. The verify flag is set if this is being called for verification rather
+than delivery. If the director doesn't have its "verify" flag set, it is
+skipped. Otherwise, the flag is passed to the director in case it needs to
+know.
 
 If the director has the "more" flag set false (i.e. "no_more" has been
 specified) then, if it fails to match a name, no further directors are tried.
@@ -996,7 +1007,10 @@ Arguments
                v_recipient if verifying a recipient address
                v_expn if processing an EXPN address
 
-Returns:       OK, OK_CONTINUE, FAIL, or DEFER, as described above
+Returns:       OK           => address was successfully directed
+               FAIL         => address could not be directed
+               DEFER        => temporary problem
+               ERROR        => serious configuration or internal problem
 */
 
 
@@ -1022,7 +1036,7 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
      d = d->next)
   {
   int loopcount = 0;
-  int yield, check_yield;
+  int yield;
   int suffixchar = 0;
   int suffixend = -1;
   char *oldlocal_part = NULL;
@@ -1145,8 +1159,8 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
 
   if (loop_detected)
     {
-    yield = FAIL;            /* Behave as if director failed */
-    goto DIRECTOR_FAILED;
+    yield = DECLINE;            /* Behave as if director failed */
+    goto DIRECTOR_DECLINED;
     }
 
   /* Prefix and suffix (if any) have been stripped; ready to call the
@@ -1163,17 +1177,9 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
   from within the director (because of setting up variables like $home or
   whatever) has failed, and the condition is one that should logically be
   tested before the "more" flag. We therefore force "more" back to be TRUE, and
-  convert the yield to FAIL. */
+  convert the yield to DECLINE. */
 
-  if (yield == FAIL_CONDITION) { more = TRUE; yield = FAIL; }
-
-  /* If yield = OK_CONTINUE the director has generated new address(es) that
-  should always be themselves processed when verifying. Without this, secondary
-  generated addresses are not necessarily processed (if an alias works, you
-  often don't want to know if its children work.) Save a bit of time in
-  avoiding duplicated tests below. */
-
-  check_yield = (yield == OK_CONTINUE)? OK : yield;
+  if (yield == FAIL_CONDITION) { more = TRUE; yield = DECLINE; }
 
   /* Success or deferral means we are finished with this address, as does an
   internal or configuration failure. If we succeed on a director that has
@@ -1185,16 +1191,16 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
   one that has just succeeded, by the loop-avoidance code. This is all handled
   by a function common to both directors and routers. */
 
-  if (check_yield == OK || check_yield == DEFER || check_yield == ERROR)
+  if (yield == OK || yield == DEFER || yield == ERROR)
     {
     addr->director = d;
 
-    if (check_yield == OK)
+    if (yield == OK)
       {
       if ((verify == v_sender && d->fail_verify_sender) ||
           (verify == v_recipient && d->fail_verify_recipient))
         {
-        yield = check_yield = FAIL;
+        yield = FAIL;
         addr->message = string_sprintf("%s director forced verify failure",
           d->name);
         if (*addr_remote == addr) *addr_remote = addr->next;
@@ -1206,12 +1212,12 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
 
     DEBUG(2)
       {
-      if (check_yield == OK)
+      if (yield == OK)
         {
         debug_printf("%s director succeeded for %s%s\n", d->name,
           addr->local_part, (d->unseen)? " (unseen)" : "");
         }
-      else if (check_yield == DEFER)
+      else if (yield == DEFER)
         {
         debug_printf("%s director deferred %s\n", d->name, addr->local_part);
         debug_printf("  message: %s\n", (addr->message == NULL)?
@@ -1228,7 +1234,15 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
     return yield;
     }
 
-  DIRECTOR_FAILED:
+  /* The only remaining yield values are DECLINE or FORCEFAIL. */
+
+  if (yield != DECLINE && yield != FORCEFAIL)
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s director returned unknown "
+      "value %d", d->name, yield);
+
+  /* The director has declined to handle this address. */
+
+  DIRECTOR_DECLINED:
 
   /* Restore prefix & suffix for the next director. */
 
