@@ -43,10 +43,12 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, forbid_pipe)) },
-  { "forbid_special",        opt_bool,
+  { "forbid_special",     opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, forbid_special)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, freeze_missing_include)) },
+  { "hide_child_in_errmsg", opt_bool,
+      (void *)offsetof(aliasfile_director_options_block, hide_child_in_errmsg) },
   { "include_domain",     opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, include_domain)) },
   { "modemask",           opt_octint,
@@ -99,28 +101,29 @@ aliasfile_director_options_block aliasfile_director_option_defaults = {
   NULL,     /* expand_pipe_transport */
   NULL,     /* reply_transport */
   NULL,     /* expand_reply_transport */
-  FALSE,    /* check_ancestor */
-  FALSE,    /* check_local_user */
-  FALSE,    /* forbid_file */
-  FALSE,    /* forbid_pipe */
-  FALSE,    /* forbid_reply */
-  FALSE,    /* one_time */
-  /********************************************/
   NULL,     /* file */
   NULL,     /* queries */
   NULL,     /* query */
-  NULL,     /* syntax_errors_to */
-  NULL,     /* syntax_errors_text */
   022,      /* modemask */
-  -1,       /* partial_match */
-  -1,       /* search_type */
   NULL,     /* owners */
   NULL,     /* owngroups */
+  -1,       /* partial_match */
+  -1,       /* search_type */
+  FALSE,    /* check_ancestor */
+  FALSE,    /* check_local_user */
+  FALSE,    /* expand */
+  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_pipe */
+  FALSE,    /* forbid_reply */
+  FALSE,    /* include_domain */
+  FALSE,    /* hide_child_in_errmsg */
+  FALSE,    /* one_time */
+  FALSE,    /* optional */
+  /********************************************/
+  NULL,     /* syntax_errors_to */
+  NULL,     /* syntax_errors_text */
   FALSE,    /* forbid_include */
   FALSE,    /* forbid_special */
-  FALSE,    /* include_domain */
-  FALSE,    /* expand */
-  FALSE,    /* optional */
   FALSE,    /* qualify_preserve_domain */
   TRUE,     /* freeze_missing_include */
   TRUE,     /* rewrite */
@@ -228,10 +231,8 @@ address_item *generated = NULL;
 char *save_qualify_domain_recipient = qualify_domain_recipient;
 char *aliastext = NULL;
 char *errors_to;
-char *filename;
-char *query;
+char *filename = NULL;
 char *error;
-void *handle;
 header_line *extra_headers;
 char *remove_headers;
 error_block *eblock = NULL;
@@ -239,163 +240,18 @@ transport_instance *transport;
 ugid_block ugid;
 void *reset_point;
 int yield = OK;
-int  extracted, rc;
+int extracted, rc;
 
 /* Perform file existence and sender verification checks. */
 
 rc = direct_check_fsc(dblock, addr);
 if (rc != OK) return rc;
 
-/* For single-key+file search types, set the required file name and expand it.
-If the expansion fails, log the incident and indicate an internal error. The
-file name has already been checked for absoluteness, at initialization time,
-but only if it did not start with an expansion, so we double check here. */
+/* There is now a common function for doing the lookup, shared with
+forwardfile - one of these days the two directors will coalesce. */
 
-if (!mac_islookup(ob->search_type, lookup_querystyle))
-  {
-  query = (ob->include_domain)?
-    string_sprintf("%s@%s", addr->local_part, addr->domain) : addr->local_part;
-  filename = expand_string(ob->file);
-  if (filename == NULL)
-    {
-    addr->message = string_sprintf("failed to expand \"%s\": %s",
-      ob->file, expand_string_message);
-    if (search_find_defer) return DEFER;
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
-
-  else if (mac_islookup(ob->search_type, lookup_absfile) &&
-           filename[0] != '/')
-    {
-    log_write(0, LOG_MAIN|LOG_PANIC, "%s director requires absolute file name "
-      "for %s: %s generated from expanding %s", dblock->name,
-      lookup_list[ob->search_type].name, filename, ob->file);
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
-
-  DEBUG(2)
-    {
-    char *pmstar = "";
-    char *pmat = "";
-    int pm = ob->partial_match;
-    if (pm > 2000)
-      {
-      pmstar = "*@";
-      pm -= 2048;
-      }
-    if (pm > 1000)
-      {
-      pmstar = "*";
-      pm -= 1024;
-      }
-    debug_printf("%s director: ", dblock->name);
-    if (pm >= 0) debug_printf("partial%d-", pm);
-    debug_printf("%s%s%s key=%s\n  file=\"%s\"\n",
-      lookup_list[ob->search_type].name, pmstar, pmat, query, filename);
-    }
-  }
-
-/* For query-style lookups, expand the query and set the filename NULL.
-For historical reasons, there may be just one query or a list of queries, but
-not both. Initialization ensures that one and only one option is set. */
-
-else
-  {
-  filename = NULL;
-  query = expand_string((ob->query == NULL)? ob->queries : ob->query);
-  if (query == NULL)
-    {
-    addr->message = string_sprintf("failed to expand \"%s\": %s",
-      (ob->query == NULL)? ob->queries : ob->query, expand_string_message);
-    if (search_find_defer) return DEFER;
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
-  DEBUG(2) debug_printf("%s director: %s query=%s\n", dblock->name,
-    lookup_list[ob->search_type].name, query);
-  }
-
-/* Open the file (or whatever) for searching, according to the search type
-that is set. The search functions return a handle identifying the search. For
-files this is a FILE * or a DBM *; for other things it is < 0. If the
-optional flag is set, failure to open is not an error; we just fail to
-direct. */
-
-handle = search_open(filename, ob->search_type, ob->modemask, ob->owners,
-  ob->owngroups, &error);
-
-if (handle == NULL)
-  {
-  if (ob->optional && errno == ENOENT)
-    {
-    DEBUG(2) debug_printf("%s director skipped: file failed to open and "
-      "optional flag set\n", dblock->name);
-    return FAIL;
-    }
-  addr->basic_errno = ERRNO_BADALIAS;
-  addr->message = error;
-
-  if (errno == ERRNO_BADMODE)
-    error = string_sprintf("%s (check the modemask option)", error);
-
-  log_write(0, LOG_MAIN|LOG_PANIC, "%s director: %s", dblock->name, error);
-  return ERROR;
-  }
-
-/* Now search the file (or whatever) for the entry we are interested in.
-The text is returned in dynamic store. If ob->queries is set, we have
-a list of queries to try. */
-
-if (ob->queries != NULL)
-  {
-  char *onequery;
-  int sep = 0;
-  while ((onequery = string_nextinlist(&query, &sep, big_buffer,
-          BIG_BUFFER_SIZE)) != NULL)
-    {
-    aliastext = search_find(handle, filename, onequery, ob->partial_match,
-      NULL, &error);
-    if (aliastext != NULL || search_find_defer) break;
-    }
-  }
-else
-  aliastext = search_find(handle, filename, query, ob->partial_match, NULL,
-    &error);
-
-/* The lookup did not succeed. Defer if it deferred unless optional is set;
-otherwise this director fails to handle the address. */
-
-if (aliastext == NULL)
-  {
-  if (search_find_defer && !ob->optional)
-    {
-    addr->message = string_sprintf("search_find deferred: %s", error);
-    return DEFER;
-    }
-
-  DEBUG(2) debug_printf("%s director failed for %s: %s\n", dblock->name,
-    addr->local_part, error);
-  return FAIL;
-  }
-
-/* If the expand option is set, pass the text through the string expander. */
-
-if (ob->expand)
-  {
-  char *newtext = expand_string(aliastext);
-  if (newtext == NULL)
-    {
-    addr->message = string_sprintf("failed to expand \"%s\" "
-      "(generated from local part %s): %s", aliastext,
-      addr->local_part, expand_string_message);
-    if (search_find_defer) return DEFER;
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
-  aliastext = newtext;
-  }
+yield = df_lookup(dblock, addr, USS &aliastext, (uschar **)&filename);
+if (yield != OK) return yield;
 
 /* Sort out the fixed or dynamic uid/gid */
 
