@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the aliasfile director */
@@ -12,19 +12,10 @@ common director fields for all directors that use shared functions. */
 
 typedef struct {
   COMMON_DIRECTOR_FIELDS;
-  char *file;
-  char *queries;
-  char *query;
   char *syntax_errors_to;
   char *syntax_errors_text;
-  int   modemask;
-  int   partial_match;
-  int   search_type;
-  int  *owners;
-  int  *owngroups;
-  BOOL  include_domain;
-  BOOL  expand;
-  BOOL  optional;
+  BOOL  forbid_include;
+  BOOL  forbid_special;
   BOOL  qualify_preserve_domain;
   BOOL  freeze_missing_include;
   BOOL  rewrite;
