@@ -16,8 +16,8 @@
 
 /* This function sets up a set of newly generated child addresses and puts them
 on the new address chain. Copy in the uid, gid and permission flags for use by
-pipes and files, set the parent, and or its ignore_error flag. Also record the
-setting for any starting director.
+pipes and files, set the parent, and "or" its af_ignore_error flag. Also record
+the setting for any starting director.
 
 If the generated address is the same as one of its ancestors, and the
 check_ancestor flag is set, do not use this generated address, but replace it
@@ -33,7 +33,7 @@ Arguments:
   addr_new                new address chain
   addr                    original address
   generated               list of generated addresses
-  errors_to               return path
+  errors_to               return path from director configuration
   extra_headers           headers to add
   remove_headers          headers to remove
   ugidptr                 points to uid/gid data for files, pipes, autoreplies
@@ -59,7 +59,7 @@ while (generated != NULL)
 
   generated = next->next;
   next->parent = addr;
-  next->ignore_error |= addr->ignore_error;
+  orflag(next, addr, af_ignore_error);
   next->start_director = dblock->new;
   addr->child_count++;
 
@@ -72,6 +72,8 @@ while (generated != NULL)
     next->onetime_parent = parent->orig;
     }
 
+  if (ob->hide_child_in_errmsg) setflag(next, af_hide_child);
+
   if (ob->check_ancestor)
     {
     for (parent = addr; parent != NULL; parent = parent->parent)
@@ -99,7 +101,7 @@ while (generated != NULL)
   because they don't go through the directing process again. Then set up uid,
   gid, home and current directories. */
 
-  if (next->pfr)
+  if (testflag(next, af_pfr))
     {
     next->director = dblock;
 
@@ -113,12 +115,12 @@ while (generated != NULL)
       if (!ugidptr->uid_set)
         {
         next->uid = pw->pw_uid;
-        next->uid_set = TRUE;
+        setflag(next, af_uid_set);
         }
       if (!ugidptr->gid_set)
         {
         next->gid = pw->pw_gid;
-        next->gid_set = TRUE;
+        setflag(next, af_gid_set);
         }
       }
 
@@ -148,9 +150,9 @@ while (generated != NULL)
 
     /* Permission options */
 
-    next->allow_pipe = !ob->forbid_pipe;
-    next->allow_file = !ob->forbid_file;
-    next->allow_reply = !ob->forbid_reply;
+    if (!ob->forbid_pipe) setflag(next, af_allow_pipe);
+    if (!ob->forbid_file) setflag(next, af_allow_file);
+    if (!ob->forbid_reply) setflag(next, af_allow_reply);
 
     /* Aliasfile can produce only pipes or files, but forwardfile can in
     addition produce autoreplies. If the transport setting fails, the error
@@ -195,16 +197,16 @@ while (generated != NULL)
     debug_printf("%s director generated %s\n  %serrors_to=%s transport=%s\n",
       dblock->name,
       next->orig,
-      next->pfr? "pipe, file, or autoreply\n  " : "",
-      errors_to,
+      testflag(next, af_pfr)? "pipe, file, or autoreply\n  " : "",
+      next->errors_address,
       (next->transport == NULL)? "NULL" : next->transport->name);
 
-    if (next->uid_set)
+    if (testflag(next, af_uid_set))
       debug_printf("  uid=%ld ", (long int)(next->uid));
     else
       debug_printf("  uid=unset ");
 
-    if (next->gid_set)
+    if (testflag(next, af_gid_set))
       debug_printf("gid=%ld ", (long int)(next->gid));
     else
       debug_printf("gid=unset ");
