@@ -33,6 +33,8 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, check_group)) },
   { "check_local_user",   opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, check_local_user)) },
+  { "data",               opt_stringptr,
+      (void *)offsetof(forwardfile_director_options_block, data) },
   { "directory2_transport",opt_transportptr,
       (void *)(offsetof(forwardfile_director_options_block, directory2_transport)) },
   { "directory_transport",opt_transportptr,
@@ -67,6 +69,8 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, forbid_reply)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, freeze_missing_include)) },
+  { "hide_child_in_errmsg", opt_bool,
+      (void *)offsetof(forwardfile_director_options_block, hide_child_in_errmsg) },
   { "ignore_eacces",      opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, ignore_eacces)) },
   { "ignore_enotdir",     opt_bool,
@@ -119,21 +123,30 @@ forwardfile_director_options_block forwardfile_director_option_defaults = {
   NULL,     /* expand_pipe_transport */
   NULL,     /* reply_transport */
   NULL,     /* expand_reply_transport */
-  FALSE,    /* check_ancestor */
-  TRUE,     /* check_local_user */
-  FALSE,    /* forbid_file */
-  FALSE,    /* forbid_pipe */
-  FALSE,    /* forbid_reply */
-  FALSE,    /* one_time */
-  /********************************************/
-  NULL,     /* file_dir */
-  NULL,     /* pw_dir */
   NULL,     /* file */
-  NULL,     /* syntax_errors_to */
-  NULL,     /* syntax_errors_text */
+  NULL,     /* queries */
+  NULL,     /* query */
   022,      /* modemask */
   NULL,     /* owners */
   NULL,     /* owngroups */
+  -1,       /* partial_match */
+  -1,       /* search_type */
+  FALSE,    /* check_ancestor */
+  TRUE,     /* check_local_user */
+  FALSE,    /* expand */
+  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_pipe */
+  FALSE,    /* forbid_reply */
+  FALSE,    /* include_domain */
+  FALSE,    /* hide_child_in_errmsg */
+  FALSE,    /* one_time */
+  FALSE,    /* optional */
+  /********************************************/
+  NULL,     /* data */
+  NULL,     /* file_dir */
+  NULL,     /* pw_dir */
+  NULL,     /* syntax_errors_to */
+  NULL,     /* syntax_errors_text */
   FALSE,    /* check_group */
   FALSE,    /* filter */
   FALSE,    /* allow_system_actions */
@@ -166,11 +179,19 @@ void forwardfile_director_init(director_instance *dblock)
 forwardfile_director_options_block *ob =
   (forwardfile_director_options_block *)(dblock->options_block);
 
-/* A file name is mandatory */
+/* The one_continue flag is turned off for forwardfile */
 
-if (ob->file == NULL)
+dblock->one_continue = FALSE;
+
+/* Either file or data must be set, but not both */
+
+if (ob->file == NULL && ob->data == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-    "no file name specified", dblock->name);
+    "one of \"file\" or \"data\" must be specified", dblock->name);
+
+if (ob->file != NULL && ob->data != NULL)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
+    "only one of \"file\" or \"data\" must be specified", dblock->name);
 
 /* A directory setting is optional, but if it exists it must be absolute,
 though we can't check for certain until it has been expanded. */
@@ -187,7 +208,7 @@ only if local user checking is set. */
 
 if (!ob->check_local_user)
   {
-  if (ob->file[0] != '/' && ob->file[0] != '$' &&
+  if (ob->file != NULL && ob->file[0] != '/' && ob->file[0] != '$' &&
       ob->file_dir == NULL)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
       "absolute file path required when check_local_user and directory are not set",
@@ -254,7 +275,7 @@ char *save_qualify_domain_recipient = qualify_domain_recipient;
 char *directory = NULL;
 char *errors_to;
 char *filename;
-char *filebuf;
+char *filebuf = NULL;
 char *error;
 char *s, *tag;
 char *remove_headers;
@@ -264,12 +285,13 @@ struct stat statbuf;
 struct passwd *pw = NULL;
 error_block *eblock = NULL;
 ugid_block ugid;
-FILE *fwd;
+FILE *fwd = NULL;
 int   yield = OK;
 uid_t saved_euid = 0;
 gid_t saved_egid = 0;
-int   extracted = 0;
 int   rc;
+int   filter_action = SPECIAL_NONE;
+BOOL  usercheckfailed = FALSE;
 BOOL  uid_ok = FALSE;
 BOOL  gid_ok = !ob->check_group;
 BOOL  restore_uid = FALSE;
@@ -282,31 +304,42 @@ addr_local = addr_local;     /* Keep picky compilers happy */
 addr_remote = addr_remote;
 
 /* If the check_local_user option is set, check that the local_part is
-the login of a local user, and fail if not. Note: the third argument to
-direct_finduser() must be NULL here, to prevent a numeric string being
-taken as a numeric uid. If the user is found, set directory to the home
-directory, and the home expansion variable as well, so that it can be
-used while expanding match_directory and ob->file_dir. */
+the login of a local user. Note: the third argument to direct_finduser() must
+be NULL here, to prevent a numeric string being taken as a numeric uid. If the
+user is found, set directory to the home directory, and the home expansion
+variable as well, so that it can be used while expanding match_directory and
+ob->file_dir. The resetting of deliver_home happens at the outer level. If the
+user is not found, we don't decline just yet, because the condition test might
+fail, and it may return FAIL_CONDITION, which should take precedence. */
 
 if (ob->check_local_user)
   {
-  if (!direct_finduser(addr->local_part, &pw, NULL))
-    {
-    DEBUG(2) debug_printf("%s director failed for %s (not a user)\n",
-      dblock->name, addr->local_part);
-    return FAIL;
-    }
+  usercheckfailed = !direct_finduser(addr->local_part, &pw, NULL);
+  if (!usercheckfailed) deliver_home = directory = pw->pw_dir;
+  }
 
-  /* Set the home directory, and if there is a setting of match_directory,
-  check for a match. The resetting of deliver_home happens at the outer
-  level. */
+/* Perform file existence and sender verification checks now that we
+potentially have $home available. */
 
-  deliver_home = directory = pw->pw_dir;
-  if (ob->pw_dir != NULL)
-    {
-    rc = direct_match_directory(addr, pw->pw_dir, ob->pw_dir, dblock->name);
-    if (rc != OK) return rc;
-    }
+yield = direct_check_fsc(dblock, addr);
+if (yield != OK) return yield;
+
+/* The condition check succeeded; we can now return DECLINE if the
+user check failed. */
+
+if (usercheckfailed)
+  {
+  DEBUG(2) debug_printf("%s director declined for %s (not a user)\n",
+    dblock->name, addr->local_part);
+  return DECLINE;
+  }
+
+/* If there is a user and a setting of match_directory, check for a match. */
+
+if (pw != NULL && ob->pw_dir != NULL)
+  {
+  rc = direct_match_directory(addr, pw->pw_dir, ob->pw_dir, dblock->name);
+  if (rc != OK) return rc;
   }
 
 /* If the file_directory option is set expand the string, and set it as the
@@ -327,61 +360,89 @@ if (ob->file_dir != NULL)
   deliver_home = directory;
   }
 
-/* Perform file existence and sender verification checks now that we
-have $home available. */
+/* The text of the forward/filter file is either a complete file, or is
+obtained by expanding the "data" option. This is an interim state, working
+towards amalgamating forwardfile and aliasfile. */
 
-yield = direct_check_fsc(dblock, addr);
-if (yield != OK) return yield;
+/* In the case of a file, sort out the file name. */
 
-/* Get the required file name and expand it. If the expansion fails, log the
-incident and indicate an internal error. */
-
-filename = expand_string(ob->file);
-
-if (filename == NULL)
+if (ob->file != NULL)
   {
-  addr->message = string_sprintf("failed to expand \"%s\": %s",
-    ob->file, expand_string_message);
-  if (search_find_defer) return DEFER;
-  addr->special_action = SPECIAL_FREEZE;
-  return ERROR;
+  /* Get the required file name and expand it. If the expansion fails, log the
+  incident and indicate an internal error. */
+
+  filename = expand_string(ob->file);
+
+  if (filename == NULL)
+    {
+    addr->message = string_sprintf("failed to expand \"%s\": %s",
+      ob->file, expand_string_message);
+    if (search_find_defer) return DEFER;
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
+
+  DEBUG(2) debug_printf("%s director: file = %s\n", dblock->name, filename);
+
+  /* If a directory is set and the file name is not absolute, construct the
+  complete file name. Otherwise set a flag to prevent an attempt at statting the
+  directory below. */
+
+  if (directory != NULL && filename[0] != '/')
+    filename = string_sprintf("%s/%s", directory, filename);
+  else stat_directory = FALSE;
+
+  /* Check that the file name is absolute. Simple checks are done in the
+  init function, but expansions mean that we have to do a final check here. */
+
+  if (filename[0] != '/')
+    {
+    log_write(0, LOG_MAIN, "%s is not an absolute path for the %s director",
+      filename, dblock->name);
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
-DEBUG(2) debug_printf("%s director: file = %s\n", dblock->name, filename);
+/* If not using a file, the text is obtained by expanding data. We do that now,
+before changing uid, so that lookups run as the exim user. If the expansion is
+forced to fail, we give up (cf non-existent file). The variable "filename" is
+set appopriately for use in messages about syntax errors. */
 
-/* If a directory is set and the file name is not absolute, construct the
-complete file name. Otherwise set a flag to prevent an attempt at statting the
-directory below. */
-
-if (directory != NULL && filename[0] != '/')
-  filename = string_sprintf("%s/%s", directory, filename);
-else stat_directory = FALSE;
-
-/* Check that the file name is absolute. Simple checks are done in the
-init function, but expansions mean that we have to do a final check here. */
-
-if (filename[0] != '/')
+else
   {
-  log_write(0, LOG_MAIN, "%s is not an absolute path for the %s director",
-    filename, dblock->name);
-  addr->special_action = SPECIAL_FREEZE;
-  return ERROR;
+  filename = "inline forwarding data";
+  filebuf = expand_string(ob->data);
+  if (filebuf == NULL)
+    {
+    if (expand_string_forcedfail)
+      {
+      DEBUG(2) debug_printf("forced failure for expansion of \"%s\"\n",ob->data);
+      return DECLINE;
+      }
+    addr->message = string_sprintf("failed to expand \"%s\": %s",
+      ob->data, expand_string_message);
+    if (search_find_defer) return DEFER;
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
 /* Sort out the fixed or dynamic uid/gid. This uid is used (a) for reading the
-file if seteuid is on and (b) for running the transports for generated file and
-pipe addresses. It is not (necessarily) the same as the uids that may own the
-file. Exim panics if an expanded string is not a number and can't be found in
-the password file. Other errors set the freezing bit. */
+file (and interpreting a filter) if seteuid is on and (b) for running the
+transports for generated file and pipe addresses. It is not (necessarily) the
+same as the uids that may own the file. Exim panics if an expanded string is
+not a number and can't be found in the password file. Other errors set the
+freezing bit. */
 
 if (!direct_get_ugid(dblock, "director", addr, &ugid)) return ERROR;
 
 /* If the seteuid option is on and a uid is available (either explicit or as a
 result of check_local_user), use seteuid() to become the local user. This is
 necessary in order to read .forward files that are in NFS-mounted home
-directories. The macros expand to -1 on systems without setuid() configured,
-but in those cases this code is never obeyed, as the option is locked out in
-the init check.
+directories. We also need to do it for running filter files. The macros expand
+to -1 on systems without setuid() configured, but in those cases this code is
+never obeyed, as the option is locked out in the init check.
 
 Even if the seteuid option is not on, we need to do this when the security
 level is 1 or 3, because in those cases this code is running seteuid to exim.
@@ -415,11 +476,12 @@ if ((ugid.uid_set || ob->check_local_user) &&
   restore_uid = TRUE;
   }
 
-/* You might think we could just test for the existence of the required file by
-attempting to open it, but life isn't that simple. In many large systems,
-.forward files in users' home directories are used, with the home directories
-possibly NFS-mounted from some remote place. It doesn't seem possible to detect
-the state of "NFS mount inaccessible" just by trying to open a file.
+/* Handle the testing and reading of a complete file. You might think we could
+just test for the existence of the required file by attempting to open it, but
+life isn't that simple. In many large systems, .forward files in users' home
+directories are used, with the home directories possibly NFS-mounted from some
+remote place. It doesn't seem possible to detect the state of "NFS mount
+inaccessible" just by trying to open a file.
 
 The common case is specified with a relative path name (relative to the home
 directory or to a specified directory), and in that case we try to do a bit
@@ -440,193 +502,205 @@ with EACCES.
 If an NFS host is taken offline, it is possible for the stat() to get stuck
 until it comes back. To guard against this, stick a timer round it. */
 
-if (directory != NULL && stat_directory)
+if (ob->file != NULL)
   {
-  int rc, saved_errno;
-  sigalrm_seen = FALSE;
-  os_non_restarting_signal(SIGALRM, sigalrm_handler);
-  alarm(30);
-  sprintf(big_buffer, "%.*s/.", big_buffer_size, directory);
-  rc = stat(big_buffer, &statbuf);
-  if (rc == 0)
+  if (directory != NULL && stat_directory)
     {
-    DEBUG(2) debug_printf("successful stat of %s\n", big_buffer);
-    }
-  else if (errno == EACCES && !sigalrm_seen)
-    {
-    rc = stat(directory, &statbuf);
+    int rc, saved_errno;
+    sigalrm_seen = FALSE;
+    os_non_restarting_signal(SIGALRM, sigalrm_handler);
+    alarm(30);
+    sprintf(big_buffer, "%.*s/.", big_buffer_size, directory);
+    rc = stat(big_buffer, &statbuf);
     if (rc == 0)
       {
-      DEBUG(2) debug_printf("successful stat of %s\n", directory);
+      DEBUG(2) debug_printf("successful stat of %s\n", big_buffer);
       }
-    }
-  saved_errno = errno;
-  alarm(0);
-  signal(SIGALRM, SIG_DFL);
+    else if (errno == EACCES && !sigalrm_seen)
+      {
+      rc = stat(directory, &statbuf);
+      if (rc == 0)
+        {
+        DEBUG(2) debug_printf("successful stat of %s\n", directory);
+        }
+      }
+    saved_errno = errno;
+    alarm(0);
+    signal(SIGALRM, SIG_DFL);
 
-  if (sigalrm_seen || rc != 0)
+    if (sigalrm_seen || rc != 0)
+      {
+      addr->message = string_sprintf("failed to stat %s (%s)", directory,
+        sigalrm_seen? "timeout" : strerror(saved_errno));
+      yield = DEFER;
+      goto RESTORE_UID;             /* skip forward */
+      }
+
+    }
+
+  /* Now try to open the file for reading. If this fails with a non-existence
+  error, we have no option but to believe that the file does not exist, so the
+  director gives up on this address. Some other cases are more dubious, and may
+  indicate configuration errors. For this reason, their handling is controlled by
+  options.
+
+  ENOTDIR means that something along the line is not a directory: there are
+  installations that set home directories to be /dev/null for non-login accounts
+  but in normal circumstances this indicates some kind of configuration error.
+
+  EACCES means there's a permissions failure. Some users turn off read
+  permission on a .forward file to suspend forwarding, but this is probably an
+  error in any kind of mailing list processing. */
+
+  fwd = fopen(filename, "r");
+  if (fwd == NULL)
+    {
+    switch(errno)
+      {
+      case ENOENT:          /* File does not exist */
+      DEBUG(2) debug_printf("%s not found\n", filename);
+      yield = DECLINE;
+      break;
+
+      case ENOTDIR:         /* Something on the path isn't a directory */
+      if (ob->ignore_enotdir)
+        {
+        DEBUG(2) debug_printf("non-directory on path %s: file assumed not to "
+          "exist\n", filename);
+        yield = DECLINE;
+        break;
+        }
+      goto DEFAULT_ERROR;
+
+      case EACCES:           /* Permission denied */
+      if (ob->ignore_eacces)
+        {
+        DEBUG(2) debug_printf("permission denied for %s: file assumed not to "
+          "exist\n", filename);
+        yield = DECLINE;
+        break;
+        }
+      /* Else fall through */
+
+      DEFAULT_ERROR:
+      default:
+      addr->message = string_open_failed(errno, "%s", filename);
+      yield = ERROR;
+      break;
+      }
+
+    goto RESTORE_UID;               /* skip forward */
+    }
+
+
+  /* Now check up on the mode of the file. It is tempting to do this stat before
+  opening the file, and use it as an existence check. However, doing that opens a
+  small security loophole in that the status could be changed before the file is
+  opened. Can't quite see what problems this might lead to, but you can't be too
+  careful where security is concerned. Fstat() on an open file can normally be
+  expected to succeed, but there are some NFS states where it does not. */
+
+  if (fstat(fileno(fwd), &statbuf) != 0)
     {
-    addr->message = string_sprintf("failed to stat %s (%s)", directory,
-      sigalrm_seen? "timeout" : strerror(saved_errno));
     yield = DEFER;
-    goto RESTORE_UID;             /* skip forward */
+    goto CLOSE_RESTORE_UID;         /* skip forward */
     }
 
-  }
-
-/* Now try to open the file for reading. If this fails with a non-existence
-error, we have no option but to believe that the file does not exist, so the
-director gives up on this address. Some other cases are more dubious, and may
-indicate configuration errors. For this reason, their handling is controlled by
-options.
-
-ENOTDIR means that something along the line is not a directory: there are
-installations that set home directories to be /dev/null for non-login accounts
-but in normal circumstances this indicates some kind of configuration error.
-
-EACCES means there's a permissions failure. Some users turn off read
-permission on a .forward file to suspend forwarding, but this is probably an
-error in any kind of mailing list processing. */
-
-fwd = fopen(filename, "r");
-if (fwd == NULL)
-  {
-  switch(errno)
+  if ((statbuf.st_mode & S_IFMT) != S_IFREG)
     {
-    case ENOENT:          /* File does not exist */
-    DEBUG(2) debug_printf("%s not found\n", filename);
-    yield = FAIL;
-    break;
-
-    case ENOTDIR:         /* Something on the path isn't a directory */
-    if (ob->ignore_enotdir)
-      {
-      DEBUG(2) debug_printf("non-directory on path %s: file assumed not to "
-        "exist\n", filename);
-      yield = FAIL;
-      break;
-      }
-    goto DEFAULT_ERROR;
-
-    case EACCES:           /* Permission denied */
-    if (ob->ignore_eacces)
-      {
-      DEBUG(2) debug_printf("permission denied for %s: file assumed not to "
-        "exist\n", filename);
-      yield = FAIL;
-      break;
-      }
-    /* Else fall through */
-
-    DEFAULT_ERROR:
-    default:
-    addr->message = string_open_failed(errno, "%s", filename);
-    yield = ERROR;
-    break;
+    addr->basic_errno = ERRNO_BADFORWARD;
+    addr->message = string_sprintf("%s is not a regular file", filename);
+    yield = DEFER;
+    goto CLOSE_RESTORE_UID;         /* skip forward */
     }
 
-  goto RESTORE_UID;               /* skip forward */
-  }
+  if ((statbuf.st_mode & ob->modemask) != 0)
+    {
+    addr->basic_errno = ERRNO_BADFORWARD;
+    addr->message =
+      string_sprintf("bad mode (0%o) for %s: 0%o bit(s) unexpected",
+        statbuf.st_mode, filename, statbuf.st_mode & ob->modemask);
+    yield = DEFER;
+    goto CLOSE_RESTORE_UID;         /* skip forward */
+    }
 
+  /* Check the owner and group, including the current user if check_local_user
+  was set. If there is nothing to check, let it go. */
 
-/* Now check up on the mode of the file. It is tempting to do this stat before
-opening the file, and use it as an existence check. However, doing that opens a
-small security loophole in that the status could be changed before the file is
-opened. Can't quite see what problems this might lead to, but you can't be too
-careful where security is concerned. Fstat() on an open file can normally be
-expected to succeed, but there are some NFS states where it does not. */
-
-if (fstat(fileno(fwd), &statbuf) != 0)
-  {
-  yield = DEFER;
-  goto CLOSE_RESTORE_UID;         /* skip forward */
-  }
-
-if ((statbuf.st_mode & S_IFMT) != S_IFREG)
-  {
-  addr->basic_errno = ERRNO_BADFORWARD;
-  addr->message = string_sprintf("%s is not a regular file", filename);
-  yield = DEFER;
-  goto CLOSE_RESTORE_UID;         /* skip forward */
-  }
-
-if ((statbuf.st_mode & ob->modemask) != 0)
-  {
-  addr->basic_errno = ERRNO_BADFORWARD;
-  addr->message =
-    string_sprintf("bad mode (0%o) for %s: 0%o bit(s) unexpected",
-      statbuf.st_mode, filename, statbuf.st_mode & ob->modemask);
-  yield = DEFER;
-  goto CLOSE_RESTORE_UID;         /* skip forward */
-  }
-
-/* Check the owner and group, including the current user if check_local_user
-was set. If there is nothing to check, let it go. */
-
-if (ob->check_local_user)
-  { if (statbuf.st_uid == pw->pw_uid) uid_ok = TRUE; }
-else if (ob->owners == NULL) uid_ok = TRUE;
-
-if (!uid_ok && ob->owners != NULL)
-  {
-  int i;
-  for (i = 1; i <= (int)(ob->owners[0]); i++)
-    if (ob->owners[i] == statbuf.st_uid) { uid_ok = TRUE; break; }
-  }
-
-/* If gid is to be checked, the local user's gid is always permissible. */
-
-if (!gid_ok)
-  {
   if (ob->check_local_user)
-    { if (statbuf.st_gid == pw->pw_gid) gid_ok = TRUE; }
-  if (!gid_ok && ob->owngroups != NULL)
+    { if (statbuf.st_uid == pw->pw_uid) uid_ok = TRUE; }
+  else if (ob->owners == NULL || (int)(ob->owners[0]) == 0) uid_ok = TRUE;
+
+  if (!uid_ok && ob->owners != NULL)
     {
     int i;
-    for (i = 1; i <= (int)(ob->owngroups[0]); i++)
-      if (ob->owngroups[i] == statbuf.st_gid) { gid_ok = TRUE; break; }
+    for (i = 1; i <= (int)(ob->owners[0]); i++)
+      if (ob->owners[i] == statbuf.st_uid) { uid_ok = TRUE; break; }
+    }
+
+  /* If gid is to be checked, the local user's gid is always permissible. */
+
+  if (!gid_ok)
+    {
+    if (ob->check_local_user)
+      { if (statbuf.st_gid == pw->pw_gid) gid_ok = TRUE; }
+    if (!gid_ok && ob->owngroups != NULL && (int)(ob->owngroups[0]) != 0)
+      {
+      int i;
+      for (i = 1; i <= (int)(ob->owngroups[0]); i++)
+        if (ob->owngroups[i] == statbuf.st_gid) { gid_ok = TRUE; break; }
+      }
+    }
+
+  if (!uid_ok || !gid_ok)
+    {
+    addr->basic_errno = ERRNO_BADFORWARD;
+    addr->message =
+      string_sprintf("bad %s for %s", uid_ok? "group" : "owner", filename);
+    yield = DEFER;
+    goto CLOSE_RESTORE_UID;         /* skip forward */
+    }
+
+  /* Processing a .forward file could use a fair bit of store, especially if it
+  is a filter file. Therefore it is a good idea to recover the store used
+  afterwards. In order to do this, we switch to the temporary stacking pool, and
+  recover that. Any new addresses that are created are put in the main pool, so
+  as to remain around afterwards. */
+
+  store_pool = POOL_TEMP;
+  reset_point = store_get(0);
+
+  /* Read the .forward file. We read the file in one go in order to minimize
+  the time we have it open. */
+
+  filebuf = store_get(statbuf.st_size + 1);
+  if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
+    {
+    store_pool = POOL_MAIN;
+    addr->basic_errno = errno;
+    addr->message = "error while reading forward file";
+    yield = DEFER;
+    goto CLOSE_RESTORE_UID;         /* skip forward */
+    }
+  filebuf[statbuf.st_size] = 0;
+
+  /* Don't pass statbuf.st_size directly to debug_printf. On some systems it is a
+  long, which may not be the same as an int. */
+
+  DEBUG(2)
+    {
+    int size = statbuf.st_size;
+    debug_printf("%d bytes read from %s\n", size, filename);
     }
   }
 
-if (!uid_ok || !gid_ok)
+/* If we already have the text from a database lookup, all we need to do here
+is to change to the temporary storage pool. */
+
+else
   {
-  addr->basic_errno = ERRNO_BADFORWARD;
-  addr->message =
-    string_sprintf("bad %s for %s", uid_ok? "group" : "owner", filename);
-  yield = DEFER;
-  goto CLOSE_RESTORE_UID;         /* skip forward */
-  }
-
-/* Processing a .forward file could use a fair bit of store, especially if it
-is a filter file. Therefore it is a good idea to recover the store used
-afterwards. In order to do this, we switch to the temporary stacking pool, and
-recover that. Any new addresses that are created are put in the main pool, so
-as to remain around afterwards. */
-
-store_pool = POOL_TEMP;
-reset_point = store_get(0);
-
-/* Read the .forward file and generate new addresses for each entry therein.
-We read the file in one go in order to minimize the time we have it open. */
-
-filebuf = store_get(statbuf.st_size + 1);
-if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
-  {
-  store_pool = POOL_MAIN;
-  addr->basic_errno = errno;
-  addr->message = "error while reading forward file";
-  yield = DEFER;
-  goto CLOSE_RESTORE_UID;         /* skip forward */
-  }
-filebuf[statbuf.st_size] = 0;
-
-/* Don't pass statbuf.st_size directly to debug_printf. On some systems it is a
-long, which may not be the same as an int. */
-
-DEBUG(2)
-  {
-  int size = statbuf.st_size;
-  debug_printf("%d bytes read from %s\n", size, filename);
+  store_pool = POOL_TEMP;
+  reset_point = store_get(0);
   }
 
 /* If the filter option is set, the file is to be interpreted as a filter
@@ -665,7 +739,7 @@ explicitly configured. */
 
 if (filter_header && ob->filter)
   {
-  int action = SPECIAL_NONE;
+  BOOL extracted;
   BOOL oldforbid_exists = expand_forbid_exists;
   BOOL oldforbid_lookup = expand_forbid_lookup;
   BOOL oldforbid_perl = expand_forbid_perl;
@@ -678,12 +752,12 @@ if (filter_header && ob->filter)
   expand_forbid_perl = ob->forbid_filter_perl;
 
   extracted = filter_interpret(filebuf, &generated, &delivered,
-    (ob->allow_system_actions)? &action : NULL,
+    (ob->allow_system_actions)? &filter_action : NULL,
     &error,
     (verify || address_test_mode)? 1 : /* skip logging if verifying/testing */
       ob->forbid_filter_logwrite? 2 :  /* lock out if forbidden */
       restore_uid? 0 : 3,              /* else permit only if uid set */
-    ob->rewrite)? 0 : -1;
+    ob->rewrite);
 
   expand_forbid_exists = oldforbid_exists;
   expand_forbid_lookup = oldforbid_lookup;
@@ -691,23 +765,6 @@ if (filter_header && ob->filter)
 
   store_pool = POOL_MAIN;              /* Reset for error strings */
 
-  /* Handle special action (will be SPECIAL_NONE unless permitted otherwise).
-  Like the system filter, freeze does not happen after a manual thaw. */
-
-  if (action == SPECIAL_FREEZE && !deliver_manual_thaw)
-    {
-    addr->message = (filter_fmsg == NULL)?
-      "frozen by filter" : string_copy(filter_fmsg);
-    addr->special_action = SPECIAL_FREEZE;
-    yield = ERROR;
-    }
-  else if (action == SPECIAL_FAIL)
-    {
-    addr->message = (filter_fmsg == NULL)?
-      "rejected by filter" : string_copy(filter_fmsg);
-    yield = FORCEFAIL;
-    }
-
   /* If extraction from a filter file failed, it is a "probable user error", to
   use a good old IBM term. By default, just defer delivery and let the user
   clean things up in due course. However, users are very lax about this. The
@@ -715,7 +772,7 @@ if (filter_header && ob->filter)
   forward files. It can be used to carry on delivering, while sending an
   error message if syntax_errors_to is set. */
 
-  if (extracted != 0)
+  if (!extracted)
     {
     if (ob->skip_syntax_errors)
       {
@@ -740,6 +797,7 @@ to get the values. The yield is 0=>OK, -1=>error, +1=> failed to open an
 
 else
   {
+  int extracted;
   DEBUG(2) debug_printf("file is not a filter file\n");
 
   /* There is a common function for use by forwarding and aliasing
@@ -800,7 +858,7 @@ goto is used above to skip to this code when errors are detected. */
 
 CLOSE_RESTORE_UID:
 
-fclose(fwd);
+if (fwd != NULL) fclose(fwd);
 
 RESTORE_UID:
 
@@ -851,43 +909,73 @@ the uid or close the file. */
 
 if (yield != OK) return yield;
 
-/* If a forward file generates no addresses, it is not an error. The director
-just fails. Compare aliasfile, which is different. However, if it is a filter
-file that has set "delivered", the director has succeeded. Log the fact,
-as otherwise messages may appear to disappear without trace. */
+/* Handle the generated addresses; set up the errors address and the additional
+and removeable headers, and then add the addresses to the addr_new chain, using
+the common function. */
 
-if (generated == NULL)
+if (generated != NULL)
   {
-  if (is_filter && delivered)
+  rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
+  if (rc != OK) return rc;
+
+  rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
+    &remove_headers);
+  if (rc != OK) return rc;
+
+  df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
+    remove_headers, &ugid, pw, directory);
+  }
+
+/* Otherwise, there are no generated addresses. If this was a .forward file,
+just decline. */
+
+else if (!is_filter) return DECLINE;
+
+/* It was a filter file. If "delivered" is not set, and no special action was
+requested, just decline. This also happens if the action is "freeze" but there
+has been a manual thaw. Otherwise, if not a special action, a filter is just
+discarding deliveries. */
+
+else
+  {
+  if (filter_action == SPECIAL_FREEZE && deliver_manual_thaw)
+    filter_action = SPECIAL_NONE;
+  if (!delivered && filter_action == SPECIAL_NONE) return DECLINE;
+
+  /* Either "delivered" or a special action is required */
+
+  if (filter_action == SPECIAL_NONE)
     {
     if (!verify && !address_test_mode)
       log_write(0, LOG_MAIN, "=> discarded <%s> D=%s", addr->orig,
         dblock->name);
     return OK;
     }
-  else return FAIL;
   }
 
-/* Set up the errors address for the children of this address. */
+/* If a filter returned a special action (freeze or fail - these can only occur
+if allow_special_actions is set) deal with them now. Like the system filter,
+freeze does not happen after a manual thaw. In case there were deliveries
+set up by the filter (which will be done), we set the child count high so that
+their completion does not mark the original address done. */
 
-rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
-if (rc != OK) return rc;
-
-/* Set up the additional and removeable headers for the children of this
-address. */
-
-rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
-  &remove_headers);
-if (rc != OK) return rc;
-
-/* Set up the new addresses and add them to the addr_new chain. A common
-function handles this. */
-
-df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
-  remove_headers, &ugid, pw, directory);
+if (filter_action == SPECIAL_FREEZE && !deliver_manual_thaw)
+  {
+  addr->message = (filter_fmsg == NULL)?
+    "frozen by filter" : string_copy(filter_fmsg);
+  addr->special_action = SPECIAL_FREEZE;
+  addr->child_count = 9999999;
+  return ERROR;
+  }
+else if (filter_action == SPECIAL_FAIL)
+  {
+  addr->message = (filter_fmsg == NULL)?
+    "rejected by filter" : string_copy(filter_fmsg);
+  return FORCEFAIL;
+  }
 
 /* If the filter interpreter returned not "delivered", we have to arrange for
-this address to be passed on to subsequent directors. Returning FAIL would
+this address to be passed on to subsequent directors. Returning DECLINE would
 appear to be the answer, but it isn't, because successful delivery of the base
 address gets it marked "done", so deferred generated addresses never get tried
 again. We have to generate a new version of the base address, as if there were
@@ -899,9 +987,7 @@ if (is_filter && !delivered)
 
   next = deliver_make_addr(addr->orig, FALSE);
   next->parent = addr;
-  next->ignore_error |= addr->ignore_error;
-  next->pfr = addr->pfr;
-  next->forced_local = addr->forced_local;  /* Keep it local */
+  copyflag(next, addr, (af_pfr|af_forced_local|af_ignore_error));
   addr->child_count++;
   next->next = *addr_new;
   *addr_new = next;
@@ -913,7 +999,7 @@ if (is_filter && !delivered)
   DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s",
     dblock->name,
     next->orig,
-    next->pfr? "  pipe, file, or autoreply\n" : "",
+    testflag(next, af_pfr)? "  pipe, file, or autoreply\n" : "",
     (errors_to != NULL)? "  errors to " : "",
     (errors_to != NULL)? errors_to : "",
     (errors_to != NULL)? "\n" : "");
