@@ -76,40 +76,48 @@ int localuser_director_entry(
 localuser_director_options_block *ob =
   (localuser_director_options_block *)(dblock->options_block);
 struct passwd *pw;
+BOOL userexists;
+int rc;
 char *errors_to;
 char *remove_headers;
 header_line *extra_headers;
-int rc;
 
 addr_new = addr_new;          /* Keep picky compilers happy */
 addr_succeed = addr_succeed;
 
-/* The third argument to direct_finduser() must be NULL here, to prevent a
-digit string being taken as a numeric uid. */
+/* Check that the local part is a valid local user account. We have to do this
+first in order to get $home available for other checks. However, we can't
+return immediately if it isn't, because the condition check logically happens
+before the user check, and gives a different result if it fails. The third
+argument to direct_finduser() must be NULL here, to prevent a digit string
+being taken as a numeric uid. */
 
-if (!direct_finduser(addr->local_part, &pw, NULL))
+userexists = direct_finduser(addr->local_part, &pw, NULL);
+
+/* Set $home if there is a user, and run the condition test. */
+
+if (userexists) deliver_home = pw->pw_dir;
+rc = direct_check_fsc(dblock, addr);
+if (rc != OK) return rc;
+
+/* Condition succeeded. If the local user check failed, we can now return
+DECLINE. */
+
+if (!userexists)
   {
-  DEBUG(2) debug_printf("%s director failed for %s: no such user\n",
+  DEBUG(2) debug_printf("%s director declined for %s: no such user\n",
     dblock->name, addr->local_part);
-  return FAIL;
+  return DECLINE;
   }
 
-/* Set the home directory, and if there is a setting of match_directory, check
-for a match. The resetting of deliver_home happens at the outer level. */
+/* If there is a setting of match_directory, check for a match. */
 
-deliver_home = pw->pw_dir;
 if (ob->pw_dir != NULL)
   {
   rc = direct_match_directory(addr, pw->pw_dir, ob->pw_dir, dblock->name);
   if (rc != OK) return rc;
   }
 
-/* Perform file existence and sender verification checks now that we
-have $home available. */
-
-rc = direct_check_fsc(dblock, addr);
-if (rc != OK) return rc;
-
 /* Set up the errors address, if any. */
 
 rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
@@ -136,9 +144,8 @@ addr->remove_headers = remove_headers;
 explicit settings, which will be filled in by the route_queue() function. */
 
 addr->uid = pw->pw_uid;
-addr->uid_set = TRUE;
 addr->gid = pw->pw_gid;
-addr->gid_set = TRUE;
+setflag(addr, af_uid_set|af_gid_set);
 
 if (dblock->home_directory != NULL) addr->home_dir = dblock->home_directory;
   else addr->home_dir = string_copy(pw->pw_dir);
