@@ -14,10 +14,24 @@
 /* Options specific to the smartuser director. */
 
 optionlist smartuser_director_options[] = {
+  { "directory2_transport",opt_transportptr,
+      (void *)(offsetof(smartuser_director_options_block, directory2_transport)) },
+  { "directory_transport",opt_transportptr,
+      (void *)(offsetof(smartuser_director_options_block, directory_transport)) },
+  { "file_transport",     opt_transportptr,
+      (void *)(offsetof(smartuser_director_options_block, file_transport)) },
+  { "forbid_file",        opt_bool,
+      (void *)(offsetof(smartuser_director_options_block, forbid_file)) },
+  { "forbid_pipe",        opt_bool,
+      (void *)(offsetof(smartuser_director_options_block, forbid_pipe)) },
+  { "hide_child_in_errmsg", opt_bool,
+      (void *)offsetof(smartuser_director_options_block, hide_child_in_errmsg) },
   { "new_address", opt_stringptr,
       (void *)(offsetof(smartuser_director_options_block, new_address)) },
   { "panic_expansion_fail", opt_bool,
       (void *)(offsetof(smartuser_director_options_block, panic_expansion_fail)) },
+  { "pipe_transport",     opt_transportptr,
+      (void *)(offsetof(smartuser_director_options_block, pipe_transport)) },
   { "qualify_preserve_domain", opt_bool,
       (void *)(offsetof(smartuser_director_options_block, qualify_preserve_domain)) },
   { "rewrite",              opt_bool,
@@ -31,8 +45,8 @@ int smartuser_director_options_count =
   sizeof(smartuser_director_options)/sizeof(optionlist);
 
 /* Default private options block for the smartuser director. The common options
-have to be present so it can use the common functions, but none of these values
-are at present changeable by any option. */
+have to be present so it can use the common functions, but not all of these
+values are changeable by an option. */
 
 smartuser_director_options_block smartuser_director_option_defaults = {
   /************** Common block ****************/
@@ -46,12 +60,24 @@ smartuser_director_options_block smartuser_director_option_defaults = {
   NULL,     /* expand_pipe_transport */
   NULL,     /* reply_transport */
   NULL,     /* expand_reply_transport */
+  NULL,     /* file */
+  NULL,     /* queries */
+  NULL,     /* query */
+  022,      /* modemask */
+  NULL,     /* owners */
+  NULL,     /* owngroups */
+  -1,       /* partial_match */
+  -1,       /* search_type */
   FALSE,    /* check_ancestor */
   FALSE,    /* check_local_user */
-  TRUE,     /* forbid_file */               /* If these are ever changed, we */
-  TRUE,     /* forbid_pipe */               /* will have to pay attention to */
-  TRUE,     /* forbid_reply */              /* user/group as for aliasfile   */
+  FALSE,    /* expand */
+  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_pipe */
+  TRUE,     /* forbid_reply */
+  FALSE,    /* include_domain */
+  FALSE,    /* hide_child_in_errmsg */
   FALSE,    /* one_time */
+  FALSE,    /* optional */
   /********************************************/
   NULL,     /* new_address */
   TRUE,     /* panic_expansion_fail */
@@ -119,6 +145,7 @@ char *raw_new_address, *error;
 header_line *extra_headers;
 address_item *generated = NULL;
 transport_instance *transport;
+ugid_block ugid;
 int yield = OK;
 int extracted, rc;
 
@@ -129,6 +156,10 @@ if (rc != OK) return rc;
 
 DEBUG(2) debug_printf("%s director called for %s\n", dblock->name, addr->orig);
 
+/* Sort out the fixed or dynamic uid/gid */
+
+if (!direct_get_ugid(dblock, "director", addr, &ugid)) return ERROR;
+
 /* Set up the errors address, if any. */
 
 rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
@@ -153,8 +184,7 @@ be verifying, because that configuration is allowed by the initialization
 function only when verify_only is set. The director is being used for some
 special kind of verification, based on generic conditions, and reaching this
 point indicates success. If there is a transport, we put the errors and header
-stuff into the current address and queue it for the transport. The
-route_queue() function will fill in any configured uid/gid. */
+stuff into the current address and queue it for the transport. */
 
 if (ob->new_address == NULL)
   {
@@ -166,7 +196,7 @@ if (ob->new_address == NULL)
   addr->home_dir = dblock->home_directory;
   addr->current_dir = dblock->current_directory;
   return route_queue(addr, addr_local, addr_remote, (router_instance *)dblock,
-    NULL, "director")? OK : ERROR;
+    &ugid, "director")? OK : ERROR;
   }
 
 
@@ -222,7 +252,9 @@ if (transport != NULL)
 
   /* Rewrite new address if configured */
 
-  if (ob->rewrite) new_address = rewrite_address(new_address, TRUE, FALSE);
+  if (ob->rewrite)
+    new_address = rewrite_address(new_address, TRUE, FALSE,
+      global_rewrite_rules, rewrite_existflags);
   domain = strchr(new_address, '@') - new_address + 1;
 
   /* Create child, making a copy of the old address, changing the domain and
@@ -256,10 +288,8 @@ if (transport != NULL)
     (generated->errors_address != NULL)? generated->errors_address : "",
     (generated->errors_address != NULL)? ")" : "");
 
-  /* The generic user/group values are picked up by this function */
-
   yield = route_queue(generated, addr_local, addr_remote,
-    (router_instance *)dblock, NULL, "director")? OK : ERROR;
+    (router_instance *)dblock, &ugid, "director")? OK : ERROR;
   }
 
 
@@ -291,12 +321,10 @@ else
     verify, FALSE, "smartuser new address");
 
   /* Since there is no transport, this is an aliasing operation, and we can use
-  the common function to deal with the generated addresses. The third last
-  argument can be NULL because pipes and files are locked out. If they are
-  ever allowed, we must sort this, as for alias and forward files. */
+  the common function to deal with the generated addresses. */
 
   df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
-    remove_headers, NULL, NULL, NULL);
+    remove_headers, &ugid, NULL, NULL);
 
   /* If no children were generated for this address, but no error was given, it
   means that there was just a :blackhole: entry. Log something. */
@@ -304,8 +332,6 @@ else
   if (addr->child_count <= 0 && !verify && !address_test_mode)
     log_write(0, LOG_MAIN, "=> :blackhole: <%s> D=%s", addr->orig,
       dblock->name);
-
-  yield = OK_CONTINUE; /* Indicate that verification should continue */
   }
 
 /* Put the original address onto the succeed queue. This ensures that any
