@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -67,6 +67,10 @@ be NULL for methods that don't need them. */
 #include "lookups/nisplus.h"
 #endif
 
+#ifdef LOOKUP_PGSQL
+#include "lookups/pgsql.h"
+#endif
+
 #ifdef LOOKUP_TESTDB
 #include "lookups/testdb.h"
 #endif
@@ -153,7 +157,7 @@ of the key strings. */
 #endif
   },
 
-/* LDAP lookup */
+/* LDAP lookup, allowing data from only one entry to be returned */
 
   {
   "ldap",                        /* lookup name */
@@ -170,7 +174,7 @@ of the key strings. */
 #endif
   },
 
-/* LDAP lookup, allowing multiple values to be returned */
+/* LDAP lookup, allowing data from more than one entry to be returned */
 
   {
   "ldapm",                       /* lookup name */
@@ -187,6 +191,23 @@ of the key strings. */
 #endif
   },
 
+/* LDAP lookup, allowing the DN from more one entry to be returned */
+
+  {
+  "ldapdn",                       /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_LDAP
+  eldap_open,       /* sic */    /* open function */
+  NULL,                          /* check function */
+  eldapdn_find,                  /* find function */
+  NULL,                          /* no close function */
+  eldap_tidy,       /* sic */    /* tidy function */
+  eldap_quote       /* sic */    /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
 /* Linear search of single file */
 
   {
@@ -272,6 +293,23 @@ of the key strings. */
 #endif
   },
 
+/* PGSQL lookup */
+
+  {
+  "pgsql",                       /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_PGSQL
+  pgsql_open,                    /* open function */
+  NULL,                          /* no check function */
+  pgsql_find,                    /* find function */
+  NULL,                          /* no close function */
+  pgsql_tidy,                    /* tidy function */
+  pgsql_quote                    /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+#endif
+  },
+
 /* Testdb lookup is for testing Exim, not useful for normal running.
 For that reason, we omit the entry entirely when not building it into
 the binary, so that attempts to use it give "unknown lookup type" instead
@@ -298,12 +336,9 @@ of "lookup type not available". */
 
 /* Table of information about all possible authentication mechamisms. All
 entries are always present if any mechanism is declared, but the functions are
-set to NULL for those that are not compiled into the binary. None of this is
-wanted when building the test program for expand.c, which just needs the tables
-for lookups. */
+set to NULL for those that are not compiled into the binary. */
 
 #ifdef HAVE_AUTH
-#ifndef TEST_EXPAND
 
 #ifdef AUTH_CRAM_MD5
 #include "auths/cram_md5.h"
@@ -346,15 +381,11 @@ auth_info auths_available[] = {
 { "", NULL, NULL, NULL, NULL, 0, NULL, 0  }
 };
 
-#endif  /* TEST_EXPAND */
 #endif  /* HAVE_AUTH */
 
 
 /* Tables of information about which directors, routers and transports are
-included in the exim binary. None are wanted when building the test program for
-expand.c, which just needs the lookup tables above. */
-
-#ifndef TEST_EXPAND
+included in the exim binary. */
 
 /* Pull in the necessary header files */
 
@@ -408,6 +439,10 @@ expand.c, which just needs the lookup tables above. */
 #include "transports/debug.h"
 #endif
 
+#ifdef TRANSPORT_LMTP
+#include "transports/lmtp.h"
+#endif
+
 #ifdef TRANSPORT_PIPE
 #include "transports/pipe.h"
 #endif
@@ -568,6 +603,20 @@ transport_info transports_available[] = {
   NULL                                         /* no closedown entry */
   },
 #endif
+#ifdef TRANSPORT_LMTP
+  {
+  "lmtp",                                      /* driver name */
+  lmtp_transport_entry,                        /* main entry point */
+  lmtp_transport_init,                         /* init entry point */
+  NULL,                                        /* no tidyup entry */
+  lmtp_transport_options,                      /* local options table */
+  &lmtp_transport_options_count,               /* number of entries */
+  &lmtp_transport_option_defaults,             /* private options defaults */
+  sizeof(lmtp_transport_options_block),        /* size of private block */
+  TRUE,                                        /* local flag */
+  NULL                                         /* no closedown entry */
+  },
+#endif
 #ifdef TRANSPORT_PIPE
   {
   "pipe",                                      /* driver name */
@@ -599,6 +648,4 @@ transport_info transports_available[] = {
 { "", NULL, NULL, NULL, NULL, 0, NULL, 0, FALSE }
 };
 
-#endif  /* TEST_EXPAND */
-
 /* End of drtables.c */
