@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file is not part of the main Exim code. The Exim monitor and some other
@@ -132,4 +132,19 @@ panic = panic;
 void
 host_build_sender_fullhost(void) { }
 
+/* This one isn't needed for test_host */
+
+#ifndef TEST_HOST
+char *
+host_ntoa(int type, const void *arg, char *buffer, int *portptr)
+{
+type = type;      /* Keep picky compilers happy */
+arg = arg;
+buffer = buffer;
+portptr = portptr;
+return NULL;
+}
+#endif
+
+
 /* End of dummies.c */
