@@ -20,6 +20,12 @@ are needed by any of the other headers, including system headers. */
 
 #include "os.h"
 
+/* If it didn't define os_find_running_interfaces, use the common function. */
+
+#ifndef os_find_running_interfaces
+#define os_find_running_interfaces os_common_find_running_interfaces
+#endif
+
 
 /* ANSI C standard includes */
 
@@ -189,13 +195,29 @@ present Exim doesn't actually use it. */
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
 
-/* While IPv6 is still young the definition of T_AAAA may not be included in
-arpa/nameser.h. Fudge it here. */
+/* If arpa/nameser.h defines a maximum name server packet size, use it,
+provided it is greater than 1024. Otherwise go for a default. PACKETSZ was used
+for this, but it seems that NS_PACKETSZ is coming into use. */
+
+#if defined(NS_PACKETSZ) && NS_PACKETSZ >= 1024
+  #define MAXPACKET NS_PACKETSZ
+#elif defined(PACKETSZ) && PACKETSZ >= 1024
+  #define MAXPACKET PACKETSZ
+#else
+  #define MAXPACKET 1024
+#endif
+
+/* While IPv6 is still young the definitions of T_AAAA and T_A6 may not be
+included in arpa/nameser.h. Fudge them here. */
 
 #ifndef T_AAAA
 #define T_AAAA 28
 #endif
 
+#ifndef T_A6
+#define T_A6 38
+#endif
+
 /* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
 header files. */
 
