@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,9 +14,9 @@ never get flushed out.
 
 This program is clever enough to cope with ndbm, which creates two files called
 <name>.dir and <name>.pag, or with db, which creates a single file called
-<name>.db. If native db is in use (USE_DB defined) then there is no extension
-to the output filename. This is also handled. If there are any other variants,
-the program won't cope.
+<name>.db. If native db is in use (USE_DB defined) or tdb is in use (USE_TDB
+defined) there is no extension to the output filename. This is also handled. If
+there are any other variants, the program won't cope.
 
 The first argument to the program is the name of the serial file; the second
 is the base name for the DBM file(s). When native db is in use, these must be
@@ -33,6 +33,20 @@ indented lines. */
 #define maxsize 10000
 
 
+#ifdef STRERROR_FROM_ERRLIST
+/* Some old-fashioned systems still around (e.g. SunOS4) don't have strerror()
+in their libraries, but can provide the same facility by this simple
+alternative function. */
+
+char *
+strerror(int n)
+{
+if (n < 0 || n >= sys_nerr) return "unknown error number";
+return sys_errlist[n];
+}
+#endif /* STRERROR_FROM_ERRLIST */
+
+
 int main(int argc, char **argv)
 {
 int started;
@@ -45,7 +59,7 @@ BOOL lowercase = TRUE;
 BOOL warn = TRUE;
 BOOL duperr = TRUE;
 BOOL lastdup = FALSE;
-#ifndef USE_DB
+#if !defined (USE_DB) && !defined(USE_TDB)
 int is_db = 0;
 struct stat statbuf;
 #endif
@@ -90,7 +104,7 @@ if (strcmp(argv[arg], "-") == 0) f = stdin; else
 /* By default Berkeley db does not put extensions on... which
 can be painful! */
 
-#ifdef USE_DB
+#if defined(USE_DB) || defined(USE_TDB)
 if (strcmp(argv[arg], argv[arg+1]) == 0)
   {
   printf("exim_dbmbuild: input and output filenames are the same\n");
@@ -117,7 +131,7 @@ if (d == NULL)
 /* Unless using native db calls, see if we have created <name>.db; if not,
 assume .dir & .pag */
 
-#ifndef USE_DB
+#if !defined(USE_DB) && !defined(USE_TDB)
 sprintf(real_dbmname, "%s.db", temp_dbmname);
 is_db = stat(real_dbmname, &statbuf) == 0;
 #endif
@@ -133,14 +147,14 @@ while (fgets(line, maxsize, f) != NULL)
 
   if (line[0] == '#') continue;
   p = line + (int)strlen(line);
-  while (p > line && isspace(p[-1])) p--;
+  while (p > line && isspace((uschar)p[-1])) p--;
   *p = 0;
   if (line[0] == 0) continue;
 
   /* A continuation line is valid only if there was a previous first
   line. */
 
-  if (isspace(line[0]))
+  if (isspace((uschar)line[0]))
     {
     char *s = line;
     if (!started)
@@ -148,7 +162,7 @@ while (fgets(line, maxsize, f) != NULL)
       printf("Unexpected continuation line ignored\n%s\n\n", line);
       continue;
       }
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     *(--s) = ' ';
     strcpy (bptr, s);
     bptr += p - s;
@@ -166,9 +180,7 @@ while (fgets(line, maxsize, f) != NULL)
 
     if (started)
       {
-      #ifdef EXIM_CLEAR_DATUMSTRUCT
-      memset(&content, 0, sizeof(content));
-      #endif
+      EXIM_DATUM_INIT(content);
       EXIM_DATUM_DATA(content) = buffer;
       EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
 
@@ -196,10 +208,8 @@ while (fgets(line, maxsize, f) != NULL)
       bptr = buffer;
       }
 
-    while (*s != 0 && *s != ':' && !isspace(*s)) s++;
-    #ifdef EXIM_CLEAR_DATUMSTRUCT
-    memset(&key, 0, sizeof(key));
-    #endif
+    while (*s != 0 && *s != ':' && !isspace((uschar)*s)) s++;
+    EXIM_DATUM_INIT(key);
     EXIM_DATUM_DATA(key) = keybuffer;
     EXIM_DATUM_SIZE(key) = s - line + add_zero;
 
@@ -225,11 +235,11 @@ while (fgets(line, maxsize, f) != NULL)
     keybuffer[i] = 0;
     started = 1;
 
-    while (isspace(*s))s++;
+    while (isspace((uschar)*s))s++;
     if (*s == ':')
       {
       s++;
-      while (isspace(*s))s++;
+      while (isspace((uschar)*s))s++;
       }
     if (*s != 0)
       {
@@ -243,10 +253,7 @@ while (fgets(line, maxsize, f) != NULL)
 if (started)
   {
   int rc;
-
-  #ifdef EXIM_CLEAR_DATUMSTRUCT
-  memset(&content, 0, sizeof(content));
-  #endif
+  EXIM_DATUM_INIT(content);
   EXIM_DATUM_DATA(content) = buffer;
   EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
 
@@ -289,7 +296,7 @@ if (yield == 0 || yield == 1)
     printf("%d duplicate key%s \n", dupcount, (dupcount > 1)? "s" : "");
     }
 
-  #ifdef USE_DB
+  #if defined(USE_DB) || defined(USE_TDB)
   strcpy(real_dbmname, temp_dbmname);
   strcpy(buffer, argv[arg+1]);
   if (rename(real_dbmname, buffer) != 0)
@@ -333,7 +340,7 @@ if (yield == 0 || yield == 1)
       }
     }
 
-  #endif /* USE_DB */
+  #endif /* USE_DB || USE_TDB */
   }
 
 /* Otherwise unlink the temporary files. */
@@ -341,7 +348,7 @@ if (yield == 0 || yield == 1)
 else
   {
   printf("dbmbuild abandoned\n");
-  #ifdef USE_DB
+  #if defined(USE_DB) || defined(USE_TDB)
   unlink(temp_dbmname);
   #else
   if (is_db)
@@ -356,7 +363,7 @@ else
     sprintf(real_dbmname, "%s.pag", temp_dbmname);
     unlink(real_dbmname);
     }
-  #endif /* USE_DB */
+  #endif /* USE_DB || USE_TDB */
   }
 
 return yield;
