@@ -15,6 +15,7 @@
 
 #include <pwd.h>                 /* Because used in includes below */
 #include "pcre/pcre.h"           /* Ditto */
+#define  MAXPACKET 1024          /* Ditto */
 
 #include "macros.h"
 #include "config.h"
@@ -113,17 +114,12 @@ if (dbdata_type == 0)
   }
 
 /* Set up the spool directory in the global place, open the database - this
-gets a read lock as well. */
+gets a read lock as well. After an open failure, errno=0 for locking errors
+that have already been logged (i.e. written to stderr). */
 
 spool_directory = argv[1];
-dbm = dbfn_open(argv[2], O_RDONLY, &dbblock);
-
-if (dbm == NULL)
-  {
-  printf("Failed to open database file %s (spool directory = %s)\n",
-    argv[2], spool_directory);
-  exit(1);
-  }
+dbm = dbfn_open(argv[2], O_RDONLY, &dbblock, TRUE);
+if (dbm == NULL) exit(1);
 
 /* Scan the file, formatting the information for each entry. Note
 that data is returned in a malloc'ed block, in order that it be
