@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -19,6 +19,7 @@
 
 #include <pwd.h>                  /* Because used in includes below */
 #include "pcre/pcre.h"            /* Ditto */
+#define  MAXPACKET 1024           /* Ditto */
 
 #include "macros.h"
 #include "config.h"
@@ -114,12 +115,12 @@ tm->tm_isdst = -1;
 for (t = s + (int)strlen(s) - 1; t >= s; t--)
   {
   if (*t == ':') continue;
-  if (!isdigit(*t)) return -1;
+  if (!isdigit((uschar)*t)) return -1;
 
   value = *t - '0';
   if (--t >= s)
     {
-    if (!isdigit(*t)) return -1;
+    if (!isdigit((uschar)*t)) return -1;
     value = value + (*t - '0')*10;
     }
 
@@ -203,7 +204,8 @@ for(;;)
   /* If the buffer contains just one digit, or just consists of "d", use the
   previous name for an update. */
 
-  if ((isdigit(buffer[0]) && !isdigit(buffer[1])) || strcmp(buffer, "d") == 0)
+  if ((isdigit((uschar)buffer[0]) && !isdigit((uschar)buffer[1])) ||
+       strcmp(buffer, "d") == 0)
     {
     if (name[0] == 0)
       {
@@ -223,12 +225,8 @@ for(;;)
   if (field[0] != 0)
     {
     int verify = 1;
-    dbm = dbfn_open(argv[2], O_RDWR, &dbblock);
-    if (dbm == NULL)
-      {
-      printf("*** failed to open the database: %s\n", strerror(errno));
-      continue;
-      }
+    dbm = dbfn_open(argv[2], O_RDWR, &dbblock, TRUE);
+    if (dbm == NULL) continue;
 
     if (strcmp(field, "d") == 0)
       {
@@ -239,7 +237,7 @@ for(;;)
       continue;
       }
 
-    else if (isdigit(field[0]))
+    else if (isdigit((uschar)field[0]))
       {
       int fieldno = atoi(field);
       if (value[0] == 0)
@@ -334,12 +332,8 @@ for(;;)
 
   /* Handle a read request, or verify after an update. */
 
-  dbm = dbfn_open(argv[2], O_RDONLY, &dbblock);
-  if (dbm == NULL)
-    {
-    printf("*** failed to open the database: %s\n", strerror(errno));
-    continue;
-    }
+  dbm = dbfn_open(argv[2], O_RDONLY, &dbblock, TRUE);
+  if (dbm == NULL) continue;
 
   record = dbfn_read_with_length(dbm, name, &length);
   if (record == NULL)
