@@ -124,6 +124,7 @@ static var_entry var_table[] = {
   { "prohibition_reason",  vtype_stringptr,   &prohibition_reason },
   { "qualify_domain",      vtype_stringptr,   &qualify_domain_sender },
   { "qualify_recipient",   vtype_stringptr,   &qualify_domain_recipient },
+  { "rbl_domain",          vtype_stringptr,   &rbl_domain },
   { "rbl_text",            vtype_stringptr,   &rbl_msg_buffer },
   { "received_for",        vtype_stringptr,   &received_for },
   { "received_protocol",   vtype_stringptr,   &received_protocol },
@@ -142,6 +143,7 @@ static var_entry var_table[] = {
   { "sender_host_address", vtype_stringptr,   &sender_host_address },
   { "sender_host_authenticated",vtype_stringptr, &sender_host_authenticated },
   { "sender_host_name",    vtype_stringptr,   &sender_host_name },
+  { "sender_host_port",    vtype_int,         &sender_host_port },
   { "sender_ident",        vtype_stringptr,   &sender_ident },
   { "sender_rcvhost",      vtype_stringptr,   &sender_rcvhost },
   { "sn0",                 vtype_filter_int,  &filter_sn[0] },
@@ -156,6 +158,10 @@ static var_entry var_table[] = {
   { "sn9",                 vtype_filter_int,  &filter_sn[9] },
   { "spool_directory",     vtype_stringptr,   &spool_directory },
   { "thisaddress",         vtype_stringptr,   &filter_thisaddress },
+#ifdef SUPPORT_TLS
+  { "tls_cipher",          vtype_stringptr,   &tls_cipher },
+  { "tls_peerdn",          vtype_stringptr,   &tls_peerdn },
+#endif
   { "tod_bsdinbox",        vtype_todbsdin,    NULL },
   { "tod_full",            vtype_todf,        NULL },
   { "tod_log",             vtype_todl,        NULL },
@@ -938,9 +944,9 @@ else if (strcmp(name, "eq") == 0 ||
     if (strncmp(sub[1], "{md5}", 5) == 0)
       {
       md5 base;
-      unsigned char digest[16];
+      uschar digest[16];
       md5_start(&base);
-      md5_end(&base, (unsigned char *)sub[0], (int)strlen(sub[0]), digest);
+      md5_end(&base, (uschar *)sub[0], (int)strlen(sub[0]), digest);
       *yield = (strcmp(auth_b64encode((char *)digest, 16), sub[1]+5) == 0)
         == testfor;
       }
@@ -1121,6 +1127,154 @@ for (i = 0; i <= expand_nmax; i++)
 
 
 
+/*************************************************
+*            Handle yes/no substrings            *
+*************************************************/
+
+/* This function is used by ${if}, ${lookup} and ${extract} to handle the
+alternative substrings that depend on whether or not the condition was true,
+or the lookup or extraction succeeded. The substrings always have to be
+expanded, to check their syntax, but "skipping" is set when the result is not
+needed - this avoids unnecessary nested lookups.
+
+Arguments:
+  skipping       TRUE if we were skipping when this item was reached
+  yes            TRUE if the first string is to be used, else use the second
+  errmsg         NULL, or an additional message for failure
+  save_lookup    a value to put back into lookup_value before the 2nd expansion
+  sptr           points to the input string pointer
+  yieldptr       points to the output string pointer
+  sizeptr        points to the output string size
+  ptrptr         points to the output string pointer
+  type           "lookup" or "if" or "extract", for error message
+
+Returns:         0 OK; lookup_value has been reset to save_lookup
+                 1 expansion failed
+                 2 expansion failed because of bracketing error
+*/
+
+static int
+process_yesno(BOOL skipping, BOOL yes, char *errmsg, char *save_lookup, char
+  **sptr, char **yieldptr, int *sizeptr, int *ptrptr, char *type)
+{
+int rc = 0;
+char *s = *sptr;    /* Local value */
+char *sub1, *sub2;
+
+/* If there are no following strings, we substitute the contents of $value for
+lookups and for extractions in the success case. In the fail case, nothing is
+substituted. In the case of "if", lack of following strings is an error. */
+
+while (isspace((uschar)*s)) s++;
+if (*s == '}')
+  {
+  if (type[0] == 'i') goto FAILED_CURLY;
+  if (yes)
+    *yieldptr = string_cat(*yieldptr, sizeptr, ptrptr, lookup_value,
+      (int)strlen(lookup_value));
+  lookup_value = save_lookup;
+  s++;
+  goto RETURN;
+  }
+
+/* Expand the first substring. Forced failures are noticed only if we actually
+want this string. Set skipping in the call in the fail case (this will always
+be the case if we were already skipping). */
+
+sub1 = expand_string_internal(s+1, TRUE, &s, !yes);
+if (sub1 == NULL && (yes || !expand_string_forcedfail)) goto FAILED;
+expand_string_forcedfail = FALSE;
+if (*s++ != '}') goto FAILED_CURLY;
+
+/* If we want the first string, add it to the output */
+
+if (yes)
+  *yieldptr = string_cat(*yieldptr, sizeptr, ptrptr, sub1, (int)strlen(sub1));
+
+/* If this is called from a lookup or an extract, we want to restore $value to
+what it was at the start of the item, so that it has this value during the
+second string expansion. For the call from "if" to this function, save_lookup
+is set to lookup_value, so that this statement does nothing. */
+
+lookup_value = save_lookup;
+
+/* There now follows either another substring, or "fail", or nothing. This
+time, forced failures are noticed only if we want the second string. We must
+set skipping in the nested call if we don't want this string, or if we were
+already skipping. */
+
+while (isspace((uschar)*s)) s++;
+if (*s == '{')
+  {
+  sub2 = expand_string_internal(s+1, TRUE, &s, yes || skipping);
+  if (sub2 == NULL && (!yes || !expand_string_forcedfail)) goto FAILED;
+  expand_string_forcedfail = FALSE;
+  if (*s++ != '}') goto FAILED_CURLY;
+
+  /* If we want the second string, add it to the output */
+
+  if (!yes)
+    *yieldptr = string_cat(*yieldptr, sizeptr, ptrptr, sub2, (int)strlen(sub2));
+  }
+
+/* If there is no second string, but the word "fail" is present when the use of
+the second string is wanted, set a flag indicating it was a forced failure
+rather than a syntactic error. Swallow the terminating } in case this is nested
+inside another lookup or if or extract. */
+
+else if (*s != '}')
+  {
+  char name[256];
+  s = read_name(name, sizeof(name), s, "_");
+  if (strcmp(name, "fail") == 0)
+    {
+    if (!yes)
+      {
+      while (isspace((uschar)*s)) s++;
+      if (*s++ != '}') goto FAILED_CURLY;
+      expand_string_message =
+        string_sprintf("\"%s\" failed and \"fail\" requested%s%s", type,
+          (errmsg == NULL || errmsg[0] == 0)? "" : ": ",
+          (errmsg == NULL)? "" : errmsg);
+      expand_string_forcedfail = TRUE;
+      goto FAILED;
+      }
+    }
+  else
+    {
+    expand_string_message =
+      string_sprintf("syntax error in \"%s\" item - \"fail\" expected", type);
+    goto FAILED;
+    }
+  }
+
+/* All we have to do now is to check on the final closing brace. */
+
+while (isspace((uschar)*s)) s++;
+if (*s++ == '}') goto RETURN;
+
+/* Get here if there is a bracketing failure */
+
+FAILED_CURLY:
+rc++;
+
+/* Get here for other failures */
+
+FAILED:
+rc++;
+
+/* Update the input pointer value before returning */
+
+RETURN:
+*sptr = s;
+return rc;
+}
+
+
+
+
+
+
 /*************************************************
 *                 Expand string                  *
 *************************************************/
@@ -1137,6 +1291,10 @@ store. Interpreted sequences are:
    ${extract {field} {separator} {string}}
                            extracts numbered field from separated string;
                            null if not found; field 0 is the whole string
+
+   Either form of extract may now have two additional arguments; these are
+   used in the success/failure cases, as for ${if} and ${lookup}.
+
    ${if cond {s1} {s2}}    conditional; the yielded string is expanded
                            {s2} can be replaced by "fail" or be omitted
    ${perl{sub}{a1}...      call Perl (if configured)
@@ -1160,6 +1318,7 @@ Database-query lookups:
 
 Operators:
    domain                  extracts domain from an address
+   escape                  escapes non-printing characters
    expand                  expands the string one more time
    hash_<n>_<m>            hash the string, making one that is of length n,
                              using m (default 26) characters from hashcodes
@@ -1172,7 +1331,7 @@ Operators:
    length_<n>              take first n chars only
    l_<n>                   ditto
    local_part              extracts local part from an address
-   mask_n                  masks an IP address
+   mask                    masks an IP address
    quote                   quote the argument if it contains anything other
                              than letters, digits, underscores, dots, & hyphens;
                              quoting means putting inside "" and \-quoting any
@@ -1238,6 +1397,9 @@ int ptr = 0;
 int size = (int)strlen(string) + 64;
 char *s = string;
 char *yield = store_get(size);
+BOOL malformed_header = FALSE;
+char *save_expand_nstring[EXPAND_MAXN+1];
+int save_expand_nlength[EXPAND_MAXN+1];
 
 expand_string_forcedfail = FALSE;
 expand_string_message = "";
@@ -1292,6 +1454,12 @@ while (*s != 0)
       s = read_header_name(name, sizeof(name), s);
       value = find_header(name);
       choplen = 1;
+
+      /* If we didn't find the header, and the header contains a closing brace
+      characters, this may be a user error where the terminating colon
+      has been omitted. Set a flag to adjust the error message. */
+
+      if (value == NULL && strchr(name, '}') != NULL) malformed_header = TRUE;
       }
     else
       {
@@ -1367,76 +1535,30 @@ while (*s != 0)
 
   if (strcmp(name, "if") == 0)
     {
-    BOOL cond = TRUE;
-    char *sub1, *sub2;
-    char *save_expand_nstring[EXPAND_MAXN+1];
-    int save_expand_nlength[EXPAND_MAXN+1];
+    BOOL cond = FALSE;
     int save_expand_nmax =
       save_expand_strings(save_expand_nstring, save_expand_nlength);
 
     s = eval_condition(s, skipping? NULL : &cond);
     if (s == NULL) goto EXPAND_FAILED;  /* message already set */
 
-    /* The condition must be followed by one or two substrings,
-    enclosed in braces, with optional space between them and
-    after them. Then there must be a final } to end. */
+    /* The handling of "yes" and "no" result strings is now in a separate
+    function that is also used by ${lookup} and ${extract}. */
 
-    while (isspace((uschar)*s)) s++;
-    if (*s != '{') goto EXPAND_FAILED_CURLY;
-
-    /* The skipping flag is already TRUE if we are in the middle of
-    some skipping text. We need to force it true also for the value
-    we are not going to use. */
-
-    sub1 = expand_string_internal(s+1, TRUE, &s, skipping || !cond);
-
-    /* Forced failures are noticed only if the condition was true */
-
-    if (sub1 == NULL && (cond || !expand_string_forcedfail))
-      goto EXPAND_FAILED;
-
-    expand_string_forcedfail = FALSE;
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-
-    while (isspace((uschar)*s)) s++;
-    if (*s == '}') sub2 = NULL;
-    else if (*s == '{')
+    switch(process_yesno(
+             skipping,                     /* were previously skipping */
+             cond,                         /* success/failure indicator */
+             NULL,                         /* message for failure */
+             lookup_value,                 /* value to reset for string2 */
+             &s,                           /* input pointer */
+             &yield,                       /* output pointer */
+             &size,                        /* output size */
+             &ptr,                         /* output current point */
+             "if"))                        /* condition type */
       {
-      sub2 = expand_string_internal(s+1, TRUE, &s, skipping || cond);
-      if (sub2 == NULL && (!cond || !expand_string_forcedfail))
-        goto EXPAND_FAILED;
-      expand_string_forcedfail = FALSE;
-      if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+      case 1: goto EXPAND_FAILED;          /* when all is well, the */
+      case 2: goto EXPAND_FAILED_CURLY;    /* returned value is 0 */
       }
-    else
-      {
-      s = read_name(name, sizeof(name), s, "_");
-      if (strcmp(name, "fail") == 0)
-        {
-        if (!cond)
-          {
-          expand_string_message = "\"if\" failed and \"fail\" requested";
-          expand_string_forcedfail = TRUE;
-          goto EXPAND_FAILED;
-          }
-        else sub2 = NULL;
-        }
-      else
-        {
-        expand_string_message = "syntax error in \"else\" substring";
-        goto EXPAND_FAILED;
-        }
-      }
-
-    while (isspace((uschar)*s)) s++;
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-
-    /* Add the appropriate string to the result */
-
-    if (cond)
-      yield = string_cat(yield, &size, &ptr, sub1, (int)strlen(sub1));
-    else if (sub2 != NULL)
-      yield = string_cat(yield, &size, &ptr, sub2, (int)strlen(sub2));
 
     /* Restore external setting of expansion variables for continuation
     at this level. */
@@ -1458,13 +1580,9 @@ while (*s != 0)
     int stype;
     int pv = -1;
     int expand_setup = 0;
-    BOOL looked_up;
-    char *ss, *key, *filename, *sub1;
+    char *ss, *key, *filename;
     char *lookup_errmsg = NULL;
     char *save_lookup_value = lookup_value;
-
-    char *save_expand_nstring[EXPAND_MAXN+1];
-    int save_expand_nlength[EXPAND_MAXN+1];
     int save_expand_nmax =
       save_expand_strings(save_expand_nstring, save_expand_nlength);
 
@@ -1621,86 +1739,24 @@ while (*s != 0)
       if (expand_setup > 0) expand_nmax = expand_setup;
       }
 
-    /* Remember whether this lookup succeeded or not; the actual value
-    of lookup_value may get changed if the nested expansions also do
-    lookups. */
+    /* The handling of "yes" and "no" result strings is now in a separate
+    function that is also used by ${if} and ${extract}. */
 
-    looked_up = lookup_value != NULL;
-
-    /* Expand the first substring; $value will get the looked up value. Set
-    skipping in the call if the lookup failed (this will always be the case
-    if we are already skipping). */
-
-    while (isspace((uschar)*s)) s++;
-    if (*s != '{') goto EXPAND_FAILED_CURLY;
-    sub1 = expand_string_internal(s+1, TRUE, &s, !looked_up);
-
-    /* Forced failures are noticed only if the original lookup succeeded. */
-
-    if (sub1 == NULL && (looked_up || !expand_string_forcedfail))
-      goto EXPAND_FAILED;
-    expand_string_forcedfail = FALSE;
-
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-
-    /* If the lookup succeeded, add the new string to the output. */
-
-    if (looked_up)
-      yield = string_cat(yield, &size, &ptr, sub1, (int)strlen(sub1));
-
-    /* Restore $value to what it was on entry so that it has this value
-    during the fail string. */
-
-    lookup_value = save_lookup_value;
-
-    /* There now follows either another substring, or "fail", or nothing.
-    This time, forced failures are noticed only if the original lookup failed.
-    We must set skipping in the nested call if this lookup succeeded, or if
-    we are already skipping. */
-
-    while (isspace((uschar)*s)) s++;
-    if (*s == '{')
+    switch(process_yesno(
+             skipping,                     /* were previously skipping */
+             lookup_value != NULL,         /* success/failure indicator */
+             lookup_errmsg,                /* message for failure */
+             save_lookup_value,            /* value to reset for string2 */
+             &s,                           /* input pointer */
+             &yield,                       /* output pointer */
+             &size,                        /* output size */
+             &ptr,                         /* output current point */
+             "lookup"))                    /* condition type */
       {
-      char *sub2 = expand_string_internal(s+1, TRUE, &s, looked_up || skipping);
-      if (sub2 == NULL && (!looked_up || !expand_string_forcedfail))
-        goto EXPAND_FAILED;
-      expand_string_forcedfail = FALSE;
-      if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-      if (!looked_up)
-        yield = string_cat(yield, &size, &ptr, sub2, (int)strlen(sub2));
+      case 1: goto EXPAND_FAILED;          /* when all is well, the */
+      case 2: goto EXPAND_FAILED_CURLY;    /* returned value is 0 */
       }
 
-    /* If the word "fail" is present, and lookup failed, set a flag indicating
-    it was a forced failure rather than a syntactic error. Swallow the
-    terminating } in case this is nested inside another lookup or if. */
-
-    else if (*s != '}')
-      {
-      s = read_name(name, sizeof(name), s, "_");
-      if (strcmp(name, "fail") == 0)
-        {
-        if (!looked_up)
-          {
-          while (isspace((uschar)*s)) s++;
-          if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-          expand_string_message =
-            string_sprintf("lookup failed and \"fail\" requested%s%s",
-              (lookup_errmsg == NULL || lookup_errmsg[0] == 0)? "" : ": ",
-              (lookup_errmsg == NULL)? "" : lookup_errmsg);
-          expand_string_forcedfail = TRUE;
-          goto EXPAND_FAILED;
-          }
-        }
-      else
-        {
-        expand_string_message = "syntax error in lookup substring";
-        goto EXPAND_FAILED;
-        }
-      }
-
-    while (isspace((uschar)*s)) s++;
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-
     /* Restore external setting of expansion variables for carrying on
     at this level, and continue. */
 
@@ -1852,9 +1908,6 @@ while (*s != 0)
     const char *rerror;
     char *subject;
     char *sub[3];
-
-    char *save_expand_nstring[EXPAND_MAXN+1];
-    int save_expand_nlength[EXPAND_MAXN+1];
     int save_expand_nmax =
       save_expand_strings(save_expand_nstring, save_expand_nlength);
 
@@ -1954,14 +2007,22 @@ while (*s != 0)
     continue;
     }
 
-  /* Handle keyed and numbered substring extraction. They are distinguised
-  by different numbers of arguments. */
+  /* Handle keyed and numbered substring extraction. If the first argument
+  consists entirely of digits, then a numerical extraction is assumed. */
 
   if (strcmp(name, "extract") == 0)
     {
     int i;
-    char *sub[4];
-    for (i = 0; i < 3; i++)
+    int j = 2;
+    int field_number = -1;
+    char *save_lookup_value = lookup_value;
+    char *sub[3];
+    int save_expand_nmax =
+      save_expand_strings(save_expand_nstring, save_expand_nlength);
+
+    /* Read the arguments */
+
+    for (i = 0; i < j; i++)
       {
       while (isspace((uschar)*s)) s++;
       if (*s == '{')
@@ -1969,44 +2030,66 @@ while (*s != 0)
         sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
         if (sub[i] == NULL) goto EXPAND_FAILED;
         if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+
+        /* The first argument must not be empty; if it consists entirely of
+        digits, this is a numerical extraction, and we expect 3 arguments. */
+
+        if (i == 0)
+          {
+          int x = 0;
+          char *p = sub[0];
+
+          if (*p == 0)
+            {
+            expand_string_message = "first argument of \"expand\" must not "
+              "be empty";
+            goto EXPAND_FAILED;
+            }
+
+          while (*p != 0 && isdigit((uschar)*p)) x = x * 10 + *p++ - '0';
+          if (*p == 0)
+            {
+            field_number = x;
+            j = 3;
+            }
+          }
         }
-      else
-        {
-        /* Handle either 2 or 3 arguments for 2 versions of extract */
-        if (i < 2) goto EXPAND_FAILED_CURLY;
-        sub[i] = NULL;
-        }
+      else goto EXPAND_FAILED_CURLY;
       }
 
-    while (isspace((uschar)*s)) s++;
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+    /* Extract either the numbered or the keyed substring into $value. */
 
-    /* If there are three arguments, this is a numbered field extraction,
-    and the first argument must be numerical. If there are two arguments,
-    this is a keyed field extraction. */
+    lookup_value = (field_number >= 0)?
+      expand_gettokened(field_number, sub[1], sub[2]) :
+      expand_getkeyed(sub[0], sub[1]);
 
-    if (sub[2] != NULL)
+    /* If no string follows, $value gets substituted; otherwise there can
+    be yes/no strings, as for lookup or if. */
+
+    switch(process_yesno(
+             skipping,                     /* were previously skipping */
+             lookup_value != NULL,         /* success/failure indicator */
+             NULL,                         /* message for failure */
+             save_lookup_value,            /* value to reset for string2 */
+             &s,                           /* input pointer */
+             &yield,                       /* output pointer */
+             &size,                        /* output size */
+             &ptr,                         /* output current point */
+             "extract"))                   /* condition type */
       {
-      char *endptr;
-      int field_number = (int)strtoul(sub[0], &endptr, 10);
-      if (*endptr != 0)
-        {
-        expand_string_message = string_sprintf("field number \"%s\" for "
-          "extract is not numeric", sub[0]);
-        goto EXPAND_FAILED;
-        }
-      sub[3] = expand_gettokened(field_number, sub[1], sub[2]);
+      case 1: goto EXPAND_FAILED;          /* when all is well, the */
+      case 2: goto EXPAND_FAILED_CURLY;    /* returned value is 0 */
       }
-    else sub[3] = expand_getkeyed(sub[0], sub[1]);
 
-    /* Insert yield if extraction succeeded; otherwise nothing. */
+    /* All done - restore numerical variables. */
+
+    restore_expand_strings(save_expand_nmax, save_expand_nstring,
+      save_expand_nlength);
 
-    if (sub[3] != NULL)
-      yield = string_cat(yield, &size, &ptr, sub[3], (int)strlen(sub[3]));
     continue;
     }
 
-  /* Handle operations on a subsequent string */
+  /* Handle various operations on a subsequent string */
 
   if (*s == ':')
     {
@@ -2403,7 +2486,9 @@ terminating brace. */
 
 if (ket_ends && *s == 0)
   {
-  expand_string_message = "missing } at end of string";
+  expand_string_message = malformed_header?
+    "missing } at end of string - could be header name not terminated by colon" :
+    "missing } at end of string";
   goto EXPAND_FAILED;
   }
 
