@@ -403,7 +403,7 @@ Returns:          points to next character after "then"
 static char *
 read_condition(char *ptr, condition_block **cond, BOOL toplevel)
 {
-char buffer[256];
+char buffer[1024];
 BOOL testfor = TRUE;
 condition_block *current_parent = NULL;
 condition_block **current = cond;
@@ -1474,8 +1474,10 @@ switch (c->type)
     int from_count = 5;
 
     self = string_sprintf("%s@%s", deliver_localpart, deliver_domain);
-    self_from = rewrite_one(self, rewrite_from, NULL, FALSE, "");
-    self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, "");
+    self_from = rewrite_one(self, rewrite_from, NULL, FALSE, "",
+      global_rewrite_rules);
+    self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, "",
+      global_rewrite_rules);
     if (self_from == NULL) self_from = self;
     if (self_to == NULL) self_to = self;
 
@@ -1489,8 +1491,10 @@ switch (c->type)
         deliver_localpart,
         (deliver_localpart_suffix == NULL)? "" : deliver_localpart_suffix,
         deliver_domain);
-      psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, "");
-      psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, "");
+      psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, "",
+        global_rewrite_rules);
+      psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, "",
+        global_rewrite_rules);
       if (psself_from == NULL) self_from = psself;
       if (psself_to == NULL) self_to = psself;
       to_count += 2;
@@ -1520,7 +1524,7 @@ switch (c->type)
 
   case cond_errormsg:
   yield = message_id[0] != 0 &&
-    (sender_address == NULL || sender_address[0] == 0 || user_null_sender);
+    (sender_address == NULL || sender_address[0] == 0);
   break;
 
   /* Only FALSE if a message is actually being processed; TRUE for address
@@ -1801,7 +1805,9 @@ while (commands != NULL)
         char *ss = parse_extract_address(s, &error, &start, &end, &domain,
           FALSE);
         if (ss != NULL)
-          expargs[i] = rewrite? rewrite_address(ss, TRUE, FALSE) :
+          expargs[i] = rewrite?
+            rewrite_address(ss, TRUE, FALSE, global_rewrite_rules,
+              rewrite_existflags) :
             rewrite_address_qualify(ss, TRUE);
         else
           {
@@ -1853,7 +1859,7 @@ while (commands != NULL)
         (s != NULL)? s : "");
 
       /* Create the new address and add it to the chain, setting the
-      ignore_error flag if necessary, and the errors address, which can be
+      af_ignore_error flag if necessary, and the errors address, which can be
       set in a system filter and to the local address in user filters. Ensure
       that the addresses are placed in the MAIN storage pool. */
 
@@ -1861,7 +1867,7 @@ while (commands != NULL)
       addr = deliver_make_addr(expargs[0], TRUE);  /* TRUE => copy s */
       addr->errors_address = (s == NULL)?
         s : string_copy(s);                        /* Default is NULL */
-      addr->ignore_error = commands->noerror;
+      if (commands->noerror) setflag(addr, af_ignore_error);
       addr->next = *generated;
       *generated = addr;
       store_pool = pool_reset;
@@ -1896,14 +1902,14 @@ while (commands != NULL)
       if (s[0] != '/') s = string_sprintf("%s/%s", deliver_home, s);
 
       /* Create the new address and add it to the chain, setting the
-      pfr flag, the ignore_error flag if necessary, and the mode value.
+      af_pfr flag, the af_ignore_error flag if necessary, and the mode value.
       Ensure that the address is placed in the MAIN storage pool. */
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(s, TRUE);  /* TRUE => copy s */
-      addr->pfr = TRUE;
+      setflag(addr, af_pfr);
+      if (commands->noerror) setflag(addr, af_ignore_error);
       addr->mode = mode;
-      addr->ignore_error = commands->noerror;
       addr->next = *generated;
       *generated = addr;
       store_pool = pool_reset;
@@ -1927,17 +1933,16 @@ while (commands != NULL)
       if (s[0] != '|') s = string_sprintf("|%s", s);
 
       /* Create the new address and add it to the chain, setting the
-      ignore_error flag if necessary. Set the expand_pipe flag so that
+      af_ignore_error flag if necessary. Set the af_expand_pipe flag so that
       each command argument is expanded in the transport after the command
       has been split up into separate arguments. Ensure the address is
       placed in the MAIN storage pool */
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(s, TRUE);  /* TRUE => copy s */
-      addr->pfr = TRUE;
-      addr->ignore_error = commands->noerror;
+      setflag(addr, af_pfr|af_expand_pipe);
+      if (commands->noerror) setflag(addr, af_ignore_error);
       addr->next = *generated;
-      addr->expand_pipe = TRUE;
       *generated = addr;
 
       /* If there are any numeric variables in existence (e.g. after a regex
@@ -2163,7 +2168,7 @@ while (commands != NULL)
 
     case mail_command:
     case vacation_command:
-    if (return_path == NULL || return_path[0] == 0 || user_null_sender)
+    if (return_path == NULL || return_path[0] == 0)
       {
       if (filter_test != NULL)
         printf("%s command ignored because return_path is empty\n",
@@ -2270,8 +2275,8 @@ while (commands != NULL)
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(string_sprintf(">%s", to), FALSE);
-      addr->pfr = TRUE;
-      addr->ignore_error = commands->noerror;
+      setflag(addr, af_pfr);
+      if (commands->noerror) setflag(addr, af_ignore_error);
       addr->next = *generated;
       *generated = addr;
       addr->reply = store_get(sizeof(reply_item));
