@@ -16,6 +16,19 @@ extern void  cleanup_perl(void);
 extern char *init_perl(char *);
 #endif
 
+#ifdef SUPPORT_TLS
+extern BOOL  tls_client_start(int, host_item *, address_item *, char *,
+               char *, char *, char *, char *, int);
+extern void  tls_close(BOOL);
+extern int   tls_feof(FILE *);
+extern int   tls_ferror(FILE *);
+extern int   tls_getc(FILE *);
+extern int   tls_read(char *, size_t);
+extern BOOL  tls_server_start(void);
+extern int   tls_ungetc(int, FILE *);
+extern int   tls_write(const char *, size_t);
+#endif
+
 extern void  accept_add_recipient(char *, char *, int, int);
 extern void  accept_bomb_out(char *);
 extern BOOL  accept_check_fs(int);
@@ -67,14 +80,16 @@ extern void  direct_set_ugid(address_item *, ugid_block *);
 extern void  direct_init(void);
 extern void  direct_tidyup(void);
 extern BOOL  directory_make(char *, char *, int, BOOL);
-extern char *dns_address_from_rr(dns_record *);
-extern int   dns_expand(unsigned char *, char *, int);
+extern dns_address *dns_address_from_rr(dns_answer *, dns_record *);
 extern void  dns_init(BOOL, BOOL);
-extern int   dns_basic_lookup(char *, int);
-extern int   dns_lookup(char *, int, char **);
-extern dns_record *dns_next_rr(int);
+extern int   dns_basic_lookup(dns_answer *, char *, int);
+extern int   dns_lookup(dns_answer *, char *, int, char **);
+extern dns_record *dns_next_rr(dns_answer *, dns_scan *, int);
 extern char *dns_text_type(int);
 
+extern void  exim_exit(int);
+extern void  exim_setugid(uid_t, gid_t, char *);
+
 extern BOOL  expand_check_condition(char *, char *, char *);
 extern char *expand_getkeyed(char *, char *);
 extern char *expand_string(char *);
@@ -89,19 +104,20 @@ extern BOOL  filter_system_interpret(address_item **, BOOL *, int *, char **);
 extern void  header_add(int, char *, ...);
 extern BOOL  header_checkname(header_line *, char *, int);
 
-extern char *host_and_ident(void);
+extern char *host_and_ident(char *, char *);
 extern int   host_aton(char *, int *);
 extern void  host_build_hostlist(host_item **, char *, BOOL);
 extern void  host_build_sender_fullhost(void);
 extern int   host_check_rbl(char *, char *, char *, char **);
-extern BOOL  host_find_byname(host_item *, char **, BOOL);
-extern BOOL  host_find_bydns(host_item *, BOOL, BOOL, BOOL, BOOL, char **,
-               BOOL *);
+extern int   host_extract_port(char *);
+extern BOOL  host_find_byname(host_item *, char *, char **, BOOL);
+extern BOOL  host_find_bydns(host_item *, char *, BOOL, BOOL, BOOL, BOOL,
+               char **, BOOL *);
 extern ip_address_item *host_find_interfaces(void);
 extern void  host_mask(int, int *, int);
 extern BOOL  host_name_lookup(void);
 extern int   host_nmtoa(int, int *, int, char *);
-extern char *host_ntoa(int, const void *, char *);
+extern char *host_ntoa(int, const void *, char *, int *);
 extern int   host_scan_for_local_hosts(host_item *, host_item *, BOOL *);
 extern int   host_self_action(address_item *, host_item *, int, BOOL, char *);
 
@@ -113,8 +129,8 @@ extern int   match_exists(char *);
 extern BOOL  match_isinlist(char *, char **, BOOL, BOOL, char **);
 extern BOOL  match_check_string(char *, char *, int, BOOL, BOOL, char **);
 extern int   match_sender(char *);
-extern void  md5_end(md5 *, const unsigned char *, int, unsigned char *);
-extern void  md5_mid(md5 *, const unsigned char *);
+extern void  md5_end(md5 *, const uschar *, int, uschar *);
+extern void  md5_mid(md5 *, const uschar *);
 extern void  md5_start(md5 *);
 extern char *moan_check_errorcopy(char *);
 extern BOOL  moan_send_message(char *, int, error_block *, header_line *,
@@ -125,6 +141,7 @@ extern void  moan_smtp_batch(char *, char *, ...);
 extern void  moan_tell_someone(char *, address_item *, char *, char *, ...);
 extern BOOL  moan_to_sender(int, error_block *, header_line *, FILE *, BOOL);
 
+extern ip_address_item *os_common_find_running_interfaces(void);
 extern int   os_getloadavg(void);
 extern void  os_restarting_signal(int, void (*)(int));
 extern void  os_non_restarting_signal(int, void (*)(int));
@@ -137,6 +154,7 @@ extern int   parse_extract_addresses(char *, char *, address_item **, char **,
 extern char *parse_find_address_end(char *, BOOL);
 extern char *parse_find_at(char *);
 extern char *parse_fix_phrase(char *);
+extern char *parse_qp(char *);
 
 extern BOOL  queue_action(char *, int, char **, int, int);
 extern void  queue_check_only(void);
@@ -158,13 +176,15 @@ extern void  readconf_rewrites(void);
 extern BOOL  regex_match_and_setup(pcre *, char *, int, int);
 extern pcre *regex_must_compile(char *, BOOL, BOOL);
 extern void  retry_add_item(address_item *, char *, BOOL, char *, BOOL);
-extern BOOL  retry_check_address(char *, host_item *, BOOL, char **, char **);
+extern BOOL  retry_check_address(char *, host_item *, char *, BOOL, char **,
+               char **);
 extern retry_config *retry_find_config(char *, char *, int, int);
 extern void  retry_update(address_item **, address_item **, address_item **);
-extern char *rewrite_address(char *, BOOL, BOOL);
+extern char *rewrite_address(char *, BOOL, BOOL, rewrite_rule *, int);
 extern char *rewrite_address_qualify(char *, BOOL);
-extern header_line *rewrite_header(header_line *, char *, char *);
-extern char *rewrite_one(char *, int, BOOL *, BOOL, char *);
+extern header_line *rewrite_header(header_line *, char *, char *,
+               rewrite_rule *, int, BOOL);
+extern char *rewrite_one(char *, int, BOOL *, BOOL, char *, rewrite_rule *);
 extern void  rewrite_test(char *);
 extern int   route_address(address_item *, address_item **, address_item **,
                address_item **, int);
@@ -179,11 +199,11 @@ extern void  route_tidyup(void);
 extern void  route_unseen(char *, address_item *, address_item **,
                address_item **, address_item **);
 
-extern int   search_check_file(int, char *, int, int *, int *, char *,
+extern int   search_check_file(int, char *, int, uid_t *, gid_t *, char *,
                char **);
 extern char *search_find(void *, char *, char *, int, int *, char **);
 extern int   search_findtype(char *, char **);
-extern void *search_open(char *, int, int, int *, int *, char **);
+extern void *search_open(char *, int, int, uid_t *, gid_t *, char **);
 extern void  search_tidyup(void);
 extern void  set_process_info(char *, ...);
 extern void  sigalrm_handler(int);
@@ -193,12 +213,12 @@ extern int   smtp_feof(FILE *);
 extern int   smtp_ferror(FILE *);
 extern int   smtp_getc(FILE *);
 extern void  smtp_printf(char *, ...);
-extern BOOL  smtp_read_response(int, char *, int, int, int);
+extern BOOL  smtp_read_response(smtp_inblock *, uschar *, int, int, int);
 extern void  smtp_send_prohibition_message(int, char *);
 extern int   smtp_setup_msg(void);
 extern BOOL  smtp_start_session(void);
 extern int   smtp_ungetc(int, FILE *);
-extern BOOL  smtp_write_command(int, char *, ...);
+extern int   smtp_write_command(smtp_outblock *, BOOL, char *, ...);
 extern BOOL  spool_move_message(char *, char *, char *, char *);
 extern BOOL  spool_open_datafile(char *);
 extern int   spool_open_temp(char *);
@@ -244,7 +264,7 @@ extern void  transport_update_waiting(host_item *, char *);
 extern BOOL  transport_write_block(int, char *, int);
 extern BOOL  transport_write_string(int, char *, ...);
 extern BOOL  transport_write_message(address_item *, int, int, int, char *,
-               char *, char *, char *);
+               char *, char *, char *, rewrite_rule *, int);
 extern void  tree_add_duplicate(char *, BOOL);
 extern void  tree_add_nonrecipient(char *, BOOL);
 extern void  tree_add_unusable(host_item *);
