@@ -38,6 +38,9 @@ BOOL   opt_perl_started       = FALSE;
 #ifdef HAVE_AUTH
 BOOL   auth_always_advertise  = TRUE;
 char  *auth_hosts             = NULL;
+#ifdef SUPPORT_TLS
+char  *auth_over_tls_hosts    = NULL;
+#endif
 char  *host_auth_accept_relay = NULL;
 pcre  *regex_AUTH             = NULL;
 BOOL   smtp_authenticated     = FALSE;
@@ -73,6 +76,29 @@ char *pgsql_servers	      = NULL;
 BOOL   move_frozen_messages   = FALSE;
 #endif
 
+/* This variable is outside the #ifdef because it keeps the code less cluttered
+in several places (e.g. during logging) if we can always refer to it. */
+
+BOOL   tls_active             = -1;
+
+#ifdef SUPPORT_TLS
+pcre  *regex_STARTTLS         = NULL;
+char  *tls_advertise_hosts    = NULL;
+char  *tls_certificate        = NULL;
+char  *tls_cipher             = NULL;
+char  *tls_dhparam            = NULL;
+char  *tls_hosts              = NULL;
+char  *tls_host_accept_relay  = NULL;
+BOOL   tls_log_cipher         = TRUE;
+BOOL   tls_log_peerdn         = FALSE;
+BOOL   tls_offered            = FALSE;
+char  *tls_peerdn             = NULL;
+char  *tls_privatekey         = NULL;
+char  *tls_verify_certificates= NULL;
+char  *tls_verify_ciphers     = NULL;
+char  *tls_verify_hosts       = NULL;
+#endif
+
 
 #ifdef SUPPORT_DSN
 /* This partial attempt at doing DSN was abandoned. This obsolete code is
@@ -233,10 +259,16 @@ director_instance director_defaults = {
    FALSE,                     /* suffix_optional */
    FALSE,                     /* fail_verify_sender */
    FALSE,                     /* fail_verify_recipient */
-   TRUE                       /* expn */
+   TRUE,                      /* expn */
+   TRUE                       /* one_continue */
 };
 
 char  *dns_again_means_nonexist = NULL;
+#if HAVE_IPV6
+BOOL   dns_ipv4_lookup        = FALSE;
+#else
+BOOL   dns_ipv4_lookup        = TRUE;
+#endif
 int    dns_retrans            = 0;
 int    dns_retry              = 0;
 char  *domain_data            = NULL;
@@ -297,6 +329,7 @@ BOOL   freeze_tell_mailmaster = FALSE;
 
 char  *gecos_name             = NULL;
 char  *gecos_pattern          = NULL;
+rewrite_rule *global_rewrite_rules = NULL;
 
 BOOL   have_seteuid           = HAVE_SETEUID || HAVE_SETRESUID;
 BOOL   header_changed         = FALSE;
@@ -376,6 +409,7 @@ BOOL   log_arguments          = FALSE;
 char  *log_buffer             = NULL;
 char  *log_file_path          = LOG_FILE_PATH
 			  "\0<--------------Space to patch log_file_path->";
+BOOL   log_incoming_port      = FALSE;
 BOOL   log_ip_options         = TRUE;
 int    log_level              = 5;
 int    log_queue_run_level    = 0;
@@ -388,6 +422,7 @@ BOOL   log_smtp_connections   = FALSE;
 BOOL   log_smtp_syntax_errors = FALSE;
 FILE  *log_stderr             = NULL;
 BOOL   log_subject            = FALSE;
+char  *login_sender_address   = NULL;
 char  *lookup_key             = NULL;
 int    lookup_open_max        = 25;
 char  *lookup_value           = NULL;
@@ -434,7 +469,8 @@ BOOL   nobody_uid_set         = FALSE;
 
 uid_t  original_euid;
 gid_t  originator_gid;
-char  *originator_login;
+char  *originator_login       = NULL;
+char  *originator_name        = NULL;
 uid_t  originator_uid;
 
 BOOL   parse_allow_group      = FALSE;
@@ -471,6 +507,10 @@ BOOL   queue_running          = FALSE;
 BOOL   queue_smtp             = FALSE;
 char  *queue_smtp_domains     = NULL;
 
+unsigned int random_seed      = 0;
+char  *raw_sender             = NULL;
+char **raw_recipients         = NULL;
+char  *rbl_domain             = NULL;
 char  *rbl_domains            = NULL;
 char  *rbl_hosts              = "*";
 header_line *rbl_header       = NULL;
@@ -480,12 +520,9 @@ BOOL   rbl_log_headers        = FALSE;
 char  *rbl_msg_buffer         = NULL;
 BOOL   rbl_reject_recipients  = TRUE;
 BOOL   rbl_warn_header        = TRUE;
-char  *raw_sender             = NULL;
-char **raw_recipients         = NULL;
 
 gid_t  real_gid;
 uid_t  real_uid;
-char  *real_sender_address;
 BOOL   really_exim            = TRUE;
 int    received_count         = 0;
 char  *received_for           = NULL;
@@ -495,12 +532,15 @@ date will be automatically added on the end. */
 
 char  *received_header_text   =
     "Received: "
-    "${if def:sender_rcvhost {from ${sender_rcvhost}\n\t}"
-    "{${if def:sender_ident {from ${sender_ident} }}"
-    "${if def:sender_helo_name {(helo=${sender_helo_name})\n\t}}}}"
+    "${if def:sender_rcvhost {from $sender_rcvhost\n\t}"
+    "{${if def:sender_ident {from $sender_ident }}"
+    "${if def:sender_helo_name {(helo=$sender_helo_name)\n\t}}}}"
     "by ${primary_hostname} "
-    "${if def:received_protocol {with ${received_protocol}}} "
-    "(Exim ${version_number} #${compile_number})\n\t"
+    "${if def:received_protocol {with $received_protocol}} "
+    #ifdef SUPPORT_TLS
+    "${if def:tls_cipher {($tls_cipher)\n\t}}"
+    #endif
+    "(Exim $version_number #$compile_number)\n\t"
     "id ${message_id}"
     "${if def:received_for {\n\tfor $received_for}}"
     "\0<---------------Space to patch received_header_text->";
@@ -526,6 +566,7 @@ BOOL   refuse_ip_options      = TRUE;
 char **regcomp_error_pointer  = NULL;
 pcre  *regex_check_dns_names  = NULL;
 pcre  *regex_From             = NULL;
+pcre  *regex_PIPELINING       = NULL;
 pcre  *regex_SIZE             = NULL;
 pcre  *regex_ismsgid          = NULL;
 char   *relay_domains         = NULL;
@@ -541,7 +582,6 @@ char  *return_path            = NULL;
 BOOL   return_path_remove     = TRUE;
 int    return_size_limit      = 100*1024;
 int    rewrite_existflags     = 0;
-rewrite_rule *rewrite_rules   = NULL;
 char  *rfc1413_hosts          = "*";
 int    rfc1413_query_timeout  = 30;
 BOOL   rfc821_domains         = FALSE;
@@ -587,9 +627,11 @@ router_instance router_defaults = {
    self_freeze,               /* self_code */
    FALSE,                     /* self_rewrite */
    TRUE,                      /* sender_rewrite */
-   NULL                       /* translate_ip_address */
+   NULL,                      /* translate_ip_address */
+   NULL                       /* ignore_target_hosts */
 };
 
+BOOL   running_in_test_harness = FALSE;
 char  *running_status         = ">>>running<<<";
 
 BOOL   search_find_defer      = FALSE;
@@ -607,6 +649,7 @@ char **sender_host_aliases    = NULL;
 char  *sender_host_address    = NULL;
 char  *sender_host_authenticated = NULL;
 char  *sender_host_name       = NULL;
+int    sender_host_port       = 0;
 BOOL   sender_host_notsocket  = FALSE;
 BOOL   sender_host_unknown    = FALSE;
 char  *sender_ident           = NULL;
@@ -614,12 +657,17 @@ BOOL   sender_local           = FALSE;
 char  *sender_rcvhost         = NULL;
 char  *sender_reject          = NULL;
 char  *sender_reject_recipients = NULL;
+BOOL   sender_set_untrusted   = FALSE;
 BOOL   sender_try_verify      = FALSE;
 char  *sender_unqualified_hosts = NULL;
 BOOL   sender_verify          = FALSE;
 BOOL   sender_verify_batch    = FALSE;
+char  *sender_verify_callback_domains = NULL;
+char  *sender_verify_callback_error = NULL;
+int    sender_verify_callback_timeout = 30;
 BOOL   sender_verify_fixup    = FALSE;
 char  *sender_verify_hosts    = "*";
+char  *sender_verify_hosts_callback = NULL;
 BOOL   sender_verify_reject   = TRUE;
 int    sender_verify_max_retry_rate = 12;
 volatile BOOL sigalrm_seen    = FALSE;
@@ -650,6 +698,7 @@ BOOL   smtp_etrn_serialize    = TRUE;
 char  *smtp_expn_hosts        = NULL;
 char  *smtp_read_error        = "";
 char  *smtp_reserve_hosts     = NULL;
+BOOL   smtp_use_pipelining    = FALSE;
 BOOL   smtp_use_size          = FALSE;
 BOOL   smtp_verify            = FALSE;
 BOOL   split_spool_directory  = FALSE;
@@ -662,6 +711,7 @@ int    store_pool             = POOL_MAIN;
 BOOL   strip_excess_angle_brackets = FALSE;
 BOOL   strip_trailing_dot     = FALSE;
 
+int    timeout_frozen_after   = 0;
 BOOL   timestamps_utc         = FALSE;
 
 transport_instance *transports = NULL;
@@ -704,6 +754,9 @@ transport_instance transport_defaults = {
    NULL,                     /* remove_headers */
    NULL,                     /* return_path */
    NULL,                     /* debug_string */
+   NULL,                     /* headers_rewrite */
+   NULL,                     /* rewrite_rules */
+   0,                        /* rewrite_existflags */
    FALSE,                    /* body_only */
    FALSE,                    /* delivery_date_add */
    FALSE,                    /* envelope_to_add */
@@ -723,6 +776,7 @@ int    transport_count;
 char **transport_filter_argv  = NULL;
 int    transport_write_timeout= 0;
 
+tree_node *tree_dns_fails     = NULL;
 tree_node *tree_duplicates    = NULL;
 tree_node *tree_nonrecipients = NULL;
 tree_node *tree_unusable      = NULL;
@@ -730,12 +784,11 @@ tree_node *tree_unusable      = NULL;
 BOOL   trusted_caller         = FALSE;
 gid_t *trusted_groups         = NULL;
 uid_t *trusted_users          = NULL;
+char  *timezone_string        = TIMEZONE_DEFAULT;
 
 char  *unknown_login          = NULL;
 char  *unknown_username       = NULL;
-char  *user_login             = NULL;
-char  *user_name              = NULL;
-BOOL   user_null_sender       = FALSE;
+BOOL   untrusted_set_sender   = FALSE;
 
 /* A regex for matching a "From_" line in an incoming message, in the form
 
