@@ -24,6 +24,9 @@ extern BOOL   opt_perl_started;       /* Set once interpreter started */
 #ifdef HAVE_AUTH
 extern BOOL   auth_always_advertise;  /* If FALSE, advertise only when needed */
 extern char  *auth_hosts;             /* These must authenticate */
+#ifdef SUPPORT_TLS
+extern char  *auth_over_tls_hosts;    /* These must use TLS for AUTH */
+#endif
 extern char  *host_auth_accept_relay; /* Relay permitted if authenticated */
 extern pcre  *regex_AUTH;             /* For recognizing AUTH settings */
 extern BOOL   smtp_authenticated;     /* Sending client has authenticated */
@@ -45,6 +48,29 @@ extern char  *pgsql_servers;          /* List of servers and connect info */
 extern BOOL   move_frozen_messages;   /* Get them out of the normal directory */
 #endif
 
+/* This variable is outside the #ifdef because it keeps the code less cluttered
+in several places (e.g. during logging) if we can always refer to it. */
+
+extern int    tls_active;             /* fd/socket when in a TLS session */
+
+#ifdef SUPPORT_TLS
+extern pcre  *regex_STARTTLS;         /* For recognizing STARTTLS settings */
+extern char  *tls_advertise_hosts;    /* host for which TLS is advertised */
+extern char  *tls_certificate;        /* Certificate file */
+extern char  *tls_cipher;             /* Cipher used */
+extern char  *tls_dhparam;            /* DH param file */
+extern char  *tls_hosts;              /* hosts which must use TLS */
+extern char  *tls_host_accept_relay;  /* hosts which may relay if using TLS */
+extern BOOL   tls_log_cipher;         /* Request cipher logging */
+extern BOOL   tls_log_peerdn;         /* Request peerdn logging */
+extern BOOL   tls_offered;            /* Server offered TLS */
+extern char  *tls_peerdn;             /* DN from peer */
+extern char  *tls_privatekey;         /* Private key file */
+extern char  *tls_verify_certificates;/* Path for certificates to check */
+extern char  *tls_verify_ciphers;     /* Ciphers to allow */
+extern char  *tls_verify_hosts;       /* host list for client verification */
+#endif
+
 
 #ifdef SUPPORT_DSN
 /* This partial attempt at doing DSN was abandoned. This obsolete code is
@@ -155,6 +181,7 @@ extern director_info directors_available[]; /* Vector of available directors */
 extern director_instance *directors;  /* Chain of instantiated directors */
 extern director_instance director_defaults; /* Default values */
 extern char  *dns_again_means_nonexist; /* Domains that are badly set up */
+extern BOOL   dns_ipv4_lookup;        /* If TRUE, don't look for AAAA & A6 */
 extern int    dns_retrans;            /* Retransmission time setting */
 extern int    dns_retry;              /* Number of retries */
 extern char  *domain_data;            /* From domain lookup */
@@ -200,6 +227,7 @@ extern BOOL   freeze_tell_mailmaster; /* Message on each freeze */
 
 extern char  *gecos_name;             /* To be expanded when pattern matches */
 extern char  *gecos_pattern;          /* Pattern to match */
+extern rewrite_rule *global_rewrite_rules;  /* Chain of rewriting rules */
 
 extern BOOL   have_seteuid;           /* TRUE if system has seteuid */
 extern BOOL   header_changed;         /* TRUE if header data has changed */
@@ -255,6 +283,7 @@ extern BOOL   log_all_parents;        /* As it says */
 extern BOOL   log_arguments;          /* Logs Exim's args */
 extern char  *log_buffer;             /* For constructing log entries */
 extern char  *log_file_path;          /* If unset, use default */
+extern BOOL   log_incoming_port;      /* For logging incoming port numbers */
 extern BOOL   log_ip_options;         /* For logging IP source routing */
 extern int    log_level;              /* Level of logging info */
 extern int    log_queue_run_level;    /* Log level for start/end */
@@ -267,6 +296,7 @@ extern BOOL   log_smtp_connections;   /* Verbose connection logging */
 extern BOOL   log_smtp_syntax_errors; /* As it says (for incoming smtp) */
 extern FILE  *log_stderr;             /* Copy of stderr for log use, or NULL */
 extern BOOL   log_subject;            /* As it says */
+extern char  *login_sender_address;   /* The actual sender address */
 extern char  *lookup_key;             /* For query expansion */
 extern lookup_info lookup_list[];     /* Vector of available lookups */
 extern int    lookup_open_max;        /* Max lookup files to cache */
@@ -326,7 +356,8 @@ extern int    optionlist_transports_size;
 
 extern uid_t  original_euid;          /* Original effective uid */
 extern gid_t  originator_gid;         /* Gid of whoever wrote spool file */
-extern char  *originator_login;       /* Login of ditto */
+extern char  *originator_login;       /* Login of same */
+extern char  *originator_name;        /* Full name of same */
 extern uid_t  originator_uid;         /* Uid of ditto */
 
 extern BOOL   parse_allow_group;      /* Allow group syntax */
@@ -363,8 +394,10 @@ extern int    queue_run_max;          /* Max queue runners */
 extern BOOL   queue_smtp;             /* Disable immediate STMP (-odqs)*/
 extern char  *queue_smtp_domains;     /* Ditto, for these domains */
 
+extern unsigned int random_seed;      /* Seed for random numbers */
 extern char  *raw_sender;             /* Before rewriting */
 extern char **raw_recipients;         /* Before rewriting */
+extern char  *rbl_domain;             /* RBL domain that caused rejection */
 extern char  *rbl_domains;            /* Realtime Block List domains */
 extern char  *rbl_hosts;              /* Hosts for which RBL is used */
 extern header_line *rbl_header;       /* RBL header to add */
@@ -376,7 +409,6 @@ extern BOOL   rbl_reject_recipients;  /* TRUE to reject recipients if found */
 extern BOOL   rbl_warn_header;        /* Add header if warning */
 extern gid_t  real_gid;               /* Real gid */
 extern uid_t  real_uid;               /* Real user running program */
-extern char  *real_sender_address;    /* Used in local SMTP processing */
 extern BOOL   really_exim;            /* FALSE in utilities */
 extern int    received_count;         /* Count of Received: headers */
 extern char  *received_for;           /* For "for" field */
@@ -401,6 +433,7 @@ extern BOOL   refuse_all_rcpts;       /* Flag for SMTP verifying */
 extern BOOL   refuse_ip_options;      /* Refuse source routing */
 extern pcre  *regex_check_dns_names;  /* For DNS name checking */
 extern pcre  *regex_From;             /* For recognizing "From_" lines */
+extern pcre  *regex_PIPELINING;       /* For recognizing PIPELINING */
 extern pcre  *regex_SIZE;             /* For recognizing SIZE settings */
 extern pcre  *regex_ismsgid;          /* Compiled r.e. for message it */
 extern char  *relay_domains;          /* Official relay domains */
@@ -416,7 +449,6 @@ extern char  *return_path;            /* Return path for a message */
 extern BOOL   return_path_remove;     /* Remove return-path headers */
 extern int    return_size_limit;      /* Limit messages returned to sender */
 extern int    rewrite_existflags;     /* Indicate which headers have rewrites */
-extern rewrite_rule *rewrite_rules;   /* Chain of rewriting rules */
 extern char  *rfc1413_hosts;          /* RFC hosts */
 extern int    rfc1413_query_timeout;  /* Timeout on RFC 1413 calls */
 extern BOOL   rfc821_domains;         /* If set, syntax is 821, not 822 */
@@ -425,6 +457,7 @@ extern char  *route_option;           /* Option string for some routers */
 extern router_info routers_available[]; /* Vector of available routers */
 extern router_instance *routers;      /* Chain of instantiated routers */
 extern router_instance router_defaults; /* Default values */
+extern BOOL   running_in_test_harness;/* TRUE when running_status is patched */
 extern char  *running_status;         /* Flag string for testing */
 
 extern BOOL   search_find_defer;      /* Set TRUE if lookup defers */
@@ -443,18 +476,24 @@ extern char **sender_host_aliases;    /* Points to list of alias names */
 extern char  *sender_host_authenticated; /* Name of authentication mechanism */
 extern char  *sender_host_name;       /* Host name from lookup, or "?" */
 extern BOOL   sender_host_notsocket;  /* Set for -bs and -bS */
+extern int    sender_host_port;       /* Port number of sender */
 extern BOOL   sender_host_unknown;    /* TRUE for -bs and -bS except inetd */
 extern char  *sender_ident;           /* Sender identity via RFC 1413 */
 extern BOOL   sender_local;           /* TRUE for local senders */
 extern char  *sender_rcvhost;         /* Host data for Received: */
 extern char  *sender_reject;          /* Spam filter list */
 extern char  *sender_reject_recipients; /* Ditto, reject RCPT TO */
+extern BOOL   sender_set_untrusted;   /* Sender set by untrusted caller */
 extern BOOL   sender_try_verify;      /* Accept if soft error while verifying */
 extern char  *sender_unqualified_hosts; /* Permitted unqualified senders */
 extern BOOL   sender_verify;          /* TRUE when verifying senders */
 extern BOOL   sender_verify_batch;    /* TRUE if batch SMTP included */
+extern char  *sender_verify_callback_domains; /* For SMTP callbacks */
+extern char  *sender_verify_callback_error;   /* ditto - holds error */
+extern int    sender_verify_callback_timeout; /* ditto */
 extern BOOL   sender_verify_fixup;    /* Fix broken senders from headers */
 extern char  *sender_verify_hosts;    /* Verification hosts */
+extern char  *sender_verify_hosts_callback; /* For SMTP callback */
 extern BOOL   sender_verify_reject;   /* Reject if verify fails */
 extern int    sender_verify_max_retry_rate; /* Max retry rate */
 extern volatile BOOL sigalrm_seen;    /* Flag for sigalrm_handler */
@@ -482,6 +521,7 @@ extern int    smtp_port;              /* Alternate SMTP port number */
 extern char  *smtp_read_error;        /* Message for SMTP input error */
 extern int    smtp_receive_timeout;   /* Applies to each received line */
 extern char  *smtp_reserve_hosts;     /* Hosts for reserved slots */
+extern BOOL   smtp_use_pipelining;    /* Global for passed connections */
 extern BOOL   smtp_use_size;          /* Global for passed connections */
 extern BOOL   smtp_verify;            /* TRUE if VRFY permitted */
 extern BOOL   split_spool_directory;  /* TRUE to use multiple subdirs */
@@ -491,6 +531,7 @@ extern int    store_pool;             /* Current pool number */
 extern BOOL   strip_excess_angle_brackets; /* Surrounding route-addrs */
 extern BOOL   strip_trailing_dot;     /* Remove dots at ends of domains */
 
+extern int    timeout_frozen_after;   /* Max time to keep frozen messages */
 extern BOOL   timestamps_utc;         /* Use UTC for all times */
 
 extern transport_instance *transport_message_filter_directory;
@@ -507,6 +548,7 @@ extern transport_instance transport_defaults; /* Default values */
 
 extern int    transport_write_timeout;/* Set to time out individual writes */
 
+extern tree_node *tree_dns_fails;     /* Tree of DNS lookup failures */
 extern tree_node *tree_duplicates;    /* Tree of duplicate addresses */
 extern tree_node *tree_nonrecipients; /* Tree of nonrecipient addresses */
 extern tree_node *tree_unusable;      /* Tree of unusable addresses */
@@ -514,12 +556,11 @@ extern tree_node *tree_unusable;      /* Tree of unusable addresses */
 extern BOOL   trusted_caller;         /* Caller is trusted */
 extern gid_t *trusted_groups;         /* List of trusted groups */
 extern uid_t *trusted_users;          /* List of trusted users */
+extern char  *timezone_string;        /* Required timezone setting */
 
 extern char  *unknown_login;          /* To use when login id unknown */
 extern char  *unknown_username;       /* Ditto */
-extern char  *user_login;             /* Login name of user */
-extern char  *user_name;              /* User full name */
-extern BOOL   user_null_sender;       /* TRUE if non-trusted supplied -f <> */
+extern BOOL   untrusted_set_sender;   /* Let untrusted users set sender */
 extern char  *uucp_from_pattern;      /* For recognizing "From " lines */
 extern char  *uucp_from_sender;       /* For building the sender */
 
