@@ -3,6 +3,7 @@
 **
 ** Author: Peter Eriksson <pen@lysator.liu.se>
 ** Fiddling: Pr Emanuelsson <pell@lysator.liu.se>
+** Also by: Philip Hazel <ph10@cus.cam.ac.uk>
 */
 
 #ifdef NeXT3
@@ -74,7 +75,12 @@ int id_parse __P7(ident_t *, id,
 	FD_ZERO(&rs);
 	FD_SET(id->fd, &rs);
 
-#if defined(__hpux) || defined(__H3050) || defined(__H3050R)
+/* Modern versions of HP-UX use the correct types for the arguments
+of select(). Older versions do not. Luckily older versions also define
+"hpux" whereas modern versions use only "__hpux" (to be more standard).
+Changed this test from __hpux to hpux on 2-Feb-99 (PH). */
+
+#if defined(hpux) || defined(__H3050) || defined(__H3050R)
 	if ((res = select(FD_SETSIZE, (int *) &rs, (int *)0, (int *)0, timeout)) < 0)
 #else
 	if ((res = select(FD_SETSIZE, &rs, (fd_set *)0, (fd_set *)0, timeout)) < 0)
@@ -198,7 +204,10 @@ int id_parse __P7(ident_t *, id,
 	else
 	    cp = id_strtok((char *)0, "\n\r", &c);
 
-	if (identifier)
+        if (!cp)         /* Added by PH to fix segfault */
+	    return -2;
+
+        if (identifier)
 	{
 	    *identifier = id_strdup(cp);
 	    if (*identifier == NULL)
