@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing log files. */
@@ -131,7 +131,7 @@ if (s1 != NULL)
   }
 if (accept_call_bombout) accept_bomb_out(s2);  /* does not return */
 if (smtp_input) smtp_closedown(s2);
-exit(EXIT_FAILURE);
+exim_exit(EXIT_FAILURE);
 }
 
 
@@ -185,8 +185,14 @@ size as buffer, and the text has been checked above to fit. */
 
 if (strcmp(name, "main") == 0) strcpy(mainlog_name, buffer);
 
+/* After a successful open, arrange for automatic closure on exec(). */
+
 *fd = open(buffer, O_APPEND|O_WRONLY, LOG_MODE);
-if (*fd >= 0) return;
+if (*fd >= 0)
+  {
+  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
+  return;
+  }
 
 /* Try creating the file */
 
@@ -203,7 +209,7 @@ if (*fd < 0 && errno == ENOENT && file_path[0] == 0)
   }
 
 /* Creation succeeded; change owner if we are currently root, and ensure
-the mode is as specified. */
+the mode is as specified. Set the close-on-exec flag. */
 
 euid = geteuid();
 
@@ -212,6 +218,7 @@ if (*fd >= 0)
   if (exim_uid_set && euid == root_uid)
     chown(buffer, exim_uid, exim_gid);
   chmod(buffer, LOG_MODE);
+  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
   return;
   }
 
@@ -328,7 +335,7 @@ if (log_buffer == NULL)
   if (log_buffer == NULL)
     {
     fprintf(stderr, "exim: failed to get store for log buffer\n");
-    exit(EXIT_FAILURE);
+    exim_exit(EXIT_FAILURE);
     }
   }
 
@@ -343,10 +350,11 @@ if (!path_inspected)
   BOOL multiple = FALSE;
   if (log_file_path[0] != 0)
     {
+    int sep = ':';              /* Fixed separator - outside use */
     char *s;
     char *ss = log_file_path;
     logging_mode = 0;
-    while ((s = string_nextinlist(&ss,':',log_buffer,LOG_BUFFER_SIZE)) != NULL)
+    while ((s = string_nextinlist(&ss,&sep,log_buffer,LOG_BUFFER_SIZE)) != NULL)
       {
       if (strcmp(s, "syslog") == 0)
         logging_mode |= LOG_MODE_SYSLOG;
@@ -462,19 +470,19 @@ sprintf(ptr, "\n");
 while(*ptr) ptr++;
 
 /* Handle loggable errors when running a utility, or when address testing.
-Write to log_stderr unless debugging (when it will already have been written).
-*/
+Write to log_stderr unless debugging (when it will already have been written),
+or unless there is no log_stderr (expn called from daemon, for example). */
 
 if (!really_exim || testing)
   {
-  if (debug_level <= 0)
+  if (debug_level <= 0 && log_stderr != NULL)
     {
     if (host_checking)
       fprintf(log_stderr, "LOG: %s", log_buffer + 20);  /* no timestamp */
     else
       fprintf(log_stderr, "%s", log_buffer);
     }
-  if ((flags & LOG_PANIC_DIE) == LOG_PANIC_DIE) exit(EXIT_FAILURE);
+  if ((flags & LOG_PANIC_DIE) == LOG_PANIC_DIE) exim_exit(EXIT_FAILURE);
   return;
   }
 
