@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -36,19 +36,19 @@ return yield;
 *************************************************/
 
 /* This needs to know more about the underlying files than is good for it!
-We need to know what the real file names are in order to check the owners
-and modes. If USE_DB is set, we know it is Berkeley DB, which an unmodified
-file name. Otherwise, for safety, we have to check for x.db or x.dir and
-x.pag. */
+We need to know what the real file names are in order to check the owners and
+modes. If USE_DB is set, we know it is Berkeley DB, which uses an unmodified
+file name. If USE_TDB is set, we know it is tdb, which does the same.
+Otherwise, for safety, we have to check for x.db or x.dir and x.pag. */
 
 BOOL
-dbmdb_check(void *handle, char *filename, int modemask, int *owners,
-  int *owngroups, char **errmsg)
+dbmdb_check(void *handle, char *filename, int modemask, uid_t *owners,
+  gid_t *owngroups, char **errmsg)
 {
 int rc;
 handle = handle;    /* Keep picky compilers happy */
 
-#ifdef USE_DB
+#if defined(USE_DB) || defined(USE_TDB)
 rc = search_check_file(-1, filename, modemask, owners, owngroups,
   "dbm", errmsg);
 #else
@@ -94,16 +94,15 @@ EXIM_DATUM key, data;
 filename = filename;    /* Keep picky compilers happy */
 errmsg = errmsg;
 
-#ifdef EXIM_CLEAR_DATUMSTRUCT
-memset(&key, 0, sizeof(key));
-memset(&data, 0, sizeof(data));
-#endif
-
+EXIM_DATUM_INIT(key);               /* Some DBM libraries require datums to */
+EXIM_DATUM_INIT(data);              /* be cleared before use. */
 EXIM_DATUM_DATA(key) = keystring;
 EXIM_DATUM_SIZE(key) = length + 1;
+
 if (EXIM_DBGET(d, key, data))
   {
   *result = string_copyn(EXIM_DATUM_DATA(data), EXIM_DATUM_SIZE(data));
+  EXIM_DATUM_FREE(data);            /* Some DBM libraries need a free() call */
   return OK;
   }
 return FAIL;
