@@ -2,11 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Many thanks to Stuart Lynne for contributing the original code for this
-driver. */
+driver. Further modification from michael@freenet-ag.de, Brian Candler,
+and Barry Pederson. */
 
 
 #include "../exim.h"
@@ -32,19 +33,52 @@ static void dummy(int x) { dummy(x-1); }
 #include <ldap.h>
 
 
-/* Annoyingly, the different LDAP libraries handle errors in different ways.
-There doesn't seem to be an automatic way of distinguishing between them. Check
-if the user set LDAP_LIB_TYPE (which causes a specific macro to be set), and if
-not set, use the original heuristic that distinguishes UMich LDAP from the rest
-and guess Netscape if not UMich. */
+/* For Exim release 3.20 the behaviour with regard to multiple attributes in
+responses was changed. Instead of comma-separated fields, space separated
+fields, quoted if necessary, with attribute names are used. The old behaviour
+can be re-instated by
 
-#ifndef LDAP_LIB_DEFINED
-  #ifdef LDAP_OPT_SIZELIMIT        /* UMich does not have this */
-  #define LDAP_NETSCAPE
-  #endif
+#define OLD_LDAP_BEHAVIOUR
+
+This should only be used if you really need it as a quick fix.
+*/
+
+
+/* Annoyingly, the different LDAP libraries handle errors in different ways,
+and some other things too. There doesn't seem to be an automatic way of
+distinguishing between them. Local/Makefile should contain a setting of
+LDAP_LIB_TYPE, which in turn causes appropriate macros to be defined for the
+different kinds. Those that matter are:
+
+LDAP_LIB_NETSCAPE
+LDAP_LIB_SOLARIS   with synonym LDAP_LIB_SOLARIS7
+LDAP_LIB_OPENLDAP2
+
+These others may be defined, but are in fact the default, so are not tested:
+
+LDAP_LIB_UMICHIGAN
+LDAP_LIB_OPENLDAP1
+*/
+
+#if defined(LDAP_LIB_SOLARIS7) && ! defined(LDAP_LIB_SOLARIS)
+#define LDAP_LIB_SOLARIS
 #endif
 
 
+/* Just in case LDAP_NO_LIMIT is not defined by some of these libraries. */
+
+#ifndef LDAP_NO_LIMIT
+#define LDAP_NO_LIMIT 0
+#endif
+
+
+/* Three types of LDAP search are implemented */
+
+#define SEARCH_LDAP_MULTIPLE 0       /* Get attributes from multiple entries */
+#define SEARCH_LDAP_SINGLE 1         /* Get attributes from one entry only */
+#define SEARCH_LDAP_DN 2             /* Get DN from one entry */
+
+
 /* Structure and anchor for caching connections. */
 
 typedef struct ldap_connection {
@@ -66,22 +100,23 @@ static LDAP_CONNECTION *ldap_connections = NULL;
 *************************************************/
 
 /* This is the function that actually does the work. It is called (indirectly
-via control_ldap_search) from both eldap_find() and eldapm_find(), with a
-difference in the "single" argument.
+via control_ldap_search) from eldap_find(), eldapdn_fing(), and eldapm_find(),
+with a difference in the "single" argument.
 
 Arguments:
   ldap_url      the URL to be looked up
   server        server host name, when URL contains none
   s_port        server port, used when URL contains no name
-  single        TRUE if a maximum of 1 result value is permitted
-                FALSE otherwise
+  search_type   SEARCH_LDAP_MULTIPLE allows values from multiple entries
+                SEARCH_LDAP_SINGLE allows values from one entry only
+                SEARCH_LDAP_DN gets the DN from one entry
   res           set to point at the result
   errmsg        set to point a message if result is not OK
   defer_break   set TRUE if no more servers to be tried after a DEFER
   user          user name for authentication, or NULL
   password      password for authentication, or NULL
-  sizelimit     max number of entries returned, or -1 for no limit
-  timelimit     max time to wait, or -1 for default
+  sizelimit     max number of entries returned, or 0 for no limit
+  timelimit     max time to wait, or 0 for no limit
 
 Returns:        OK or FAIL or DEFER
                 FAIL is given only if a lookup was performed successfully, but
@@ -89,16 +124,18 @@ Returns:        OK or FAIL or DEFER
 */
 
 static int
-perform_ldap_search(char *ldap_url, char *server, int s_port, BOOL single,
+perform_ldap_search(char *ldap_url, char *server, int s_port, int search_type,
   char **res, char **errmsg, BOOL *defer_break, char *user, char *password,
   int sizelimit, int timelimit)
 {
-LDAPURLDesc  *ludp;
+LDAPURLDesc  *ludp = NULL;
 LDAPMessage  *result;
-LDAPMessage  *e;
 BerElement   *ber;
 LDAP_CONNECTION *lcp;
 
+struct timeval timeout;
+struct timeval *timeoutptr = NULL;
+
 char  *attr;
 char  *data = NULL;
 char  *host;
@@ -106,16 +143,22 @@ char  **attrp;
 char  **values;
 char  **firstval;
 int   attr_count = 0;
+int   error_yield = DEFER;
+int   msgid;
 int   rc;
 int   port;
 int   ptr = 0;
+int   rescount = 0;
 int   size = 0;
 BOOL  add_newline = FALSE;
 BOOL  attribute_found = FALSE;
 
 DEBUG(9)
-  debug_printf("perform_ldap_search: ldap URL =\"%s\" server=%s port=%d\n",
-    ldap_url, server, s_port);
+  debug_printf("perform_ldap_search: ldap%s URL =\"%s\" server=%s port=%d "
+    "sizelimit=%d timelimit=%d\n",
+    (search_type == SEARCH_LDAP_MULTIPLE)? "m" :
+    (search_type == SEARCH_LDAP_DN)? "dn" : "",
+    ldap_url, server, s_port, sizelimit, timelimit);
 
 /* Check if LDAP thinks the URL is a valid LDAP URL */
 
@@ -123,27 +166,23 @@ if (!ldap_is_ldap_url(ldap_url))
   {
   *errmsg = string_sprintf("ldap_is_ldap_url: not an LDAP url \"%s\"\n",
     ldap_url);
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  *defer_break = TRUE;
-  return DEFER;
+  goto RETURN_ERROR_BREAK;
   }
 
-/* Parse the URL to get host and port, too bad there isn't an
-ldap_url_[init|open] */
+/* Parse the URL */
 
 if ((rc = ldap_url_parse(ldap_url, &ludp)) != 0)
   {
   *errmsg = string_sprintf("ldap_url_parse: (error %d) parsing \"%s\"\n", rc,
     ldap_url);
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  *defer_break = TRUE;
-  return DEFER;
+  goto RETURN_ERROR_BREAK;
   }
 
 /* If the host name is empty, take it from the separate argument, if
-one is given. */
+one is given. OpenLDAP 2.0.6 sets an unset hostname to "" rather than
+empty, but expects NULL later to mean "default", annoyingly. */
 
-if (ludp->lud_host == NULL && server != NULL)
+if ((ludp->lud_host == NULL || ludp->lud_host[0] == 0) && server != NULL)
   {
   host = server;
   port = s_port;
@@ -151,6 +190,7 @@ if (ludp->lud_host == NULL && server != NULL)
 else
   {
   host = ludp->lud_host;
+  if (host[0] == 0) host = NULL;
   port = ludp->lud_port;
   }
 
@@ -182,12 +222,9 @@ if (lcp == NULL)
   LDAP *ld = ldap_open(host, (port != 0)? port : LDAP_PORT);
   if (ld == NULL)
     {
-    ldap_free_urldesc(ludp);
     *errmsg = string_sprintf("failed to open connection to LDAP server %s:%d "
       "- %s", host, port, strerror(errno));
-    DEBUG(9) debug_printf("%s\n", *errmsg);
-    *defer_break = FALSE;
-    return DEFER;
+    goto RETURN_ERROR;
     }
 
   DEBUG(9) debug_printf("Opened connection to LDAP server %s:%d\n", host, port);
@@ -209,10 +246,6 @@ else
     host, port);
   }
 
-/* Finished with the broken-down URL */
-
-ldap_free_urldesc(ludp);
-
 /* Bind with the user/password supplied, or an anonymous bind if these values
 are NULL, unless a cached connection is already bound with the same values. */
 
@@ -231,9 +264,7 @@ if (!lcp->bound ||
     {
     *errmsg = string_sprintf("failed to bind the LDAP connection to server "
       "%s:%d - LDAP error %d: %s", host, port, rc, ldap_err2string(rc));
-    DEBUG(9) debug_printf("%s\n", *errmsg);
-    *defer_break = FALSE;
-    return DEFER;
+    goto RETURN_ERROR;
     }
   lcp->bound = TRUE;
   lcp->user = (user == NULL)? NULL : string_copy_malloc(user);
@@ -243,72 +274,25 @@ if (!lcp->bound ||
 /* Before doing the search, set the time and size limits (if given). Here again
 the different implementations of LDAP have chosen to do things differently. */
 
-#if defined LDAP_LIB_NETSCAPE
-if (sizelimit > 0)
-  ldap_set_option(lcp->ld, LDAP_OPT_SIZELIMIT, (void *)&sizelimit);
-if (timelimit > 0)
-  ldap_set_option(lcp->ld, LDAP_OPT_TIMELIMIT, (void *)&timelimit);
+#if defined(LDAP_OPT_SIZELIMIT)
+ldap_set_option(lcp->ld, LDAP_OPT_SIZELIMIT, (void *)&sizelimit);
+ldap_set_option(lcp->ld, LDAP_OPT_TIMELIMIT, (void *)&timelimit);
 #else
-if (sizelimit > 0) lcp->ld->ld_sizelimit = sizelimit;
-if (timelimit > 0) lcp->ld->ld_timelimit = timelimit;
+lcp->ld->ld_sizelimit = sizelimit;
+lcp->ld->ld_timelimit = timelimit;
 #endif
 
-/* Use the LDAP URL search function; no need to do our own parsing. If it
-succeeds, wait for results - this will time out if no results are available
-in a reasonable time. */
+/* Start the search on the server. */
 
-if (ldap_url_search(lcp->ld, ldap_url, 0) == -1 ||
-    (rc = ldap_result(lcp->ld, LDAP_RES_ANY, 1, NULL, &result)) == -1)
+DEBUG(9) debug_printf("Start search\n");
+
+msgid = ldap_search(lcp->ld, ludp->lud_dn, ludp->lud_scope, ludp->lud_filter,
+  ludp->lud_attrs, 0);
+
+if (msgid == -1)
   {
-  char *matched, *error;
-
-  /* Annoyingly, the different implementations of LDAP have gone for different
-  methods of handling error codes and generating error messages. */
-
-  #if defined LDAP_LIB_SOLARIS7     /* Solaris 7 LDAP */
-    *errmsg = string_sprintf("ldap search failed: %s",
-      ldap_err2string(ldap_result2error(lcp->ld, result, 0)));
-  #elif defined LDAP_LIB_NETSCAPE   /* Netscape SDK */
-    (void)ldap_get_lderrno(lcp->ld, &matched, &error);
-    *errmsg = string_sprintf("ldap search failed: %s (%s)", error, matched);
-  #else                             /* UMich LDAP */
-    matched = lcp->ld->ld_matched;
-    error = lcp->ld->ld_error;
-    *errmsg = string_sprintf("ldap search failed: %s (%s)", error, matched);
-  #endif
-
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  *defer_break = TRUE;
-  return DEFER;
-  }
-
-if (rc == 0)
-  {
-  *errmsg = string_sprintf("LDAP search timed out");
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  *defer_break = FALSE;
-  return DEFER;
-  }
-
-/* Check if we have too many entries */
-
-rc = ldap_count_entries(lcp->ld, result);
-if (single && rc > 1)
-  {
-  *errmsg = string_sprintf("LDAP search: too many (%d) results "
-    "(filter not specific enough)", rc);
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  *defer_break = TRUE;
-  return DEFER;
-  }
-
-/* Check if we have too few (zero) entries */
-
-if (rc < 1)
-  {
-  *errmsg = string_sprintf("LDAP search: no results");
-  DEBUG(9) debug_printf("%s\n", *errmsg);
-  return FAIL;
+  *errmsg = string_sprintf("ldap search initiation failed");
+  goto RETURN_ERROR;
   }
 
 /* Initialize chunk of store in which to return the answer. */
@@ -317,97 +301,234 @@ size = 100;
 ptr = 0;
 data = store_get(size);
 
-/* Loop through returned entries; we have already checked above for zero
-entries. */
+/* Loop to pick up results as they come in, setting a timeout if one was
+given. */
 
-for(e = ldap_first_entry(lcp->ld, result);
-    e != NULLMSG;
-    e = ldap_next_entry(lcp->ld, e))
+if (timelimit > 0)
   {
-  BOOL add_comma = FALSE;
+  timeout.tv_sec = timelimit;
+  timeout.tv_usec = 0;
+  timeoutptr = &timeout;
+  }
 
-  DEBUG(9) debug_printf("LDAP entry loop\n");
+while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
+        LDAP_RES_SEARCH_ENTRY)
+  {
+  LDAPMessage  *e;
 
-  /* Results for multiple entries values are separated by newlines. */
+  DEBUG(9) debug_printf("ldap_result loop\n");
 
-  if (add_newline)
-    data = string_cat(data, &size, &ptr, "\n", 1);
-  else
-    add_newline = TRUE;
-
-  /* Loop through the entry, grabbing attribute values. Multiple attribute
-  values are separated by commas. */
-
-  for (attr = ldap_first_attribute(lcp->ld, e, &ber);
-       attr != NULL;
-       attr = ldap_next_attribute(lcp->ld, e, ber))
+  for(e = ldap_first_entry(lcp->ld, result);
+      e != NULL;
+      e = ldap_next_entry(lcp->ld, e))
     {
-    if (attr[0])
+    BOOL add_space = FALSE;
+
+    DEBUG(9) debug_printf("LDAP entry loop\n");
+
+    rescount++;   /* Count results */
+
+    /* Results for multiple entries values are separated by newlines. */
+
+    if (add_newline)
+      data = string_cat(data, &size, &ptr, "\n", 1);
+    else
+      add_newline = TRUE;
+
+    /* If the data we want is actually the DN rather than any attribute values,
+    extract it. If there are multiple entries, the DNs will be concatenated,
+    but we test for this case below, as for SEARCH_LDAP_SINGLE, and give an
+    error. */
+
+    if (search_type == SEARCH_LDAP_DN)
       {
-      /* Get array of values for this attribute. */
-
-      if ((firstval = values = ldap_get_values(lcp->ld, e, attr)) != NULL)
+      char *dn = ldap_get_dn(lcp->ld, e);
+      if (dn != NULL)
         {
-        while (*values)
-          {
-          DEBUG(9) debug_printf("LDAP attr loop %s:%s\n", attr, *values);
-
-          if (add_comma)
-            data = string_cat(data, &size, &ptr, ", ", 2);
-          else
-            add_comma = TRUE;
-
-          if (attr_count > 1)
-            {
-            data = string_cat(data, &size, &ptr, attr, strlen(attr));
-            data = string_cat(data, &size, &ptr, "=", 1);
-            }
-
-          data = string_cat(data, &size, &ptr, *values, strlen(*values));
-          data[ptr] = '\0';
-          values++;
-          attribute_found = TRUE;
-          }
-        ldap_value_free(firstval);
+        data = string_cat(data, &size, &ptr, dn, (int)strlen(dn));
+        data[ptr] = 0;
+        #if defined LDAP_LIB_NETSCAPE || defined LDAP_LIB_OPENLDAP2
+        ldap_memfree(dn);
+        #else   /* OPENLDAP 1, UMich, Solaris */
+        free(dn);
+        #endif
+        attribute_found = TRUE;
         }
       }
 
-#ifdef HAVE_NETSCAPE_LDAP_SDK
+    /* Otherwise, loop through the entry, grabbing attribute values. Multiple
+    attribute values are separated by spaces and quoted if necessary, just like
+    NIS+. */
 
-    /* Netscape LDAP's attr's are dynamically allocated and need to be freed.
-    UMich LDAP stores them in static storage and does not require this. */
+    else for (attr = ldap_first_attribute(lcp->ld, e, &ber);
+              attr != NULL;
+              attr = ldap_next_attribute(lcp->ld, e, ber))
+      {
+      if (attr[0] != 0)
+        {
+        /* Get array of values for this attribute. */
 
-    ldap_memfree(attr);
-#endif
+        if ((firstval = values = ldap_get_values(lcp->ld, e, attr)) != NULL)
+          {
+          while (*values != NULL)
+            {
+            DEBUG(9) debug_printf("LDAP attr loop %s:%s\n", attr, *values);
+
+            if (add_space)
+              {
+              #ifdef OLD_LDAP_BEHAVIOUR
+              data = string_cat(data, &size, &ptr, ", ", 2);
+              #else
+              if (attr_count != 1)
+                data = string_cat(data, &size, &ptr, " ", 1);
+              else
+                data = string_cat(data, &size, &ptr, ", ", 2);
+              #endif
+              }
+            else
+              add_space = TRUE;
+
+            #ifdef OLD_LDAP_BEHAVIOUR
+            if (attr_count > 1)
+            #else
+            if (attr_count != 1)
+            #endif
+
+              {
+              data = string_cat(data, &size, &ptr, attr, strlen(attr));
+              data = string_cat(data, &size, &ptr, "=", 1);
+              }
+
+            #if OLD_LDAP_BEHAVIOUR
+            data = string_cat(data, &size, &ptr, *values, strlen(*values));
+            #else
+
+            /* New behaviour: quote the value if it contains spaces or is
+            empty, except for single attributes. */
+
+            if (attr_count != 1 && ((*values)[0] == 0 ||
+                strchr(*values, ' ') != NULL))
+              {
+              char *value = *values;
+              int j;
+              int len = (int)strlen(value);
+              data = string_cat(data, &size, &ptr, "\"", 1);
+              for (j = 0; j < len; j++)
+                {
+                if (value[j] == '\"' || value[j] == '\\')
+                  data = string_cat(data, &size, &ptr, "\\", 1);
+                data = string_cat(data, &size, &ptr, value+j, 1);
+                }
+              data = string_cat(data, &size, &ptr, "\"", 1);
+              }
+            else
+              data = string_cat(data, &size, &ptr, *values, strlen(*values));
+            #endif
+
+            data[ptr] = 0;
+            values++;
+            attribute_found = TRUE;
+            }
+
+          ldap_value_free(firstval);
+          }
+        }
+
+      #if defined LDAP_LIB_NETSCAPE || defined LDAP_LIB_OPENLDAP2
+
+      /* Netscape and OpenLDAP2 LDAP's attrs are dynamically allocated and need
+      to be freed. UMich LDAP stores them in static storage and does not require
+      this. */
+
+      ldap_memfree(attr);
+      #endif
+      }        /* End "for" loop for extracting attributes from an entry */
+    }          /* End "for" loop for extracting entries from a result */
+
+  /* Free the result */
+
+  ldap_msgfree(result);
+  }            /* End "while" loop for multiple results */
+
+DEBUG(9) debug_printf("search ended by ldap_result yielding %d\n",rc);
+
+if (rc == 0)
+  {
+  *errmsg = "ldap_result timed out";
+  goto RETURN_ERROR;
+  }
+
+/* Annoyingly, the different implementations of LDAP have gone for different
+methods of handling error codes and generating error messages. */
+
+if (rc == -1)
+  {
+  DEBUG(9) debug_printf("ldap_result failed\n");
+
+  #if defined LDAP_LIB_SOLARIS || defined LDAP_LIB_OPENLDAP2
+    *errmsg = string_sprintf("ldap_result failed: %s",
+      ldap_err2string(ldap_result2error(lcp->ld, result, 0)));
+
+  #elif defined LDAP_LIB_NETSCAPE
+    {
+    char *error, *matched;
+    (void)ldap_get_lderrno(lcp->ld, &matched, &error);
+    *errmsg = string_sprintf("ldap_result failed: %s (%s)", error, matched);
     }
 
-#if HAVE_NETSCAPE_LDAP_SDK_AND_CHECK_IF_BER_FREE_IS_REALLY_REQUIRED
+  #else                             /* UMich LDAP aka OpenLDAP 1.x */
+    *errmsg = string_sprintf("ldap_result failed: %s (%s)",
+      lcp->ld->ld_error,
+      lcp->ld->ld_matched);
+  #endif
 
-  /* TODO UMich ldap_next_attribute() calls ber_free().
-  Need to check if Netscape LDAP SDK does or does not.... */
+  goto RETURN_ERROR;
+  }
 
-  if (ber != NULL) ber_free(ber, 0);
-#endif
+/* Check if we have too many results */
+
+if (search_type != SEARCH_LDAP_MULTIPLE && rescount > 1)
+  {
+  *errmsg = string_sprintf("LDAP search: more than one entry (%d) was returned "
+    "(filter not specific enough?)", rescount);
+  goto RETURN_ERROR_BREAK;
+  }
+
+/* Check if we have too few (zero) entries */
+
+if (rescount < 1)
+  {
+  *errmsg = string_sprintf("LDAP search: no results");
+  error_yield = FAIL;
+  goto RETURN_ERROR_BREAK;
   }
 
 /* If an entry was found, but it had no attributes, we behave as if no entries
-were found. */
+were found, that is, the lookup failed. */
 
 if (!attribute_found)
   {
   *errmsg = "LDAP search: found no attributes";
-  DEBUG(9) debug_printf("%s", *errmsg);
-  return FAIL;
+  error_yield = FAIL;
+  goto RETURN_ERROR;
   }
 
 /* Otherwise, it's all worked */
 
-else
-  {
-  DEBUG(9) debug_printf("LDAP search: returning: %s\n", data);
-  *res = data;
-  return OK;
-  }
+DEBUG(9) debug_printf("LDAP search: returning: %s\n", data);
+*res = data;
+ldap_free_urldesc(ludp);
+return OK;
+
+/* Error returns */
+
+RETURN_ERROR_BREAK:
+*defer_break = TRUE;
+
+RETURN_ERROR:
+if (ludp != NULL) ldap_free_urldesc(ludp);
+DEBUG(9) debug_printf("%s\n", *errmsg);
+return error_yield;
 }
 
 
@@ -416,35 +537,38 @@ else
 *        Internal search control function        *
 *************************************************/
 
-/* This function is called from eldap_find() and eldapm_find() with a
-difference in the "single" argument. It controls calls to perform_ldap_search()
-which actually does the work. We call that repeatedly for certain types of
-defer in the case when the URL contains no host name and ldap_default_servers
-is set to a list of servers to try. This gives more control than just passing
-over a list of hosts to ldap_open() because it handles other kinds of defer as
-well as just a failure to open. Note that the URL is defined to contain either
-zero or one "hostport" only.
+/* This function is called from eldap_find(), eldapdn_find(), and eldapm_find()
+with a difference in the "search_type" argument. It controls calls to
+perform_ldap_search() which actually does the work. We call that repeatedly for
+certain types of defer in the case when the URL contains no host name and
+ldap_default_servers is set to a list of servers to try. This gives more
+control than just passing over a list of hosts to ldap_open() because it
+handles other kinds of defer as well as just a failure to open. Note that the
+URL is defined to contain either zero or one "hostport" only.
 
 Parameter data in addition to the URL can be passed as preceding text in the
 string, as items of the form XXX=yyy. The URL itself can be detected because it
 must begin "ldap:///".
 
 Arguments:
-  ldap_url    the URL to be looked up, optionally preceded by other parameter
+  ldap_url      the URL to be looked up, optionally preceded by other parameter
                 settings
-  single      TRUE if a maximum of 1 result value is permitted; FALSE otherwise
-  res         set to point at the result
-  errmsg      set to point a message if result is not OK
+  search_type   SEARCH_LDAP_MULTIPLE allows values from multiple entries
+                SEARCH_LDAP_SINGLE allows values from one entry only
+                SEARCH_LDAP_DN gets the DN from one entry
+  res           set to point at the result
+  errmsg        set to point a message if result is not OK
 
-Returns:      OK or FAIL or DEFER
+Returns:        OK or FAIL or DEFER
 */
 
 static int
-control_ldap_search(char *ldap_url, BOOL single, char **res, char **errmsg)
+control_ldap_search(char *ldap_url, int search_type, char **res, char **errmsg)
 {
-BOOL defer_break;
-int timelimit = -1;
-int sizelimit = -1;
+BOOL defer_break = FALSE;
+int timelimit = LDAP_NO_LIMIT;
+int sizelimit = LDAP_NO_LIMIT;
+int sep = 0;
 char *url = ldap_url;
 char *user = NULL;
 char *password = NULL;
@@ -463,7 +587,7 @@ while (strncmp(url, "ldap://", 7) != 0)
     {
     int namelen = ++url - name;
     char *value = string_dequote(&url);
-    if (isspace(*url))
+    if (isspace((uschar)*url))
       {
       if (strncmpic(name, "USER=", namelen) == 0) user = value;
       else if (strncmpic(name, "PASS=", namelen) == 0) password = value;
@@ -476,7 +600,7 @@ while (strncmp(url, "ldap://", 7) != 0)
           namelen, name);
         return DEFER;
         }
-      while (isspace(*url)) url++;
+      while (isspace((uschar)*url)) url++;
       continue;
       }
     }
@@ -491,14 +615,14 @@ DEBUG(9) debug_printf("LDAP parameters: user=%s pass=%s size=%d time=%d\n",
 
 if (ldap_default_servers == NULL || strncmpic(url, "ldap:///", 8) != 0)
   {
-  return perform_ldap_search(url, NULL, 0, single, res, errmsg, &defer_break,
-    user, password, sizelimit, timelimit);
+  return perform_ldap_search(url, NULL, 0, search_type, res, errmsg,
+    &defer_break, user, password, sizelimit, timelimit);
   }
 
 /* Loop through the default servers until OK or FAIL */
 
 list = ldap_default_servers;
-while ((server = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   int rc;
   int port = 0;
@@ -508,8 +632,8 @@ while ((server = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
     *colon = 0;
     port = atoi(colon+1);
     }
-  rc = perform_ldap_search(url, server, port, single, res, errmsg, &defer_break,
-    user, password, sizelimit, timelimit);
+  rc = perform_ldap_search(url, server, port, search_type, res, errmsg,
+    &defer_break, user, password, sizelimit, timelimit);
   if (rc != DEFER || defer_break) return rc;
   }
 
@@ -530,14 +654,21 @@ int
 eldap_find(void *handle, char *filename, char *ldap_url, int length,
   char **result, char **errmsg)
 {
-return(control_ldap_search(ldap_url, TRUE, result, errmsg));
+return(control_ldap_search(ldap_url, SEARCH_LDAP_SINGLE, result, errmsg));
 }
 
 int
 eldapm_find(void *handle, char *filename, char *ldap_url, int length,
   char **result, char **errmsg)
 {
-return(control_ldap_search(ldap_url, FALSE, result, errmsg));
+return(control_ldap_search(ldap_url, SEARCH_LDAP_MULTIPLE, result, errmsg));
+}
+
+int
+eldapdn_find(void *handle, char *filename, char *ldap_url, int length,
+  char **result, char **errmsg)
+{
+return(control_ldap_search(ldap_url, SEARCH_LDAP_DN, result, errmsg));
 }
 
 
