@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -37,8 +37,8 @@ return f;
 *************************************************/
 
 BOOL
-lsearch_check(void *handle, char *filename, int modemask, int *owners,
-  int *owngroups, char **errmsg)
+lsearch_check(void *handle, char *filename, int modemask, uid_t *owners,
+  gid_t *owngroups, char **errmsg)
 {
 return search_check_file(fileno((FILE *)handle), filename, modemask,
   owners, owngroups, "lsearch", errmsg) == 0;
@@ -70,18 +70,21 @@ while (fgets(buffer, sizeof(buffer), f) != NULL)
   char *yield;
   char *s = buffer;
 
-  while (p > 0 && isspace(buffer[p-1])) p--;
+  while (p > 0 && isspace((uschar)buffer[p-1])) p--;
   buffer[p] = 0;
-  if (buffer[0] == 0 || buffer[0] == '#' || isspace(buffer[0])) continue;
-  while (*s != 0 && *s != ':' && !isspace(*s)) s++;
+
+  if (buffer[0] == 0 || buffer[0] == '#' || isspace((uschar)buffer[0]))
+    continue;
+  while (*s != 0 && *s != ':' && !isspace((uschar)*s)) s++;
+
   if (s-buffer != length || strncmpic(buffer, keystring, length) != 0)
     continue;
 
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   if (*s == ':')
     {
     s++;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     }
 
   size = 100;
@@ -93,12 +96,12 @@ while (fgets(buffer, sizeof(buffer), f) != NULL)
   while (fgets(buffer, sizeof(buffer), f) != NULL)
     {
     p = (int)strlen(buffer);
-    while (p > 0 && isspace(buffer[p-1])) p--;
+    while (p > 0 && isspace((uschar)buffer[p-1])) p--;
     buffer[p] = 0;
     if (buffer[0] == 0 || buffer[0] == '#') continue;
-    if (!isspace(buffer[0])) break;
+    if (!isspace((uschar)buffer[0])) break;
     s = buffer;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     *(--s) = ' ';
     yield = string_cat(yield, &size, &ptr, s, (int)strlen(s));
     }
