@@ -2,13 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the lsearch lookup */
 
 extern void *lsearch_open(char *, char **);
-extern BOOL  lsearch_check(void *, char *, int, int *, int *, char **);
+extern BOOL  lsearch_check(void *, char *, int, uid_t *, gid_t *, char **);
 extern int   lsearch_find(void *, char *, char *, int, char **, char **);
 extern void  lsearch_close(void *);
 
