@@ -181,15 +181,12 @@ for (i = 0; i < eo->en_cols.en_cols_len; i++)
       for (j = 0; j < len; j++)
         {
         if (value[j] == '\"' || value[j] == '\\')
-          yield = string_cat(yield, &ssize, &offset,
-            "\\", 1);
-        yield = string_cat(yield, &ssize, &offset,
-            value+j, 1);
+          yield = string_cat(yield, &ssize, &offset, "\\", 1);
+        yield = string_cat(yield, &ssize, &offset, value+j, 1);
         }
       yield = string_cat(yield, &ssize, &offset, "\"", 1);
       }
-    else yield = string_cat(yield, &ssize, &offset,
-      value, len);
+    else yield = string_cat(yield, &ssize, &offset, value, len);
 
     yield = string_cat(yield, &ssize, &offset, " ", 1);
     }
