@@ -6,15 +6,27 @@
 /* See the file NOTICE for conditions of use and distribution. */
 
 
-/* These aren't macros, but they're the sort of general definition
-that fits in this file and we need to have them defined early. Some
-operating systems (naughtily, imo) include a definition for "uchar"
-in the standard header files, so we use "uschar". This is just a typing
+/* These aren't macros, but they're the sort of general definition that fits in
+this file and we need to have them defined early. Some operating systems
+(naughtily, imo) include a definition for "uchar" in the standard header files,
+so we use "uschar". Solaris has u_char in sys/types.h. This is just a typing
 convenience, of course. */
 
 typedef int BOOL;
 typedef unsigned char uschar;
 
+/* These macros save typing for the casting that is needed to cope with the
+mess that is "char" in ISO/ANSI C. Having now been bitten enough times, I'm
+working towards declaring everything explicitly unsigned - this transformation
+is by no means complete, as it is just getting done as I go along. At the end
+of the transformation, some of the casts that have been put in will strictly be
+unnecessary, but they do no harm. I will try to have a tidy in due course. */
+
+#define CS   (char *)
+#define CSS  (char **)
+#define US   (unsigned char *)
+#define USS  (unsigned char **)
+
 /* These two macros make it possible to obtain the result of macro-expanding
 a string as a text string. This is sometimes useful for debugging output. */
 
@@ -22,12 +34,29 @@ a string as a text string. This is sometimes useful for debugging output. */
 #define mac_expanded_string(s) mac_string(s)
 
 
+/* The address_item structure has a word full of 1-bit flags. These macros
+manipulate them. */
+
+#define setflag(addr,flag)    addr->flags |= (flag)
+#define clearflag(addr,flag)  addr->flags &= ~(flag)
+
+#define testflag(addr,flag)       ((addr->flags & (flag)) != 0)
+#define testflagsall(addr,flag)   ((addr->flags & (flag)) == (flag))
+
+#define copyflag(addrnew,addrold,flag) \
+  addrnew->flags = (addrnew->flags & ~(flag)) | (addrold->flags & (flag))
+
+#define orflag(addrnew,addrold,flag) \
+  addrnew->flags |= addrold->flags & (flag)
+
+
 /* We need a special return code for "no recipients and failed to send an error
 message". ANSI C defines only EXIT_FAILURE and EXIT_SUCCESS. On the assumption
 that these are always 1 and 0 on Unix systems ... */
 
 #define EXIT_NORECIPIENTS 2
 
+
 /* Character-handling macros. It seems that the set of standard functions in
 ctype.h aren't actually all that useful. One reason for this is that email is
 international, so the concept of using a locale to vary what they do is not
@@ -53,6 +82,23 @@ unsigned. */
   ((uschar)(c) > 127 && print_topbitchars))
 
 
+/* When adding things to log lines, they come in pairs, as strings. This macro
+packages up two calls to string_cat(). */
+
+#define mac_cat2(s,size,ptr,s1,s2) \
+  string_cat(string_cat(s,size,ptr,s1,(int)strlen(s1)),size,ptr,s2,(int)strlen(s2))
+
+
+/* When built with TLS support, the act of flushing SMTP output becomes
+a no-op once an SSL session is in progress. */
+
+#ifdef SUPPORT_TLS
+#define mac_smtp_fflush() if (!tls_active) fflush(smtp_out);
+#else
+#define mac_smtp_fflush() fflush(smtp_out);
+#endif
+
+
 /* Define which ends of pipes are for reading and writing, as some systems
 don't make both fd's two-way. */
 
@@ -219,19 +265,18 @@ enum {
 
 /* Returns from the directing, routing, transport and authentication functions
 (not all apply to all of them). Some other functions also use these convenient
-values. */
+values, and some additional values are used only by non-driver functions. */
 
 enum {
   OK,                       /* Success */
-  OK_CONTINUE,              /* Success; if verifying, always continue with
-                               generated address(es) - directors only */
-  DEFER,                    /* Failed soft */
-  FAIL,                     /* Failed hard */
+  DEFER,                    /* Defer delivery */
+  DECLINE,                  /* Declined to handle the address, pass to next
+                               driver unless no_more is set */
+  PASS,                     /* Pass to next driver, even if no_more is set */
+  FORCEFAIL,                /* Failed, and don't ever pass to next driver */
   FAIL_CONDITION,           /* Failed a condition test in a director */
-  FAILMORE,                 /* Fail, but override no_more */
-  FORCEFAIL,                /* Failed, and don't pass on */
   ISLOCAL,                  /* Remote address turned out to be local */
-  PASS,                     /* Pass to next - routers only */
+  FAIL,                     /* Transport failed */
   ERROR,                    /* Soft failed with internal or config error */
   PANIC,                    /* Hard failed with internal error */
   BAD64,                    /* Bad base64 data (auth) */
@@ -294,6 +339,8 @@ to conflict with system errno values. */
 #define ERRNO_BADCREATE      (-35)   /* Creation outside home in appendfile */
 #define ERRNO_LISTDEFER      (-36)   /* Can't check a list; lookup defer */
 #define ERRNO_DNSDEFER       (-37)   /* DNS lookup defer */
+#define ERRNO_TLSFAILURE     (-38)   /* Failed to start TLS session */
+#define ERRNO_TLSREQUIRED    (-39)   /* Mandatory TLS session not started */
 
 /* These must be last, so all retry deferments can easily be identified */
 
@@ -333,9 +380,12 @@ used so they can easily be seen in the spool file. */
 enum { hn_bcc, hn_cc, hn_date, hn_from, hn_msgid, hn_sender, hn_to,
   hn_replyto, hn_subject };
 
-/* Types of item in options lists */
+/* Types of item in options lists. These are the bottom bits of the "type"
+field, which is an int. The opt_void value is used for entries in tables that
+point to special types of value that are accessed only indirectly (e.g. the
+rewrite data that is built out of a string option.) */
 
-enum { opt_stringptr, opt_transportptr, opt_int,
+enum { opt_void, opt_stringptr, opt_transportptr, opt_rewrite, opt_int,
   opt_octint, opt_mkint, opt_fixed, opt_time, opt_timelist, opt_bool,
   opt_bool_verify, opt_bool_set, opt_uid, opt_gid, opt_uidlist, opt_gidlist,
   opt_expand_uid, opt_expand_gid, opt_searchtype, opt_local_batch };
@@ -346,11 +396,14 @@ that are automatically maintained from others. Another high bit is used to
 flag driver options that although private (so as to be settable only on some
 drivers), are stored in the instance block so as to be accessible from outside.
 A third high bit is set when an option is read, so as to be able to give an
-error if any option is set twice. */
+error if any option is set twice. Finally, there's a bit which is set when an
+option is set with the "hide" prefix, to prevent -bP from showing it to
+non-admin callers. */
 
 #define opt_hidden  0x100
 #define opt_public  0x200
 #define opt_set     0x400
+#define opt_secure  0x800
 #define opt_mask    0x0ff
 
 /* Verify types when directing and routing */
@@ -376,7 +429,7 @@ enum { hstatus_unknown, hstatus_usable, hstatus_unusable,
 
 /* Reasons why a host is unusable (for clearer log messages) */
 
-enum { hwhy_unknown, hwhy_retry, hwhy_failed, hwhy_deferred };
+enum { hwhy_unknown, hwhy_retry, hwhy_failed, hwhy_deferred, hwhy_ignored };
 
 /* Domain lookup types for routers */
 
@@ -385,7 +438,7 @@ enum { lk_pass, lk_byname, lk_bydns, lk_bydns_a, lk_bydns_mx };
 /* Values for the self_code fields */
 
 enum { self_freeze, self_defer, self_send, self_reroute, self_local,
-  self_fail, self_forcefail };
+  self_pass, self_forcefail };
 
 /* Flags for rewrite rules */
 
@@ -438,7 +491,8 @@ enum {
   HOST_FIND_FAILED,     /* failed to find the host */
   HOST_FIND_AGAIN,      /* could not resolve at this time */
   HOST_FOUND,           /* found host */
-  HOST_FOUND_LOCAL      /* found, but MX points to local host */
+  HOST_FOUND_LOCAL,     /* found, but MX points to local host */
+  HOST_IGNORED          /* found but ignored - used internally only */
 };
 
 /* Actions applied to specific messages. */
