@@ -296,9 +296,13 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
 
     if (f == NULL)
       {
+      #ifdef STAND_ALONE
+      char *listname = "list";
+      #else
       char *listname = readconf_find_option(listptr);
       if (listname[0] == 0)
         listname = string_sprintf("\"%s\"", *listptr);
+      #endif
       log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
         string_open_failed(errno,
           "%s when checking for %s in %s", ss, s, listname));
@@ -450,6 +454,7 @@ while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
     {
     uid_t uid;
     gid_t gid;
+    BOOL ok;
     struct passwd *pw;
     char *comma = strchr(file, ',');
 
@@ -474,10 +479,15 @@ while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
     at that point. Then set the uid. */
 
     if (comma != NULL) *comma = 0;
+    ok = direct_finduser(file, &pw, &uid);
+    if (comma != NULL) *comma = ',';
 
-    if (!direct_finduser(file, &pw, &uid))
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "user \"%s\" for file existence "
-        "test not found", file);
+    if (!ok)
+      {
+      DEBUG(1) debug_printf("user \"%s\" for file existence test not found\n",
+        file);
+      return FAIL;
+      }
 
     /* If there was no comma, the gid is either that associated with the
     user, or unchanged. */
@@ -489,9 +499,11 @@ while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
     else
       {
       if (!direct_findgroup(comma + 1, &gid))
-        log_write(0, LOG_MAIN|LOG_PANIC_DIE, "group \"%s\" for file existence "
-          "test not found", comma + 1);
-      *comma = ',';
+        {
+        DEBUG(1) debug_printf("group \"%s\" for file existence test not found\n",
+          comma + 1);
+        return FAIL;
+        }
       }
 
     /* Change the effective uid/gid, and proceed to the next file. */
@@ -540,7 +552,6 @@ if (restore_uid)
   DEBUG(9) debug_print_ids("restored");
   }
 
-errno = save_errno;
 return yield;
 }
 
@@ -939,8 +950,12 @@ while ((ss = string_nextinlist(&list, separator, buffer, sizeof(buffer)))
 
     if (f == NULL)
       {
+      #ifdef STAND_ALONE
+      char *opt_name = "list";
+      #else
       char *opt_name = readconf_find_option(listptr);
       if (opt_name[0] == 0) opt_name = string_sprintf("\"%s\"", *listptr);
+      #endif
       log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
         string_open_failed(errno, "%s when checking for %s in %s",
           ss, origaddress, opt_name));
