@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1996 */
+/* Copyright (c) University of Cambridge 1996 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #ifdef STAND_ALONE
@@ -272,7 +272,7 @@ switch (n)
 
 #ifndef OS_STREXIT
 /*************************************************
-*      Provide strexit() for systems without   *
+*      Provide strexit() for systems without     *
 *************************************************/
 
 /* Actually, I don't know of any system that has a strexit() function to turn
@@ -438,13 +438,260 @@ return -1;
 
 
 
+#if !defined FIND_RUNNING_INTERFACES
+/*************************************************
+*     Find all the running network interfaces    *
+*************************************************/
+
+/* Finding all the running interfaces is something that has os-dependent
+tweaks, even in the IPv4 case, and it gets worse for IPv6, which is why this
+code is now in the os-dependent source file. There is a common function which
+works on most OS (except IRIX) for IPv4 interfaces, and, with some variations
+controlled by macros, on at least one OS for IPv6 and IPv4 interfaces. On Linux
+with IPv6, the common function is used for the IPv4 interfaces and additional
+code used for IPv6. Consequently, the real function is called
+os_common_find_running_interfaces() so that it can be called from the Linux
+function. On non-Linux systems, the macro for os_find_running_interfaces just
+calls the common function; on Linux it calls the Linux function.
+
+This function finds the addresses of all the running interfaces on the machine.
+A chain of blocks containing the textual form of the addresses is returned.
+
+Problems:
+
+  (1) Solaris 2 has the SIOGIFNUM call to get the number of interfaces, but
+  other OS (including Solaris 1) appear not to. So just screw in a largeish
+  fixed number, defined by MAXINTERFACES. This is in the config.h file and
+  can be changed in Local/Makefile. Unfortunately, the www addressing scheme
+  means that some hosts have a very large number of virtual interfaces. Such
+  hosts are recommended to set local_interfaces to avoid problems with this.
+
+  (2) If the standard code is run on IRIX, it does not return any alias
+  interfaces. There is special purpose code for that operating system, which
+  uses the sysctl() function. The code is in OS/os.c-IRIX, and this code isn't
+  used on that OS.
+
+  (3) Some experimental/developing OS (e.g. GNU/Hurd) do not have any means
+  of finding the interfaces. If NO_FIND_INTERFACES is set, a fudge-up is used
+  instead.
+
+  (4) Some operating systems set the IP address in what SIOCGIFCONF returns;
+  others do not, and require SIOCGIFADDR to be called to get it. For most of
+  the former, calling the latter does no harm, but it causes grief on Linux and
+  BSD systems in the case of IP aliasing, so a means of cutting it out is
+  provided.
+
+Arguments:    none
+Returns:      a chain of ip_address_items, each pointing to a textual
+              version of an IP address
+*/
+
+
+#ifndef NO_FIND_INTERFACES
+
+/* If there is IPv6 support, and SIOCGLIFCONF is defined, define macros to
+use these new, longer versions of the old IPv4 interfaces. Otherwise, define
+the macros to use the historical versions. */
+
+#if HAVE_IPV6 && defined SIOCGLIFCONF
+#define V_ifconf        lifconf
+#define V_ifreq         lifreq
+#define V_GIFADDR       SIOCGLIFADDR
+#define V_GIFCONF       SIOCGLIFCONF
+#define V_GIFFLAGS      SIOCGLIFFLAGS
+#define V_ifc_buf       lifc_buf
+#define V_ifc_family    lifc_family
+#define V_ifc_flags     lifc_flags
+#define V_ifc_len       lifc_len
+#define V_ifr_addr      lifr_addr
+#define V_ifr_flags     lifr_flags
+#define V_ifr_name      lifr_name
+#define V_FAMILY_QUERY  AF_UNSPEC
+#define V_family        ss_family
+#else
+#define V_ifconf        ifconf
+#define V_ifreq         ifreq
+#define V_GIFADDR       SIOCGIFADDR
+#define V_GIFCONF       SIOCGIFCONF
+#define V_GIFFLAGS      SIOCGIFFLAGS
+#define V_ifc_buf       ifc_buf
+#define V_ifc_family    ifc_family
+#define V_ifc_flags     ifc_flags
+#define V_ifc_len       ifc_len
+#define V_ifr_addr      ifr_addr
+#define V_ifr_flags     ifr_flags
+#define V_ifr_name      ifr_name
+#define V_family        sa_family
+#endif
+
+/* In all cases of IPv6 support, use an IPv6 socket. Otherwise (at least on
+Solaris 8) the call to read the flags doesn't work for IPv6 interfaces. If
+we find we can't actually make an IPv6 socket, the code will revert to trying
+an IPv4 socket. */
+
+#if HAVE_IPV6
+#define FAMILY          AF_INET6
+#else
+#define FAMILY          AF_INET
+#endif
+
+/* OK, after all that preliminary stuff, here's the code. */
+
+ip_address_item *
+os_common_find_running_interfaces(void)
+{
+struct V_ifconf ifc;
+struct V_ifreq ifreq, *ifr;
+int vs;
+ip_address_item *yield = NULL;
+ip_address_item *last = NULL;
+ip_address_item  *next;
+char *cp;
+char buf[MAXINTERFACES*sizeof(struct V_ifreq)];
+
+/* We have to create a socket in order to do ioctls on it to find out
+what we want to know. */
+
+if ((vs = socket(FAMILY, SOCK_DGRAM, 0)) < 0)
+  {
+  #if HAVE_IPV6
+  DEBUG(9) debug_printf("Unable to create IPv6 socket to find interface "
+    "addresses:\n  error %d %s\nTrying for an IPv4 socket\n", errno,
+    strerror(errno));
+  vs = socket(AF_INET, SOCK_DGRAM, 0);
+  if (vs < 0)
+  #endif
+  log_write(0, LOG_PANIC_DIE, "Unable to create IPv4 socket to find interface "
+    "addresses: %d %s", errno, strerror(errno));
+  }
+
+/* Get the interface configuration. Some additional data is required when the
+new structures are in use. */
+
+ifc.V_ifc_len = sizeof(buf);
+ifc.V_ifc_buf = buf;
+
+#ifdef V_FAMILY_QUERY
+ifc.V_ifc_family = V_FAMILY_QUERY;
+ifc.V_ifc_flags = 0;
+#endif
+
+if (ioctl(vs, V_GIFCONF, (char *)&ifc) < 0)
+  log_write(0, LOG_PANIC_DIE, "Unable to get interface configuration: %d %s",
+    errno, strerror(errno));
+
+/* For each interface, check it is an IP interface, get its flags, and see if
+it is up; if not, skip.
+
+BSD systems differ from others in what SIOCGIFCONF returns. Other systems
+return a vector of ifreq structures whose size is as defined by the structure.
+BSD systems allow sockaddrs to be longer than their sizeof, which in turn makes
+the ifreq structures longer than their sizeof. The code below has its origins
+in amd and ifconfig; it uses the sa_len field of each sockaddr to determine
+each item's length. */
+
+for (cp = buf; cp < buf + ifc.V_ifc_len; cp +=
+    #ifdef HAVE_SA_LEN
+    mac_max(ifr->ifr_addr.sa_len, sizeof(ifr->ifr_addr)) +
+      sizeof(ifr->ifr_name))
+    #else
+    sizeof(*ifr))
+    #endif
+  {
+  ifr = (struct V_ifreq *)cp;
+
+  /* If not an IP interface, skip */
+
+  if (ifr->V_ifr_addr.V_family != AF_INET
+  #if HAVE_IPV6
+    && ifr->V_ifr_addr.V_family != AF_INET6
+  #endif
+    ) continue;
+
+  /* Get the interface flags, and if the interface is down, continue */
+
+  ifreq = *ifr;
+  if (ioctl(vs, V_GIFFLAGS, (char *)&ifreq) < 0)
+    log_write(0, LOG_PANIC_DIE, "Unable to get flags for %s interface: %d %s",
+      ifreq.V_ifr_name, errno, strerror(errno));
+  if ((ifreq.V_ifr_flags & IFF_UP) == 0) continue;
+
+  /* On some operating systems we have to get the IP address of the interface
+  by another call. On others, it's already there, but we must reinstate the
+  data in ifreq, because SIOCGIFFLAGS may wreck it. */
+
+  #ifndef SIOCGIFCONF_GIVES_ADDR
+  if (ioctl(vs, V_GIFADDR, (char *)&ifreq) < 0)
+    log_write(0, LOG_PANIC_DIE, "Unable to get IP address for %s interface: "
+      "%d %s", ifreq.V_ifr_name, errno, strerror(errno));
+  #else
+  ifreq = *ifr;
+  #endif
+
+  /* Create a data block for the address, fill in the data, and put it on the
+  chain. This data has to survive for ever, so use malloc. */
+
+  next = store_malloc(sizeof(ip_address_item));
+  next->next = NULL;
+  (void)host_ntoa(-1, &ifreq.V_ifr_addr, next->address, NULL);
+
+  if (yield == NULL) yield = last = next; else
+    {
+    last->next = next;
+    last = next;
+    }
+
+  DEBUG(9) debug_printf("Actual local interface address is %s (%s)\n",
+    last->address, ifreq.V_ifr_name);
+  }
+
+/* Close the socket, and return the chain of data blocks. */
+
+close(vs);
+return yield;
+}
+
+#else  /* NO_FIND_INTERFACES */
+
+/* Some experimental or developing OS (e.g. GNU/Hurd) do not have the ioctls,
+and there is no other way to get a list of the (IP addresses of) local
+interfaces. Instead, we fake a local_interfaces="127.0.0.1" without setting it.
+If local_interfaces is unset, the default behaviour is to listen to all
+interfaces on a single socket. The only difference is that the interfaces will
+not be recognized as local. Hence, the user should add further local IPs to
+host_treat_as_local. However, setting local_interfaces is probably the best
+approach. */
+
+ip_address_item *
+os_common_find_running_interfaces(void)
+{
+ip_address_item *yield = NULL;
+char *my_local_interfaces = "127.0.0.1"; /* local_interfaces */
+
+/* We know that local_interfaces is NULL when we come here from
+host_find_interfaces(). Also we know that if we set it to something else it
+will behave differently. In particular, it won't call us again, so this is not
+a loop. */
+
+local_interfaces = my_local_interfaces;
+yield = host_find_interfaces();
+local_interfaces = NULL;
+return yield;
+}
+
+#endif /* NO_FIND_INTERFACES */
+#endif /* FIND_RUNNING_INTERFACES */
+
+
+
+
+
 /*************************************************
 **************************************************
 *             Stand-alone test program           *
 **************************************************
 *************************************************/
 
-/* This is for testing os_getloadavg(). */
 
 #ifdef STAND_ALONE
 
@@ -458,12 +705,6 @@ return -1;
   #endif
 #endif
 
-void
-sigalrm_handler(int sig)
-{
-printf("SIGALRM handler run\n");
-}
-
 
 int main(int argc, char **argv)
 {
@@ -496,8 +737,8 @@ else
   printf("Read: %s", buffer);
   }
 alarm(0);
-printf("Testing load averages\n");
 
+printf("Testing load averages (last test - ^C to kill)\n");
 for (;;)
   {
   int avg;
