@@ -30,7 +30,8 @@ addr->orig = address;
 return addr;
 }
 
-char *rewrite_address(char *recipient, BOOL dummy1, BOOL dummy2)
+char *rewrite_address(char *recipient, BOOL dummy1, BOOL dummy2, rewrite_rule
+  *dummy3, int dummy4)
 {
 return recipient;
 }
@@ -747,6 +748,46 @@ return NULL;
 
 
 
+/*************************************************
+*        Quote according to RFC 2047             *
+*************************************************/
+
+/* This function is used for quoting text in headers according to RFC 2047,
+assuming iso-8859-1 as the character set. The result is returned in big_buffer.
+
+Arguments:
+  s          the string to quote - already checked for non-printing chars
+
+Returns:     pointer to big_buffer, containing the quoted string
+*/
+
+char *
+parse_qp(char *s)
+{
+char *t;
+strcpy(big_buffer, "=?iso-8859-1?Q?");
+t = big_buffer + (int)strlen(big_buffer);
+for (; *s != 0; s++)
+  {
+  int ch = (uschar)*s;
+  if (t > big_buffer + big_buffer_size - 8) break;
+  if (!mac_isprint(ch) || strchr(" \t?=()<>@,;:\\\".[]_", ch) != NULL)
+    {
+    if (ch == ' ') *t++ = '_'; else
+      {
+      sprintf(t, "=%02X", ch);
+      while (*t != 0) t++;
+      }
+    }
+  else *t++ = ch;
+  }
+sprintf(t, "?=");
+return big_buffer;
+}
+
+
+
+
 /*************************************************
 *            Fix up an RFC 822 "phrase"          *
 *************************************************/
@@ -772,29 +813,55 @@ but
 Sheesh! This is tedious code. It is a great pity that the syntax of RFC822 is
 the way it is...
 
-Non-printing characters are turned into question marks, which do not need to be
-quoted. The result is passed back in the big buffer, and must therefore be
+August 2000: Additional code added:
+
+  Previously, non-printing characters were turned into question marks, which do
+  not need to be quoted.
+
+  Now, a different tactic is used if there are any non-printing characters. The
+  encoding method from RFC 2047 is used, assuming iso-8859-1 as the character
+  set, and the whole text is turned into Q-P.
+
+  We *could* use this for all cases, getting rid of the messy original code,
+  but leave it for now.
+
+The result is passed back in the big buffer, and must therefore be
 copied - this makes sense, since it is usually going to be added to some other
 string. In order to be sure there is going to be no overflow, restrict the
-length of the input to 1/3 of the big buffer size - this allows for every
-single character to be quoted without overflowing, and that wouldn't happen
-because of amalgamation. If the phrase is too long, return a fixed string -
-heck, this is just a paranoia check, after all.
+length of the input to 1/4 of the big buffer size - this allows for every
+single character to be quoted or encoded without overflowing, and that wouldn't
+happen because of amalgamation. If the phrase is too long, return a fixed
+string - heck, this is just a paranoia check, after all.
 
 Argument: an RFC822 phrase
 Returns:  the fixed RFC822 phrase
 */
 
 char *
-parse_fix_phrase(char *s)
+parse_fix_phrase(char *phrase)
 {
 int ch;
 BOOL quoted = FALSE;
-char *t = big_buffer + 1;
-char *yield = t;
+char *s, *t, *yield;
 
-while (isspace((uschar)*s)) s++;
-if ((int)strlen(s) > big_buffer_size/3) return "Name too long";
+while (isspace((uschar)*phrase)) phrase++;
+if ((int)strlen(phrase) > big_buffer_size/4) return "Name too long";
+
+/* See if there are any non-printing characters, and if so, use the Q-P
+encoding for the whole thing. */
+
+s = phrase;
+for (s = phrase; *s != 0; s++)
+  {
+  if (!mac_isprint((uschar)*s)) break;
+  }
+
+if (*s != 0) return parse_qp(phrase);
+
+/* No non-printers; use the RFC 822 quoting rules */
+
+s = phrase;
+yield = t = big_buffer + 1;
 
 while ((ch = (uschar)*s++) != 0)
   {
@@ -1347,7 +1414,7 @@ for (;;)
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(t, TRUE);
-      addr->pfr = TRUE;              /* indicates pipe/file/reply */
+      setflag(addr, af_pfr);              /* indicates pipe/file/reply */
       store_pool = pool_reset;
       }
 
@@ -1397,7 +1464,9 @@ for (;;)
       /* Address was successfully parsed. Rewrite, and then make an address
       block in the MAIN storage pool. */
 
-      recipient = rewrite? rewrite_address(recipient, TRUE, FALSE) :
+      recipient = rewrite?
+        rewrite_address(recipient, TRUE, FALSE, global_rewrite_rules,
+          rewrite_existflags) :
         rewrite_address_qualify(recipient, TRUE);
 
       store_pool = POOL_MAIN;
@@ -1429,7 +1498,7 @@ for (;;)
 **************************************************
 *************************************************/
 
-#if defined STAND_ALONE && !defined TEST_EXPAND
+#if defined STAND_ALONE
 int main(void)
 {
 int start, end, domain;
@@ -1439,6 +1508,14 @@ big_buffer = store_malloc(big_buffer_size);
 
 /* strip_trailing_dot = TRUE; */
 
+printf("Testing parse_fix_phrase\n");
+
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
+  {
+  buffer[(int)strlen(buffer)-1] = 0;
+  printf("%s\n", parse_fix_phrase(buffer));
+  }
+
 printf("Testing parse_extract_address without group syntax\n");
 
 while (fgets(buffer, sizeof(buffer), stdin) != NULL)
@@ -1483,18 +1560,10 @@ while (fgets(buffer, sizeof(buffer), stdin) != NULL)
       }
 
     s = ss + (terminator? 1:0);
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     }
   }
 
-printf("Testing parse_fix_phrase\n");
-
-while (fgets(buffer, sizeof(buffer), stdin) != NULL)
-  {
-  buffer[(int)strlen(buffer)-1] = 0;
-  printf("%s\n", parse_fix_phrase(buffer));
-  }
-
 printf("Testing parse_find_at\n");
 
 while (fgets(buffer, sizeof(buffer), stdin) != NULL)
@@ -1521,7 +1590,7 @@ while (fgets(buffer, sizeof(buffer), stdin) != NULL)
       {
       address_item *addr = anchor;
       anchor = anchor->next;
-      printf("%d %s\n", addr->pfr, addr->orig);
+      printf("%d %s\n", testflag(addr, af_pfr), addr->orig);
       }
     }
   else printf("Failed: %d %s\n", extracted, errmess);
