@@ -388,8 +388,10 @@ for (i  = (queue_run_in_order? -1 : 0);
     struct stat statbuf;
     char buffer[256];
 
-    if (stop_id != NULL && strcmp(f->text, stop_id) > 0) continue;
-    if (start_id != NULL && strcmp(f->text, start_id) < 0) continue;
+    if (stop_id != NULL && strncmp(f->text, stop_id, MESSAGE_ID_LENGTH) > 0)
+      continue;
+    if (start_id != NULL && strncmp(f->text, start_id, MESSAGE_ID_LENGTH) < 0)
+      continue;
 
     message_subdir[0] = f->dir_char;
     sprintf(buffer, "%s/input/%s/%s", spool_directory, message_subdir, f->text);
@@ -731,9 +733,8 @@ for (; f != NULL; f = f->next)
 
   if (env_read && sender_address != NULL)
     {
-    if (user_null_sender && sender_address[0] != 0)
-      printf(" <> (%s)", sender_address);
-    else printf(" <%s>", sender_address);
+    printf(" <%s>", sender_address);
+    if (sender_set_untrusted) printf(" (%s)", originator_login);
     }
 
   if (rc != spool_read_OK)
@@ -1135,7 +1136,6 @@ switch(action)
         else  /* MSG_EDIT_SENDER */
           {
           sender_address = receiver;
-          user_null_sender = FALSE;
           log_write(0, LOG_MAIN, "sender address changed to <%s> by %s",
             receiver, username);
           }
