@@ -42,9 +42,12 @@ trying is filled into the last_try field of the host block. If a host is
 generally usable, a check is made to see if there is a retry delay on this
 specific message at this host.
 
+If a non-standard port is being used, it is added to the retry key.
+
 Arguments:
   domain              the address domain
   host                pointer to a host item
+  portstring          "" for standard port, ":xxxx" for a non-standard port
   include_ip_address  TRUE to include the address in the key - this is
                         usual, but sometimes is not wanted
   retry_host_key      where to put a pointer to the key for the host-specific
@@ -57,8 +60,8 @@ Returns:    TRUE if the host has expired but is usable because
 */
 
 BOOL
-retry_check_address(char *domain, host_item *host, BOOL include_ip_address,
-  char **retry_host_key, char **retry_message_key)
+retry_check_address(char *domain, host_item *host, char *portstring,
+  BOOL include_ip_address, char **retry_host_key, char **retry_message_key)
 {
 BOOL yield = FALSE;
 time_t now = time(NULL);
@@ -81,8 +84,8 @@ host->status = hstatus_usable;
 host names are lower cased (that's what %S does). */
 
 host_key = include_ip_address?
-  string_sprintf("T:%S:%s", host->name, host->address) :
-  string_sprintf("T:%S", host->name);
+  string_sprintf("T:%S:%s%s", host->name, host->address, portstring) :
+  string_sprintf("T:%S%s", host->name, portstring);
 
 /* Generate the message-specific key */
 
@@ -107,7 +110,8 @@ if (node != NULL)
 /* Open the retry database, giving up if there isn't one. Otherwise, search for
 the retry records, and then close the database again. */
 
-if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock)) == NULL) return FALSE;
+if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock, FALSE)) == NULL)
+  return FALSE;
 host_retry_record = dbfn_read(dbm_file, host_key);
 message_retry_record = dbfn_read(dbm_file, message_key);
 dbfn_close(dbm_file);
@@ -149,7 +153,7 @@ if (host_retry_record != NULL)
 
   if (now < host_retry_record->next_try && !deliver_force)
     {
-    /* Check for ultimate addres timeout if host not expired */
+    /* Check for ultimate address timeout if host not expired */
 
     if (!host_retry_record->expired)
       {
@@ -448,12 +452,8 @@ for (i = 0; i < 3; i++)
         reached their retry next try time. */
 
         if (dbm_file == NULL)
-          dbm_file = dbfn_open("retry", O_RDWR|O_CREAT, &dbblock);
-        if (dbm_file == NULL)
-          {
-          log_write(0, LOG_MAIN, "Failed to open retry database");
-          return;
-          }
+          dbm_file = dbfn_open("retry", O_RDWR|O_CREAT, &dbblock, TRUE);
+        if (dbm_file == NULL) return;
 
         /* If there are no deferred addresses, that is, if this message is
         completing, and the retry item is for a message-specific SMTP error,
@@ -511,12 +511,15 @@ for (i = 0; i < 3; i++)
               retry->destination);
           }
 
-        /* Set up the message for the database retry record. */
+        /* Set up the message for the database retry record. Because DBM
+        records have a maximum data length, we enforce a limit. There isn't
+        much point in keeping a huge message here, anyway. */
 
         message = (rti->basic_errno > 0)? strerror(rti->basic_errno) :
           (rti->message == NULL)?
           "unknown error" : string_printing(rti->message);
         message_length = strlen(message);
+        if (message_length > 100) message_length = 100;
 
         /* Read a retry record from the database or construct a new one.
         Ignore an old one if it is too old since it was last updated. */
@@ -649,13 +652,15 @@ for (i = 0; i < 3; i++)
           retry_record = newr;
           }
 
-        /* Set up the retry record */
+        /* Set up the retry record; message_length may be less than the string
+        length for very long error strings. */
 
         retry_record->last_try = now;
         retry_record->next_try = next_try;
         retry_record->basic_errno = rti->basic_errno;
         retry_record->more_errno = rti->more_errno;
-        strcpy(retry_record->text, message);
+        strncpy(retry_record->text, message, message_length);
+        retry_record->text[message_length] = 0;
 
         DEBUG(5)
           {
@@ -667,7 +672,7 @@ for (i = 0; i < 3; i++)
             retry_record->more_errno, retry_record->text);
           }
 
-        dbfn_write(dbm_file, rti->destination, retry_record,
+        (void)dbfn_write(dbm_file, rti->destination, retry_record,
           sizeof(dbdata_retry) + message_length);
         }                            /* Loop for each retry item */
 
@@ -687,15 +692,17 @@ for (i = 0; i < 3; i++)
     move the address to the failed queue, and restart this loop without
     updating paddr.
 
-    If there were several addresses batched in the same delivery, only the top
-    one will have retry items attached to it, but we want to handle all the
-    same. Each will have a pointer back to its "top" address, and they will
-    precede the item with the retries. We have saved information about them
-    in passing (below) so they can all be cut out at once. */
+    If there were several addresses batched in the same remote delivery, only
+    the original top one will have host retry items attached to it, but we want
+    to handle all the same. Each will have a pointer back to its "top" address,
+    and they will now precede the item with the retries because addresses are
+    inverted when added to these final queues. We have saved information about
+    them in passing (below) so they can all be cut out at once. */
 
     if (i == 2)   /* Handling defers */
       {
-      if (endaddr->retries != NULL && !endaddr->retry_skipped && timed_out)
+      if (endaddr->retries != NULL && !testflag(endaddr, af_retry_skipped) &&
+          timed_out)
         {
         if (last_first == endaddr) paddr = saved_paddr;
         addr = *paddr;
@@ -706,7 +713,7 @@ for (i = 0; i < 3; i++)
 
         for (;; addr = addr->next)
           {
-          addr->retry_timedout = TRUE;
+          setflag(addr, af_retry_timedout);
           addr->message = (addr->message == NULL)? "retry timeout exceeded" :
             string_sprintf("%s: retry timeout exceeded", addr->message);
           log_write(0, LOG_MAIN, "** %s%s%s%s: retry timeout exceeded",
