@@ -59,23 +59,25 @@ forward files, the uid may be that of the user. Ensure it is reset while
 expanding a replacement, in case that involves file lookups.
 
 Arguments:
-  s           address to rewrite
-  flag        indicates which header this address comes from; it must
-                match the flags in the rewriting rule
-  whole       if not NULL, set TRUE if any rewriting rule contained the
-                "whole" bit and it is a header that is being rewritten
-  add_header  if TRUE and rewriting occurs, add an "X-rewrote-xxx" header
-                if headers are in existence; this should be TRUE only when
-                a message is being received, not during delivery
-  name        name of header, for use when adding X-rewrote-xxxx
+  s              address to rewrite
+  flag           indicates which header this address comes from; it must
+                   match the flags in the rewriting rule
+  whole          if not NULL, set TRUE if any rewriting rule contained the
+                   "whole" bit and it is a header that is being rewritten
+  add_header     if TRUE and rewriting occurs, add an "X-rewrote-xxx" header
+                   if headers are in existence; this should be TRUE only when
+                   a message is being received, not during delivery
+  name           name of header, for use when adding X-rewrote-xxxx
+  rewrite_rules  chain of rewriting rules
 
-Returns:      new address if rewritten; the input address if no change;
-              for a header rewrite, if the "whole" bit is set, the entire
-              rewritten address is returned, not just the active bit.
+Returns:         new address if rewritten; the input address if no change;
+                 for a header rewrite, if the "whole" bit is set, the entire
+                 rewritten address is returned, not just the active bit.
 */
 
 char *
-rewrite_one(char *s, int flag, BOOL *whole, BOOL add_header, char *name)
+rewrite_one(char *s, int flag, BOOL *whole, BOOL add_header, char *name,
+  rewrite_rule *rewrite_rules)
 {
 rewrite_rule *rule;
 char *yield = s;
@@ -329,23 +331,27 @@ flag causes a header to be added, recording the old address. This is marked
 and debugging purposes.
 
 Arguments:
-  s             the address to be considered
-  is_recipient  TRUE for recipient addresses; FALSE otherwise
-  add_header    add "X-rewrote-xxx" header when rewriting; this is
-                  set TRUE only for calls from the reception functions
+  s              the address to be considered
+  is_recipient   TRUE for recipient addresses; FALSE otherwise
+  add_header     add "X-rewrote-xxx" header when rewriting; this is
+                   set TRUE only for calls from the reception functions
+  rewrite_rules  points to chain of rewrite rules
+  existflags     bits indicating which headers there are rewrites for
+                 (just an optimisation)
 
-Returns:        possibly rewritten address
+Returns:         possibly rewritten address
 */
 
 char *
-rewrite_address(char *s, BOOL is_recipient, BOOL add_header)
+rewrite_address(char *s, BOOL is_recipient, BOOL add_header,
+  rewrite_rule *rewrite_rules, int existflags)
 {
 int flag = is_recipient? rewrite_envto : rewrite_envfrom;
 s = rewrite_address_qualify(s, is_recipient);
-if ((rewrite_existflags & flag) != 0)
+if ((existflags & flag) != 0)
   {
   char *new = rewrite_one(s, flag, NULL, add_header, is_recipient?
-    "original-recipient" : "sender");
+    "original-recipient" : "sender", rewrite_rules);
   if (new != s) s = new;
   }
 return s;
@@ -375,19 +381,22 @@ in any way. For locally sourced messages, unqualified addresses always get
 qualified.
 
 Arguments:
-  h           pointer to header line block
-  flag        indicates which header this is
-  routed_old  if not NULL, this is a rewrite caused by a router, changing
-                this domain into routed_new
-  routed_new  new routed domain if routed_old is not NULL
+  h              pointer to header line block
+  flag           indicates which header this is
+  routed_old     if not NULL, this is a rewrite caused by a router, changing
+                   this domain into routed_new
+  routed_new     new routed domain if routed_old is not NULL
+  rewrite_rules  points to chain of rewriting rules
+  existflags     bits indicating which rewrites exist
+  replace        if TRUE, insert the new header in the chain after the old
+                   one, and mark the old one "replaced"
 
-Returns:      NULL if header unchanged; otherwise the rewritten header, which
-              has been inserted into the header chain following the old one,
-              which in turn has been marked "old"
+Returns:         NULL if header unchanged; otherwise the rewritten header
 */
 
 static header_line *
-rewrite_one_header(header_line *h, int flag, char *routed_old, char *routed_new)
+rewrite_one_header(header_line *h, int flag, char *routed_old,
+  char *routed_new, rewrite_rule *rewrite_rules, int existflags, BOOL replace)
 {
 int lastnewline = 0;
 header_line *newh = NULL;
@@ -480,10 +489,10 @@ while (*s != 0)
   "whole" flag set, adjust the pointers so that the whole address gets
   replaced, except possibly a final \n. */
 
-  if ((rewrite_existflags & flag) != 0)
+  if ((existflags & flag) != 0)
     {
     BOOL whole;
-    new = rewrite_one(recipient, flag, &whole, FALSE, NULL);
+    new = rewrite_one(recipient, flag, &whole, FALSE, NULL, rewrite_rules);
     if (new != recipient)
       {
       changed = TRUE;
@@ -577,10 +586,10 @@ while (*s != 0)
 parse_allow_group = FALSE;  /* Reset group flags */
 parse_found_group = FALSE;
 
-/* If a rewrite happened, put the new header into the chain following the old
-one, and mark the old one as replaced. */
+/* If a rewrite happened and "replace" is true, put the new header into the
+chain following the old one, and mark the old one as replaced. */
 
-if (newh != NULL)
+if (newh != NULL && replace)
   {
   newh->next = h->next;
   if (newh->next == NULL) header_last = newh;
@@ -609,37 +618,47 @@ The old header line is flagged as "old". Old headers are saved on the spool for
 debugging but are never sent to any recipients.
 
 Arguments:
-  h            header line to rewrite
-  routed_old  if not NULL, this is a rewrite caused by a router, changing
-                this domain into routed_new
-  routed_new  new routed domain if routed_old is not NULL
+  h               header line to rewrite
+  routed_old     if not NULL, this is a rewrite caused by a router, changing
+                   this domain into routed_new
+  routed_new     new routed domain if routed_old is not NULL
+  rewrite_rules  points to chain of rewrite rules
+  existflags     bits indicating which rewrites exist
+  replace        if TRUE, the new header is inserted into the header chain
+                    after the old one, and the old one is marked replaced
 
-Returns:      NULL if header unchanged; otherwise the rewritten header - it is
-                up to the caller to chain it as required
+Returns:         NULL if header unchanged; otherwise the rewritten header
 */
 
 header_line *
-rewrite_header(header_line *h, char *routed_old, char *routed_new)
+rewrite_header(header_line *h, char *routed_old, char *routed_new,
+  rewrite_rule *rewrite_rules, int existflags, BOOL replace)
 {
 switch (h->type)
   {
   case htype_sender:
-  return rewrite_one_header(h, rewrite_sender, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_sender, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
 
   case htype_from:
-  return rewrite_one_header(h, rewrite_from, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_from, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
 
   case htype_to:
-  return rewrite_one_header(h, rewrite_to, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_to, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
 
   case htype_cc:
-  return rewrite_one_header(h, rewrite_cc, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_cc, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
 
   case htype_bcc:
-  return rewrite_one_header(h, rewrite_bcc, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_bcc, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
 
   case htype_replyto:
-  return rewrite_one_header(h, rewrite_replyto, routed_old, routed_new);
+  return rewrite_one_header(h, rewrite_replyto, routed_old, routed_new,
+    rewrite_rules, existflags, replace);
   }
 
 return NULL;
@@ -674,7 +693,8 @@ if (rewrite_existflags == 0)
 
 if ((rewrite_existflags & rewrite_smtp) != 0)
   {
-  char *new = rewrite_one(s, rewrite_smtp, NULL, FALSE, "");
+  char *new = rewrite_one(s, rewrite_smtp, NULL, FALSE, "",
+    global_rewrite_rules);
   if (new != s)
     {
     printf("    SMTP: %s\n", new);
@@ -704,7 +724,8 @@ for (i = 0; i < 8; i++)
   {
   BOOL whole = FALSE;
   int flag = 1 << i;
-  char *new = rewrite_one(recipient, flag, &whole, FALSE, "");
+  char *new = rewrite_one(recipient, flag, &whole, FALSE, "",
+    global_rewrite_rules);
   printf("%s: ", rrname[i]);
   if (whole || (flag & rewrite_all_headers) == 0) printf("%s\n", new);
     else printf("%.*s%s%s\n", start, s, new, s+end);
@@ -714,7 +735,8 @@ for (i = 0; i < 8; i++)
 
 if ((rewrite_existflags & rewrite_X) != 0)
   {
-  char *new = rewrite_one(recipient, rewrite_X, NULL, FALSE, "");
+  char *new = rewrite_one(recipient, rewrite_X, NULL, FALSE, "",
+    global_rewrite_rules);
   if (new != recipient) printf("       X: %s\n", new);
   }
 }
