@@ -50,6 +50,8 @@ optionlist optionlist_routers[] = {
                  (void *)offsetof(router_instance, extra_headers) },
   { "headers_remove",     opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, remove_headers) },
+  { "ignore_target_hosts",       opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, ignore_target_hosts) },
   { "initgroups",         opt_bool | opt_public,
                  (void *)(offsetof(router_instance, initgroups)) },
   { "local_parts",        opt_stringptr|opt_public,
@@ -127,8 +129,8 @@ for (r = routers; r != NULL; r = r->next)
   if      (strcmp(s, "freeze") == 0)    r->self_code = self_freeze;
   else if (strcmp(s, "defer") == 0)     r->self_code = self_defer;
   else if (strcmp(s, "send") == 0)      r->self_code = self_send;
-  else if (strcmp(s, "fail_soft") == 0) r->self_code = self_fail;
-  else if (strcmp(s, "fail_hard") == 0) r->self_code = self_forcefail;
+  else if (strcmp(s, "pass") == 0)      r->self_code = self_pass;
+  else if (strcmp(s, "fail") == 0)      r->self_code = self_forcefail;
   else if (strcmp(s, "local") == 0)     r->self_code = self_local;
   else if (strncmp(s, "reroute:", 8) == 0)
     {
@@ -143,6 +145,11 @@ for (r = routers; r != NULL; r = r->next)
     r->self = s;
     r->self_code = self_reroute;
     }
+
+  /* The next two are obsolete - retained for compatibility */
+  else if (strcmp(s, "fail_soft") == 0) r->self_code = self_pass;
+  else if (strcmp(s, "fail_hard") == 0) r->self_code = self_forcefail;
+
   else log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
       "%s is not valid for the self option", r->name, s);
 
@@ -230,9 +237,10 @@ else
 
 DEBUG(2)
   {
-  debug_printf("queued for %s transport: local_part=%s domain=%s\n",
+  debug_printf("queued for %s transport: local_part=%s domain=%s\n"
+    "  errors_to=%s\n",
     (addr->transport == NULL)? "<unset>" : addr->transport->name,
-    addr->local_part, addr->domain);
+    addr->local_part, addr->domain, addr->errors_address);
   debug_printf("  domain_data=%s local_part_data=%s\n", domain_data,
     local_part_data);
   }
@@ -358,11 +366,15 @@ if ((verify == v_none || verify == v_expn) && dr->verify_only)
   return TRUE;
   }
 
-/* Skip this driver if domain mismatch */
+/* Skip this driver if domain mismatch. Set $key not to expand, so the user
+doesn't have to remember the backslash. */
 
 if (dr->domains != NULL)
   {
-  char *domains = expand_string_panic(dr->domains, dr->name, driver_type);
+  char *domains;
+  lookup_key = "$key";
+  domains = expand_string_panic(dr->domains, dr->name, driver_type);
+  lookup_key = NULL;
   if (!match_isinlist(domain, &domains, TRUE, TRUE, &domain_data))
     {
     DEBUG(9) debug_printf("%s %s skipped: domain mismatch\n",
@@ -371,12 +383,15 @@ if (dr->domains != NULL)
     }
   }
 
-/* Skip this driver if local part mismatch */
+/* Skip this driver if local part mismatch. Set $key not to expand, so the
+user doesn't have to remember the backslash. */
 
 if (dr->local_parts != NULL)
   {
-  char *local_parts = expand_string_panic(dr->local_parts, dr->name,
-    driver_type);
+  char *local_parts;
+  lookup_key = "$key";
+  local_parts = expand_string_panic(dr->local_parts, dr->name, driver_type);
+  lookup_key = NULL;
   if (!match_isinlist(local_part, &local_parts, FALSE,
         (driver_type[0] == 'd' && locally_caseless),
         &local_part_data))
@@ -463,7 +478,7 @@ addr->parent = parent;
 /* Set up the cloned address and put it on the new address queue */
 
 new->parent = parent;
-new->ignore_error |= addr->ignore_error;
+copyflag(new, addr, af_ignore_error|af_forced_local);
 new->errors_address = addr->errors_address;
 new->next = *addr_new;
 *addr_new = new;
@@ -496,24 +511,11 @@ if (addr->transport != NULL &&
 *                 Route one address              *
 *************************************************/
 
-/* This function is passed in one address item, for processing by the
-routers. It has been determined that the address is (apparently) not for one of
-the local domains. The action can be one of:
-
-  . add the address to the chain on addr_remote for remote delivery and
-      return OK;
-  . return FAIL;
-  . return DEFER;
-  . return ISLOCAL - this means that the address turned out to be local
-      after all - usually as the result of DNS expansion to fully qualified.
-  . return PASS - meaning the router did something to the address, and wants
-      it passed to the next router.
-
-Although FAIL and PASS are conceptually different, in fact they have the same
-effect, except for debugging output.
-
-The verify flag is set if this is being called for verification rather than
-delivery. If the router doesn't have its "verify" flag set, it is skipped.
+/* This function is passed in one address item, for processing by the routers.
+It has been determined that the address is (apparently) not for one of the
+local domains. The verify flag is set if this is being called for verification
+rather than delivery. If the router doesn't have its "verify" flag set, it is
+skipped.
 
 Arguments:
   addr         address to route
@@ -524,7 +526,11 @@ Arguments:
                v_sender if verifying a sender address
                v_recipient if verifying a recipient address
 
-Returns:       OK, FAIL, DEFER, ISLOCAL or PASS (see above)
+Returns:       OK      => address successfully routed
+               FAIL    => address could not be routed
+               DEFER   => some temporary problem
+               ISLOCAL => address routed to the local host
+               ERROR   => some major internal or configuration failure
 */
 
 int
@@ -594,7 +600,7 @@ for (r = routers; more && r != NULL; r = r->next)
   rewrite flag; routers must explicitly request rewriting. */
 
   debug_print_string(r->debug_string);
-  addr->rewrite_headers = FALSE;
+  clearflag(addr, af_rewrite_headers);
 
   /* Do file existence tests - must do after setting expansion values
   as the string is expanded - and if they succeed, do any necessary sender
@@ -610,7 +616,7 @@ for (r = routers; more && r != NULL; r = r->next)
 
     case FAIL:
     DEBUG(9) debug_printf("%s router: file existence failure\n", r->name);
-    yield = FAIL;
+    yield = DECLINE;
     break;
 
     case OK:
@@ -627,7 +633,7 @@ for (r = routers; more && r != NULL; r = r->next)
         {
         DEBUG(9) debug_printf("%s router: sender match failure\n",
           r->name);
-        yield = FAIL;
+        yield = DECLINE;
         }
       }
     else if (r->condition != NULL &&
@@ -643,15 +649,33 @@ for (r = routers; more && r != NULL; r = r->next)
         {
         DEBUG(9) debug_printf("%s router: condition failure; more set TRUE\n",
           r->name);
-        yield = FAIL;
+        yield = DECLINE;
         more = TRUE;      /* Condition tests are logically before "more" */
         }
       }
+
+    /* All conditions are OK - we are now going to run the router. First
+    expand ignore_target_hosts. */
+
     else
       {
-      addr->local_host_removed = FALSE;
+      char *ignore_target_hosts = NULL;
+      if (r->ignore_target_hosts != NULL)
+        {
+        ignore_target_hosts = expand_string(r->ignore_target_hosts);
+        if (ignore_target_hosts == NULL &&
+            (search_find_defer || !expand_string_forcedfail))
+          {
+          addr->message = string_sprintf("expansion of \"%s\" failed "
+            "in %s router: %s", r->ignore_target_hosts, r->name,
+            expand_string_message);
+          yield = DEFER;
+          break;
+          }
+        }
+      clearflag(addr, af_local_host_removed);
       yield = (r->info->code)(r, addr, addr_local, addr_remote,
-        verify != v_none);
+        verify != v_none, ignore_target_hosts);
       }
     break;
     }
@@ -672,34 +696,31 @@ for (r = routers; more && r != NULL; r = r->next)
     break;
     }
 
-  /* Router modified the address; loop for next router. */
+  /* Router (might have) modified the address; loop for next router, whatever
+  the setting of "more". */
 
   if (yield == PASS)
     {
-    DEBUG(2) debug_printf("%s router passed, rewriting %s as %s\n", r->name,
-      old_route_domain, addr->route_domain);
-    continue;
-    }
-
-  /* Failed to handle this address; loop for next router, respecting the
-  setting of "more". */
-
-  if (yield == FAIL)
-    {
-    DEBUG(2) debug_printf("%s router failed\n", r->name);
-    continue;
-    }
-
-  /* Failed to handle this address; loop for next router, ignoring the
-  setting of "more". */
-
-  if (yield == FAILMORE)
-    {
-    DEBUG(2) debug_printf("%s router failed; \"more\" overridden\n", r->name);
+    DEBUG(2)
+      {
+      debug_printf("%s router passed; \"more\" overridden\n", r->name);
+      if (strcmp(old_route_domain, addr->route_domain) != 0)
+        debug_printf("domain %s rewritten as %s\n", old_route_domain,
+          addr->route_domain);
+      }
     more = TRUE;
     continue;
     }
 
+  /* Declined to handle this address; loop for next router, respecting the
+  setting of "more". */
+
+  if (yield == DECLINE)
+    {
+    DEBUG(2) debug_printf("%s router declined\n", r->name);
+    continue;
+    }
+
   /* Router wants this address to be failed; do not loop for next router. */
 
   if (yield == FORCEFAIL)
@@ -733,14 +754,19 @@ for (r = routers; more && r != NULL; r = r->next)
     return yield;
     }
 
+  /* The yield must be either OK or ISLOCAL. */
 
-  /* The yield is either OK or ISLOCAL. In both cases we rewrite envelope and
-  headers if requested before finishing with this address. The rewriting takes
-  its data from what the router did to the route_address, which may no longer
-  be the same as the envelope address (in which case the rewriting probably
-  isn't relevant there or in the headers, but the code does support it). */
+  if (yield != OK && yield != ISLOCAL)
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s router returned unknown value %d",
+      r->name, yield);
 
-  if (addr->rewrite_headers &&
+  /* In both cases we rewrite envelope and headers if requested before
+  finishing with this address. The rewriting takes its data from what the
+  router did to the route_address, which may no longer be the same as the
+  envelope address (in which case the rewriting probably isn't relevant there
+  or in the headers, but the code does support it). */
+
+  if (testflag(addr, af_rewrite_headers) &&
       strcmp(old_route_domain, addr->route_domain) != 0)
     {
     DEBUG(9) debug_printf("rewriting after %s router\n", r->name);
@@ -763,7 +789,8 @@ for (r = routers; more && r != NULL; r = r->next)
       for (h = header_list->next; h != NULL; h = h->next)
         {
         header_line *newh =
-          rewrite_header(h, old_route_domain, addr->route_domain);
+          rewrite_header(h, old_route_domain, addr->route_domain,
+            global_rewrite_rules, rewrite_existflags, TRUE);
         if (newh != NULL)
           {
           h = newh;
@@ -790,7 +817,8 @@ for (r = routers; more && r != NULL; r = r->next)
     {
     DEBUG(2) debug_printf("%s router found %s to be local, or locally MX'd\n",
       r->name, old_route_domain);
-    if (!addr->rewrite_headers && strcmp(addr->domain, old_route_domain) == 0)
+    if (!testflag(addr, af_rewrite_headers) &&
+         strcmp(addr->domain, old_route_domain) == 0)
       addr->domain = addr->route_domain;
     addr->router = r;
     return ISLOCAL;
@@ -845,7 +873,7 @@ for (r = routers; more && r != NULL; r = r->next)
             h->name = newaddress;
             h->address = NULL;
 
-            switch (host_find_byname(h, NULL, TRUE))
+            switch (host_find_byname(h, NULL, NULL, TRUE))
               {
               case HOST_FOUND_LOCAL:
               DEBUG(2) debug_printf("translated address is local host\n");
@@ -890,7 +918,7 @@ for (r = routers; more && r != NULL; r = r->next)
     }
 
   /* Record which router did it, and return success, having cleared any
-  temporary error message set by a router that failed, and handled the
+  temporary error message set by a router that declined, and handled the
   "unseen" option. */
 
   addr->router = r;
