@@ -12,10 +12,10 @@
 
 /* Codes for the host_find_failed option. */
 
-#define hff_freeze     0
-#define hff_defer      1
-#define hff_fail_soft  2
-#define hff_fail_hard  3
+#define hff_freeze   0
+#define hff_defer    1
+#define hff_pass     2
+#define hff_fail     3
 
 
 
@@ -101,10 +101,14 @@ if      (strcmp(ob->host_find_failed, "freeze") == 0)
   ob->hff_code = hff_freeze;
 else if (strcmp(ob->host_find_failed, "defer") == 0)
   ob->hff_code = hff_defer;
-else if (strcmp(ob->host_find_failed, "fail_soft") == 0)
-  ob->hff_code = hff_fail_soft;
-else if (strcmp(ob->host_find_failed, "fail_hard") == 0)
-  ob->hff_code = hff_fail_hard;
+else if (strcmp(ob->host_find_failed, "pass") == 0)
+  ob->hff_code = hff_pass;
+else if (strcmp(ob->host_find_failed, "fail") == 0)
+  ob->hff_code = hff_fail;
+else if (strcmp(ob->host_find_failed, "fail_soft") == 0)  /* Obsolete */
+  ob->hff_code = hff_pass;
+else if (strcmp(ob->host_find_failed, "fail_hard") == 0)  /* Obsolete */
+  ob->hff_code = hff_fail;
 else
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
     "unrecognized setting for host_find_failed option", rblock->name);
@@ -255,10 +259,12 @@ domainlist_router_entry(
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
   address_item **addr_remote,     /* add it to this if successful & remote */
-  BOOL verify)                    /* TRUE if verifying */
+  BOOL verify,                    /* TRUE if verifying */
+  char *ignore_target_hosts)      /* ignore host list (expanded) */
 {
 BOOL matched = FALSE;
 BOOL individual_transport_set = FALSE;
+BOOL self_send = FALSE;
 int lookup_type, rc, host_sep, host_count;
 host_item *h, *next_h, *prev;
 char *routelist_item = NULL;
@@ -445,9 +451,9 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL ||
   }
 
 
-/* If we haven't matched anything, the router fails */
+/* If we haven't matched anything, the router declines. */
 
-if (!matched) return FAIL;
+if (!matched) return DECLINE;
 
 
 /* After a match, we expand the hostlist item. It may then pointing to an empty
@@ -462,12 +468,12 @@ newhostlist = expand_string_copy(hostlist);
 lookup_value = NULL;                        /* Finished with */
 expand_nmax = -1;
 
-/* If the expansion was forced to fail, just fail the router. Otherwise
-there is a configuration problem. */
+/* If the expansion was forced to fail, just decline. Otherwise there is a
+configuration problem. */
 
 if (newhostlist == NULL)
   {
-  if (expand_string_forcedfail) return FAIL;
+  if (expand_string_forcedfail) return DECLINE;
   addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
     rblock->name, hostlist , expand_string_message);
   if (search_find_defer) return DEFER;
@@ -654,11 +660,15 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
   if (h->address != NULL) continue;
 
   if (lookup_type == lk_byname)
-    rc = host_find_byname(h, &canonical_name, FALSE);
+    rc = host_find_byname(h, ignore_target_hosts, &canonical_name, FALSE);
   else
-    rc = host_find_bydns(h, (lookup_type == lk_bydns_mx),
+    {
+    BOOL removed;
+    rc = host_find_bydns(h, ignore_target_hosts, (lookup_type == lk_bydns_mx),
       (lookup_type == lk_bydns_a), ob->qualify_single, ob->search_parents,
-      &canonical_name, &(addr->local_host_removed));
+      &canonical_name, &removed);
+    if (removed) setflag(addr, af_local_host_removed);
+    }
 
   /* Temporary failure defers, unless pass_on_timeout is set */
 
@@ -668,7 +678,7 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
       {
       DEBUG(2) debug_printf("%s router timed out & pass_on_timeout set\n",
         rblock->name);
-      return FAIL;
+      return PASS;
       }
     addr->message = string_sprintf("host lookup for %s did not complete "
       "(DNS timeout?)", h->name);
@@ -680,22 +690,25 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
 
   if (rc == HOST_FIND_FAILED)
     {
-    if (ob->hff_code == hff_fail_soft) return FAILMORE;
+    if (ob->hff_code == hff_pass) return PASS;
 
-    addr->message = string_sprintf("lookup of host \"%s\" failed in %s router",
-      h->name, rblock->name);
+    addr->message =
+      string_sprintf("lookup of host \"%s\" failed in %s router%s",
+        h->name, rblock->name,
+        host_find_failed_syntax? ": syntax error in name" : "");
 
     if (ob->hff_code == hff_defer) return DEFER;
-    if (ob->hff_code == hff_fail_hard) return FORCEFAIL;
+    if (ob->hff_code == hff_fail) return FORCEFAIL;
 
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
 
   /* A local host gets chopped, with its successors, if there are previous
-  hosts. Otherwise the self option is used. */
+  hosts. Otherwise the self option is used. If it is set to "send", any
+  subsequent hosts that are also the local host do not get chopped. */
 
-  if (rc == HOST_FOUND_LOCAL)
+  if (rc == HOST_FOUND_LOCAL && !self_send)
     {
     if (prev != NULL)
       {
@@ -705,12 +718,13 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
         for (; h != NULL; h = h->next) debug_printf("  %s\n", h->name);
         }
       prev->next = NULL;
+      setflag(addr, af_local_host_removed);
       break;
       }
-    addr->local_host_removed = TRUE;
     rc = host_self_action(addr, h, rblock->self_code, rblock->self_rewrite,
       rblock->self);
-    if (rc != OK) return rc;
+    if (rc != OK) return rc;    /* Anything other than "send" */
+    self_send = TRUE;
     }
   }
 
