@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the domainlist router */
@@ -14,10 +14,11 @@ typedef struct {
   int   partial_match;
   int   modemask;
   int   hff_code;
-  int  *owners;
-  int  *owngroups;
-  BOOL qualify_single;
-  BOOL search_parents;
+  uid_t *owners;
+  gid_t *owngroups;
+  BOOL  hosts_randomize;
+  BOOL  qualify_single;
+  BOOL  search_parents;
   char *host_find_failed;
   char *route_file;
   char *route_queries;
@@ -37,7 +38,7 @@ extern domainlist_router_options_block domainlist_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int domainlist_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL);
+  address_item **, address_item **, BOOL, char *);
 extern void domainlist_router_init(router_instance *);
 
 /* End of routers/domainlist.h */
