@@ -71,7 +71,8 @@ ipliteral_router_entry(
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
   address_item **addr_remote,     /* add it to this if successful & remote */
-  BOOL verify)                    /* TRUE if verifying */
+  BOOL verify,                    /* TRUE if verifying */
+  char *ignore_target_hosts)      /* ignore host list (expanded) */
 {
 /*
 ipliteral_router_options_block *ob =
@@ -82,6 +83,8 @@ char *domain = addr->route_domain;
 int len = (int)strlen(domain);
 int rc;
 
+ignore_target_hosts = ignore_target_hosts;   /* Stop compiler warnings */
+
 DEBUG(2) debug_printf("%s router called for %s: route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
 
