@@ -26,7 +26,7 @@ extern ipliteral_router_options_block ipliteral_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int ipliteral_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL);
+  address_item **, address_item **, BOOL, char *);
 
 extern void ipliteral_router_init(router_instance *);
 
