@@ -128,7 +128,8 @@ iplookup_router_entry(
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
   address_item **addr_remote,     /* add it to this if successful & remote */
-  BOOL verify)                    /* TRUE if verifying */
+  BOOL verify,                    /* TRUE if verifying */
+  char *ignore_target_hosts)      /* ignore host list (expanded) */
 {
 u_short net_port;
 char *query = NULL;
@@ -146,6 +147,7 @@ int sep = 0;
 
 addr_local = addr_local;    /* Keep picky compilers happy */
 addr_remote = addr_remote;
+ignore_target_hosts = ignore_target_hosts;
 
 DEBUG(2) debug_printf("%s router called for %s: route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
@@ -191,7 +193,8 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
   host->next = NULL;
 
   if (string_is_ip_address(host->name, NULL)) host->address = host->name;
-    else if (host_find_byname(host, NULL, FALSE) == HOST_FIND_FAILED) continue;
+    else if (host_find_byname(host, NULL, NULL, FALSE) == HOST_FIND_FAILED)
+      continue;
 
   /* Loop for possible multiple IP addresses for the given name. */
 
@@ -371,7 +374,7 @@ if (hostname == NULL)
 
 
 /* If a response pattern was supplied, match the returned string against it. A
-failure to match causes the router to fail. After a successful match, the
+failure to match causes the router to decline. After a successful match, the
 numerical variables for expanding the rerouted address are set up. */
 
 if (re != NULL)
@@ -380,7 +383,7 @@ if (re != NULL)
     {
     DEBUG(9) debug_printf("%s router: %s failed to match response %s\n",
       rblock->name, ob->response_pattern, reply);
-    return FAIL;
+    return DECLINE;
     }
   }
 
@@ -404,7 +407,7 @@ else
       {
       DEBUG(9) debug_printf("%s router: failed to match identification "
         "in response %s\n", rblock->name, reply);
-      return FAIL;
+      return DECLINE;
       }
     }
 
