@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -33,7 +33,7 @@ extern iplookup_router_options_block iplookup_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int iplookup_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL);
+  address_item **, address_item **, BOOL, char *);
 
 extern void iplookup_router_init(router_instance *);
 
