@@ -90,7 +90,8 @@ lookuphost_router_entry(
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
   address_item **addr_remote,     /* add it to this if successful & remote */
-  BOOL verify)                    /* TRUE if verifying */
+  BOOL verify,                    /* TRUE if verifying */
+  char *ignore_target_hosts)      /* list of hosts to ignore (expanded) */
 {
 host_item h;
 int rc;
@@ -109,6 +110,7 @@ DEBUG(2)
   rblock->name, addr->orig, (ob->gethostbyname)? "gethostbyname" : "dns",
   addr->route_domain);
 
+
 /* Set up the first of any widening domains. The code further down copes with
 either pre- or post-widening, but at present there is no way to turn on
 pre-widening, as actually doing so seems like a rather bad idea, and nobody has
@@ -161,7 +163,7 @@ for (;;)
     DEBUG(2) debug_printf("%s router trying %s after widening failed\n",
       rblock->name, h.name);
     }
-  else return FAIL;
+  else return DECLINE;
 
   /* Set up the rest of the initial host item. Others may get chained on if
   there is more than one IP address. We set it up here instead of outside the
@@ -179,7 +181,8 @@ for (;;)
   additional host_items if necessary. */
 
   if (ob->gethostbyname)
-    rc = host_find_byname(&h, &fully_qualified_name, FALSE);
+    rc = host_find_byname(&h, ignore_target_hosts, &fully_qualified_name,
+      FALSE);
 
   /* Unfortunately, we cannot set the mx_only option in advance, because the
   DNS lookup may extend an unqualified name. Therefore, we must do the test
@@ -187,8 +190,11 @@ for (;;)
 
   else
     {
-    rc = host_find_bydns(&h, FALSE, FALSE, ob->qualify_single,
-      ob->search_parents, &fully_qualified_name, &(addr->local_host_removed));
+    BOOL removed;
+    rc = host_find_bydns(&h, ignore_target_hosts, FALSE, FALSE,
+      ob->qualify_single, ob->search_parents, &fully_qualified_name,
+      &removed);
+    if (removed) setflag(addr, af_local_host_removed);
 
     /* If host found with only A records, test for the domain's being in the
     mx_domains list, and not turned into a local domain. */
@@ -232,7 +238,7 @@ for (;;)
       {
       DEBUG(2) debug_printf("%s router timed out & pass_on_timeout set\n",
         rblock->name);
-      return FAIL;
+      return PASS;
       }
     addr->message = "host lookup did not complete";
     return DEFER;
@@ -253,16 +259,22 @@ for (;;)
   if (h.mx >= 0 && h.address == NULL)
     {
     addr->message = "all relevant MX records point to non-existent hosts";
-    if (string_is_ip_address(h.name, NULL))
+    if (!allow_mx_to_ip && string_is_ip_address(h.name, NULL))
       addr->message = string_sprintf("%s: it appears that the DNS operator for "
          "this domain has installed an invalid MX record with an IP address "
          "instead of a domain name on the right hand side", addr->message);
-    return FAIL;
+    return DECLINE;
     }
 
-  /* If there's a syntax error, do not continue with any widening. */
+  /* If there's a syntax error, do not continue with any widening, and note
+  the error. */
 
-  if (host_find_failed_syntax) return FAIL;
+  if (host_find_failed_syntax)
+    {
+    addr->message = string_sprintf("mail domain \"%s\" is syntactically "
+      "invalid", addr->domain);
+    return DECLINE;
+    }
   }
 
 
@@ -276,12 +288,12 @@ in its full form, and force header rewriting. */
 if (strcmp(addr->route_domain, fully_qualified_name) != 0)
   {
   addr->route_domain = fully_qualified_name;
-  addr->rewrite_headers = ob->rewrite_headers;
+  if (ob->rewrite_headers) setflag(addr, af_rewrite_headers);
   search_find_defer = FALSE;
   if (rc == HOST_FOUND_LOCAL ||
       match_isinlist(fully_qualified_name, &local_domains, TRUE, TRUE, NULL))
     {
-    addr->rewrite_headers = TRUE;
+    setflag(addr, af_rewrite_headers);
     return ISLOCAL;
     }
   else if (search_find_defer)
@@ -317,10 +329,10 @@ if (rc == HOST_FOUND_LOCAL)
 
 /* Otherwise, insist on being a secondary MX if so configured */
 
-else if (ob->check_secondary_mx && !addr->local_host_removed)
+else if (ob->check_secondary_mx && !testflag(addr, af_local_host_removed))
   {
   DEBUG(2) debug_printf("check_secondary_mx set and local host not secondary\n");
-  return FAIL;
+  return DECLINE;
   }
 
 /* Set up the errors address, if any. The function uses only fields that
