@@ -29,7 +29,7 @@ extern lookuphost_router_options_block lookuphost_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int lookuphost_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL);
+  address_item **, address_item **, BOOL, char *);
 
 extern void lookuphost_router_init(router_instance *);
 
