@@ -89,7 +89,8 @@ queryprogram_router_entry(
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
   address_item **addr_remote,     /* add it to this if successful & remote */
-  BOOL verify)                    /* TRUE if verifying */
+  BOOL verify,                    /* TRUE if verifying */
+  char *ignore_target_hosts)      /* ignore host list (expanded) */
 {
 int fd_in, fd_out, i, len, rc;
 pid_t pid;
@@ -358,11 +359,14 @@ if (returned[2][0] != 0 && strcmp(returned[2], "+") != 0)
     h->next = NULL;
 
     if (lookup_type == lk_byname)
-      rc = host_find_byname(h, &canonical_name, FALSE);
+      rc = host_find_byname(h, ignore_target_hosts, &canonical_name, FALSE);
     else
-      rc = host_find_bydns(h, (lookup_type == lk_bydns_mx),
-        (lookup_type == lk_bydns_a), FALSE, FALSE, &canonical_name,
-        &(addr->local_host_removed));
+      {
+      BOOL removed;
+      rc = host_find_bydns(h, ignore_target_hosts, (lookup_type == lk_bydns_mx),
+        (lookup_type == lk_bydns_a), FALSE, FALSE, &canonical_name, &removed);
+      if (removed) setflag(addr, af_local_host_removed);
+      }
 
     if (rc == HOST_FIND_AGAIN)
       {
