@@ -97,13 +97,14 @@ Returns:     -1 stat() or fstat() failed
               0 OK
              +1 something didn't match
 
-Side effect: sets errno to ERRNO_BADUGID or ERRNO_BADMODE for bad uid/gid
-             or bad mode; otherwise leaves it to what fstat set it to.
+Side effect: sets errno to ERRNO_BADUGID, ERRNO_NOTREGULAR or ERRNO_BADMODE for
+             bad uid/gid, not a regular file, or bad mode; otherwise leaves it
+             to what fstat set it to.
 */
 
 int
-search_check_file(int fd, char *filename, int modemask, int *owners,
-  int *owngroups, char *type, char **errmsg)
+search_check_file(int fd, char *filename, int modemask, uid_t *owners,
+  gid_t *owngroups, char *type, char **errmsg)
 {
 int i;
 struct stat statbuf;
@@ -117,8 +118,15 @@ if ((fd >= 0 && fstat(fd, &statbuf) != 0) ||
   return -1;
   }
 
-if ((statbuf.st_mode & S_IFMT) != S_IFREG ||
-    (statbuf.st_mode & modemask) != 0)
+if ((statbuf.st_mode & S_IFMT) != S_IFREG)
+  {
+  *errmsg = string_sprintf("%s is not a regular file (%s lookup)",
+    filename, type);
+  errno = ERRNO_NOTREGULAR;
+  return +1;
+  }
+
+if ((statbuf.st_mode & modemask) != 0)
   {
   *errmsg = string_sprintf("%s (%s lookup): file mode %.4o should not contain "
     "%.4o", filename, type,  statbuf.st_mode & 07777,
@@ -280,8 +288,8 @@ Returns:         an identifying handle for the open database;
 */
 
 void *
-search_open(char *filename, int search_type, int modemask, int *owners,
-  int *owngroups, char **message)
+search_open(char *filename, int search_type, int modemask, uid_t *owners,
+  gid_t *owngroups, char **message)
 {
 void *handle;
 tree_node *t;
