@@ -226,6 +226,7 @@ header_list = header_last = NULL;
 sender_address = NULL;
 sender_host_address = NULL;
 sender_host_name = NULL;
+sender_host_port = 0;
 sender_host_authenticated = NULL;
 interface_address = NULL;
 sender_helo_name = NULL;
@@ -234,6 +235,11 @@ sender_ident = NULL;
 recipients_list = NULL;
 tree_nonrecipients = NULL;
 
+#ifdef SUPPORT_TLS
+tls_cipher = NULL;
+tls_peerdn = NULL;
+#endif
+
 /* Generate the full name and open the file. If message_subdir is already
 set, just look in the given directory. Otherwise, look in both the split
 and unsplit directories, as for the data file above. */
@@ -304,7 +310,7 @@ deliver_freeze = FALSE;
 deliver_frozen_at = 0;
 host_lookup_failed = FALSE;
 local_error_message = FALSE;
-user_null_sender = FALSE;
+sender_set_untrusted = FALSE;
 
 #ifdef SUPPORT_DSN
 /* This partial attempt at doing DSN was abandoned. This obsolete code is
@@ -337,12 +343,19 @@ for (;;)
       deliver_firsttime = TRUE;
     else if (strcmp(big_buffer, "-manual_thaw") == 0)
       deliver_manual_thaw = TRUE;
-    else if (strcmp(big_buffer, "-user_null_sender") == 0)
-      user_null_sender = TRUE;
     else if (strncmp(big_buffer, "-auth_id", 8) == 0)
       authenticated_id = string_copy(big_buffer + 9);
     else if (strncmp(big_buffer, "-auth_sender", 12) == 0)
       authenticated_sender = string_copy(big_buffer + 13);
+    else if (strncmp(big_buffer, "-sender_set_untrusted", 21) == 0)
+      sender_set_untrusted = TRUE;
+
+    #ifdef SUPPORT_TLS
+    else if (strncmp(big_buffer, "-tls_cipher", 11) == 0)
+      tls_cipher = string_copy(big_buffer + 12);
+    else if (strncmp(big_buffer, "-tls_peerdn", 11) == 0)
+      tls_peerdn = string_copy(big_buffer + 12);
+    #endif
 
     #ifdef SUPPORT_DSN
     /* This partial attempt at doing DSN was abandoned. This obsolete code is
@@ -354,8 +367,16 @@ for (;;)
       dsn_envid = string_copy(big_buffer + 11);
     #endif
 
+    /* We now record the port number after the address, separated by a
+    dot. For compatibility during upgrading, do nothing if there
+    isn't a value (it gets left at zero). */
+
     else if (strncmp(big_buffer, "-host_address", 13) == 0)
+      {
+      sender_host_port = host_extract_port(big_buffer + 14);
       sender_host_address = string_copy(big_buffer + 14);
+      }
+
     else if (strncmp(big_buffer, "-host_auth", 10) == 0)
       sender_host_authenticated = string_copy(big_buffer + 11);
     else if (strncmp(big_buffer, "-host_name", 10) == 0)
@@ -368,6 +389,8 @@ for (;;)
       sender_ident = string_copy(big_buffer + 7);
     else if (strncmp(big_buffer, "-received_protocol", 18) == 0)
       received_protocol = string_copy(big_buffer + 19);
+    else if (strncmp(big_buffer, "-N", 2) == 0)
+      dont_deliver = TRUE;
 
   /* To allow new versions of Exim that add additional flags to interwork
   with older versions that do not understand them, just ignore any flagged
