@@ -148,7 +148,7 @@ if (sender_helo_name != NULL)
 
 if (sender_host_address != NULL)
   {
-  fprintf(f, "-host_address %s\n", sender_host_address);
+  fprintf(f, "-host_address %s.%d\n", sender_host_address, sender_host_port);
   if (sender_host_name != NULL)
     fprintf(f, "-host_name %s\n", sender_host_name);
   if (sender_host_authenticated != NULL)
@@ -181,12 +181,18 @@ if (authenticated_sender != NULL)
   fprintf(f, "-auth_sender %s\n", authenticated_sender);
 if (deliver_firsttime) fprintf(f, "-deliver_firsttime\n");
 if (deliver_freeze) fprintf(f, "-frozen %d\n", deliver_frozen_at);
+if (dont_deliver) fprintf(f, "-N\n");
 if (host_lookup_failed) fprintf(f, "-host_lookup_failed\n");
 if (sender_local) fprintf(f, "-local\n");
 if (local_error_message) fprintf(f, "-localerror\n");
 if (deliver_manual_thaw) fprintf(f, "-manual_thaw\n");
 if (header_names != header_names_normal) fprintf(f, "-resent\n");
-if (user_null_sender) fprintf(f, "-user_null_sender\n");
+if (sender_set_untrusted) fprintf(f, "-sender_set_untrusted\n");
+
+#ifdef SUPPORT_TLS
+if (tls_cipher != NULL) fprintf(f, "-tls_cipher %s\n", tls_cipher);
+if (tls_peerdn != NULL) fprintf(f, "-tls_peerdn %s\n", tls_peerdn);
+#endif
 
 #ifdef SUPPORT_DSN
 /* This partial attempt at doing DSN was abandoned. This obsolete code is
