@@ -116,8 +116,9 @@ if (strchr(s, ':') != NULL)
     /* Check for at least one and not more than 4 hex digits for this
     component. */
 
-    if (!isxdigit(*s++)) return 0;
-    if (isxdigit(*s) && isxdigit(*(++s)) && isxdigit(*(++s))) s++;
+    if (!isxdigit((uschar)*s++)) return 0;
+    if (isxdigit((uschar)*s) && isxdigit((uschar)*(++s)) &&
+      isxdigit((uschar)*(++s))) s++;
 
     /* If the component is terminated by colon and there is more to
     follow, skip over the colon. If there is no more to follow the address is
@@ -530,8 +531,8 @@ va_list ap;
 char buffer[STRING_SPRINTF_BUFFER_SIZE];
 va_start(ap, format);
 if (!string_vformat(buffer, sizeof(buffer), format, ap))
-  log_write(0, LOG_PANIC_DIE, "string_sprintf expansion was longer than %d",
-    sizeof(buffer));
+  log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+    "string_sprintf expansion was longer than %d", sizeof(buffer));
 va_end(ap);
 return string_copy(buffer);
 }
@@ -1069,7 +1070,7 @@ else
 there is at least one intermediate parent, show it in brackets, and continue
 with all of them if all are wanted. */
 
-if ((all_parents || addr->pfr) &&
+if ((all_parents || testflag(addr, af_pfr)) &&
     addr->parent != NULL &&
     addr->parent != topaddr)
   {
@@ -1162,7 +1163,7 @@ while (fgets(buffer, 256, stdin) != NULL)
     s = strchr(ss, ',');
     if (s == NULL) s = ss + (int)strlen(ss);
 
-    if (isdigit(*ss))
+    if (isdigit((uschar)*ss))
       {
       strncpy(outbuf, ss, s-ss);
       if (strchr(outbuf, '.') != NULL)
