@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,6 +14,8 @@ struct address_item;
 struct auth_info;
 struct driver_info;
 struct director_info;
+struct smtp_inblock;
+struct smtp_outblock;
 struct transport_info;
 struct router_info;
 
@@ -63,6 +65,16 @@ typedef struct host_item {
   char *ident_string;            /* RFC 1413 ident string */
 } host_item;
 
+/* Chain of rewrite rules, read from the rewrite config, or parsed from the
+rewrite_headers field of a transport. */
+
+typedef struct rewrite_rule {
+  struct rewrite_rule *next;
+  int flags;
+  char *key;
+  char *replacement;
+} rewrite_rule;
+
 /* Directors, routers, and transports have similar data blocks. Each driver
 that is compiled into the code is represented by a xxx_info block; the active
 drivers are represented by a chain of xxx_instance blocks. To make it possible
@@ -135,8 +147,11 @@ typedef struct transport_instance {
   char *filter_command;          /* For on-the-fly-filtering */
   char *add_headers;             /* Add these headers */
   char *remove_headers;          /* Remove these headers */
-  char *return_path;             /* Overriding return path */
+  char *return_path;             /* Overriding (rewriting) return path */
   char *debug_string;            /* Debugging output */
+  char *headers_rewrite;         /* Rules for rewriting headers */
+  rewrite_rule *rewrite_rules;   /* Parsed rewriting rules */
+  int   rewrite_existflags;      /* Bits showing which headers are rewritten */
   BOOL  body_only;               /* Deliver only the body */
   BOOL  delivery_date_add;       /* Add Delivery-Date header */
   BOOL  envelope_to_add;         /* Add Envelope-To header */
@@ -157,7 +172,7 @@ typedef struct transport_instance {
 
 typedef struct transport_info {
   char *driver_name;             /* Driver name */
-  void (*code)(
+  BOOL (*code)(
     transport_instance *,
     struct address_item *);
   void (*init)(
@@ -227,6 +242,7 @@ typedef struct director_instance {
   BOOL  fail_verify_sender;      /* Fail verify if sender match this director */
   BOOL  fail_verify_recipient;   /* Fail verify if recipient match this director */
   BOOL  expn;                    /* Use this director when processing EXPN */
+  BOOL  one_continue;            /* Continue verify if generated just one child */
 } director_instance;
 
 
@@ -303,6 +319,7 @@ typedef struct router_instance {
   BOOL  self_rewrite;            /* TRUE to rewrite headers if making local */
   BOOL  sender_rewrite;          /* TRUE to rewrite verified sender */
   char *translate_ip_address;    /* IP address translation fudgery */
+  char *ignore_target_hosts;     /* Target hosts to ignore */
 } router_instance;
 
 
@@ -315,7 +332,8 @@ typedef struct router_info {
     struct address_item *,
     struct address_item **,
     struct address_item **,
-    BOOL);
+    BOOL,
+    char *);
   void (*init)(
     struct router_instance *);
   void (*tidyup)(
@@ -339,8 +357,8 @@ typedef struct lookup_info {
     void *,                      /* handle */
     char *,                      /* file name */
     int,                         /* modemask for file checking */
-    int *,                       /* owners for file checking */
-    int *,                       /* owngroups for file checking */
+    uid_t *,                     /* owners for file checking */
+    gid_t *,                     /* owngroups for file checking */
     char **);                    /* for error messages */
   int (*find)(                   /* find function */
     void *,                      /* handle */
@@ -384,7 +402,8 @@ typedef struct auth_info {
     struct auth_instance *);
   int (*clientcode)(             /* client function */
     struct auth_instance *,
-    int,                         /* socket */
+    struct smtp_inblock *,       /* socket and input buffer */
+    struct smtp_outblock *,      /* socket and output buffer */
     int,                         /* command timeout */
     char *,                      /* buffer for reading response */
     int);                        /* sizeof buffer */
@@ -475,7 +494,9 @@ some only for local. */
 typedef struct address_item {
   struct address_item *next;     /* for chaining addresses */
   struct address_item *parent;   /* parent address */
+  struct address_item *shadow_parent; /* parent when shadow delivering */
   struct address_item *first;    /* point to first after group delivery */
+  unsigned int flags;            /* a row of bits, defined below */
   char *onetime_parent;          /* saved original parent for onetime */
   char *orig;                    /* as read from the spool */
   char *unique;                  /* used for disambiguating .forwarded */
@@ -486,16 +507,10 @@ typedef struct address_item {
   char *route_domain;            /* domain to be used by routers */
   char *errors_address;          /* where to send errors (NULL => sender) */
   char *self_hostname;           /* after routing->directors */
-  BOOL pfr;                      /* pipe or file or reply delivery */
-  BOOL rewrite_headers;          /* set TRUE by routers that want it done */
-  BOOL local;                    /* local/remote flag */
-  BOOL local_part_quoted;        /* local part was quoted */
-  BOOL forced_local;             /* by a self option on a router */
-  BOOL delivered;                /* already delivered */
-  BOOL ignore_error;             /* ignore delivery error */
-  BOOL local_host_removed;       /* local host was backup */
   header_line *extra_headers;    /* additional headers */
   char *remove_headers;          /* list of those to remove */
+  char *domain_data;             /* from "domains" lookup */
+  char *local_part_data;         /* from "local_part" lookup */
   director_instance *start_director; /* generated address starts here */
   director_instance *director;   /* the director that directed */
   router_instance *router;       /* OR the router that routed */
@@ -506,18 +521,11 @@ typedef struct address_item {
   int  child_count;              /* number of child addresses */
   uid_t uid;                     /* uid for local transporting */
   gid_t gid;                     /* gid for local transporting */
-  BOOL uid_set;                  /* Flags for setting */
-  BOOL gid_set;                  /* Flags for setting */
   int  mode;                     /* mode for local transporting to a file */
   char *home_dir;                /* home directory for local transporting */
   char *current_dir;             /* current directory for local transporting */
   char *route_option;            /* routing option string */
   char **pipe_expandn;           /* numeric expansions for pipe from filter */
-  BOOL initgroups;               /* use initgroups() for local transporting */
-  BOOL expand_pipe;              /* expand pipe arguments */
-  BOOL allow_pipe;               /* allow pipe in generated address */
-  BOOL allow_file;               /* allow file in generated address */
-  BOOL allow_reply;              /* allow autoreply in generated address */
   int  transport_return;         /* result of delivery attempt */
   int  basic_errno;              /* status after failure */
   int  more_errno;               /* additional error information */
@@ -525,19 +533,47 @@ typedef struct address_item {
   char *shadow_message;          /* info about shadow transporting */
   int  special_action;           /* used when when deferred or failed; also */
                                  /* contains = or - when successful SMTP delivered */
-  BOOL dr_retry_exists;          /* director or router retry record exists */
-  BOOL retry_skipped;            /* true if retry caused some skipping */
-  BOOL retry_timedout;           /* true if retry timed out */
   retry_item *retries;           /* chain of retry information */
+  char *local_retry_key;         /* key for local transport retries */
   char *return_filename;         /* ) some transports write text into a file */
   int   return_file;             /* ) for return to the sender */
   reply_item *reply;             /* and some generate reply messages */
+
+  #ifdef SUPPORT_TLS
+  char *cipher;                  /* Cipher used for transport */
+  char *peerdn;                  /* DN of server's certificate */
+  #endif
+
+  /* These are the remains of an attempt to put in some DSN support that
+  was abandoned. */
+
   #ifdef SUPPORT_DSN
   char *dsn_orcpt;               /* DSN orcpt value */
   int  dsn_flags;                /* DSN flags */
   #endif
 } address_item;
 
+/* Bits for the flags field above */
+
+#define af_allow_file          0x00000001 /* allow file in generated address */
+#define af_allow_pipe          0x00000002 /* allow pipe in generated address */
+#define af_allow_reply         0x00000004 /* allow autoreply in generated address */
+#define af_dr_retry_exists     0x00000008 /* director or router retry record exists */
+#define af_expand_pipe         0x00000010 /* expand pipe arguments */
+#define af_forced_local        0x00000020 /* by a self option on a router */
+#define af_gid_set             0x00000040 /* gid field is set */
+#define af_ignore_error        0x00000080 /* ignore delivery error */
+#define af_initgroups          0x00000100 /* use initgroups() for local transporting */
+#define af_local               0x00000200 /* local/remote flag */
+#define af_local_host_removed  0x00000400 /* local host was backup */
+#define af_local_part_quoted   0x00000800 /* local part was quoted */
+#define af_lt_retry_exists     0x00001000 /* local transport retry exists */
+#define af_pfr                 0x00002000 /* pipe or file or reply delivery */
+#define af_retry_skipped       0x00004000 /* true if retry caused some skipping */
+#define af_retry_timedout      0x00008000 /* true if retry timed out */
+#define af_rewrite_headers     0x00010000 /* set TRUE by routers that want it done */
+#define af_uid_set             0x00020000 /* uid field is set */
+#define af_hide_child          0x00040000 /* hide child in bounce/defer msgs */
 
 /* Tables of normal and resent- header names consist of items of this type */
 
@@ -580,15 +616,6 @@ typedef struct retry_config {
   retry_rule *rules;
 } retry_config;
 
-/* Chain of rewrite rules, read from the rewrite config. */
-
-typedef struct rewrite_rule {
-  struct rewrite_rule *next;
-  int flags;
-  char *key;
-  char *replacement;
-} rewrite_rule;
-
 /* Structure for each node in a tree, of which there are various kinds */
 
 typedef struct tree_node {
@@ -624,9 +651,37 @@ typedef struct {
   char  name[DNS_MAXNAME];        /* domain name */
   int   type;                     /* record type */
   int   size;                     /* size of data */
-  u_char *data;                   /* pointer to data */
+  uschar *data;                   /* pointer to data */
 } dns_record;
 
+/* Structure for holding the result of a DNS query. */
+
+typedef struct {
+  int   answerlen;                /* length of the answer */
+  uschar answer[MAXPACKET];       /* the answer itself */
+} dns_answer;
+
+/* Structure for holding the intermediate data while scanning a DNS answer
+block. */
+
+typedef struct {
+  int   rrcount;                  /* count of RRs in the answer */
+  uschar *aptr;                   /* pointer in the answer while scanning */
+  dns_record srr;                 /* data from current record in scan */
+} dns_scan;
+
+/* Structure for holding a chain of IP addresses that are extracted from
+an A, AAAA, or A6 record. For the first two, there is only ever one address,
+but the chaining feature of A6 allows for several addresses to be realized from
+a single initial A6 record. The structure defines the address field of length
+1. In use, a suitable sized block is obtained to hold the complete textual
+address. */
+
+typedef struct dns_address {
+  struct dns_address *next;
+  uschar address[1];
+} dns_address;
+
 /* Structure used for holding intermediate data during MD5 computations. */
 
 typedef struct md5 {
@@ -635,4 +690,28 @@ typedef struct md5 {
   }
 md5;
 
+/* Structure used to hold incoming packets of SMTP responses for a specific
+socket. The packets which may contain multiple lines (and in some cases,
+multiple responses). */
+
+typedef struct smtp_inblock {
+  int sock;                       /* the socket */
+  int buffersize;                 /* the size of the buffer */
+  uschar *ptr;                    /* current position in the buffer */
+  uschar *ptrend;                 /* end of data in the buffer */
+  uschar *buffer;                 /* the buffer itself */
+} smtp_inblock;
+
+/* Structure used to hold buffered outgoing packets of SMTP commands for a
+specific socket. The packets which may contain multiple lines when pipelining
+is in use. */
+
+typedef struct smtp_outblock {
+  int sock;                       /* the socket */
+  int cmd_count;                  /* count of buffered commands */
+  int buffersize;                 /* the size of the buffer */
+  uschar *ptr;                    /* current position in the buffer */
+  uschar *buffer;                 /* the buffer itself */
+} smtp_outblock;
+
 /* End of structs.h */
