@@ -28,6 +28,10 @@ are other options living inside this structure which can be set only from
 certain transports. */
 
 optionlist optionlist_transports[] = {
+  { "*headers_rewrite_flags", opt_int|opt_public|opt_hidden,
+                 (void *)offsetof(transport_instance, rewrite_existflags) },
+  { "*headers_rewrite_rules", opt_void|opt_public|opt_hidden,
+                 (void *)offsetof(transport_instance, rewrite_rules) },
   { "add_headers",      opt_stringptr|opt_public|opt_hidden,
                  (void *)offsetof(transport_instance, add_headers) },
   { "body_only",        opt_bool|opt_public,
@@ -46,6 +50,8 @@ optionlist optionlist_transports[] = {
                  (void *)offsetof(transport_instance, headers_only) },
   { "headers_remove",   opt_stringptr|opt_public,
                  (void *)offsetof(transport_instance, remove_headers) },
+  { "headers_rewrite",  opt_rewrite|opt_public,
+                 (void *)offsetof(transport_instance, headers_rewrite) },
   { "message_size_limit", opt_int|opt_public,
                  (void *)offsetof(transport_instance, message_size_limit) },
   { "remove_headers",   opt_stringptr|opt_public|opt_hidden,
@@ -174,6 +180,11 @@ for (i = 0; i < 100; i++)
     debug_printf("writing data block fd=%d size=%d timeout=%d\n",
       fd, len, transport_write_timeout);
   if (transport_write_timeout > 0) alarm(transport_write_timeout);
+
+  #ifdef SUPPORT_TLS
+  if (tls_active == fd) rc = tls_write(block, len); else
+  #endif
+
   rc = write(fd, block, len);
   save_errno = errno;
 
@@ -407,8 +418,7 @@ are in the in-store data structure, and the rest of the message is in the open
 file descriptor deliver_datafile. Make sure we start it at the beginning.
 
 . If add_return_path is TRUE, a "return-path:" header is added to the message,
-  containing the sender's address, unless it is null, or the user_null_sender
-  flag is set (indicating an error message), in which case just "<>" is given.
+  containing the envelope sender's address.
 
 . If add_envelope_to is TRUE, a "envelope-to:" header is added to the message,
   giving the top-level envelope address that caused this delivery to happen.
@@ -436,7 +446,7 @@ Arguments:
     add_envelope_to       if TRUE, add a "envelope-to" header
     add_delivery_date     if TRUE, add a "delivery-date" header
     use_crlf              if TRUE, turn NL into CR LF
-    dot_end               if TRUE, send a terminating "." line at the end
+    end_dot               if TRUE, send a terminating "." line at the end
     no_headers            if TRUE, omit the headers
     no_body               if TRUE, omit the body
   size_limit            if > 0, this is a limit to the size of message written;
@@ -450,6 +460,8 @@ Arguments:
   remove_headers        a colon-separated list of headers to remove, or NULL
   check_string          a string to check for at the start of lines, or NULL
   escape_string         a string to insert in front of any check string
+  rewrite_rules         chain of header rewriting rules
+  rewrite_existflags    flags for the rewriting rules
 
 Returns:                TRUE on success; FALSE (with errno) on failure.
                         In addition, the global variable transport_count
@@ -459,7 +471,7 @@ Returns:                TRUE on success; FALSE (with errno) on failure.
 static BOOL
 internal_transport_write_message(address_item *addr, int fd, int options,
   int size_limit, char *add_headers, char *remove_headers, char *check_string,
-  char *escape_string)
+  char *escape_string, rewrite_rule *rewrite_rules, int rewrite_existflags)
 {
 int written = 0;
 int len;
@@ -488,7 +500,8 @@ after the headers. */
 
 if ((options & topt_escape_headers) == 0) nl_check_length = -nl_check_length;
 
-/* Write the headers if required, including any that have to be added */
+/* Write the headers if required, including any that have to be added. If there
+are header rewriting rules, apply them. */
 
 if ((options & topt_no_headers) == 0)
   {
@@ -498,7 +511,7 @@ if ((options & topt_no_headers) == 0)
     {
     char buffer[SENDER_ADDRESS_MAXLENGTH + 20];
     sprintf(buffer, "Return-path: <%.*s>\n", SENDER_ADDRESS_MAXLENGTH,
-      user_null_sender? "" : return_path);
+      return_path);
     len = (int)strlen(buffer);
     if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
     }
@@ -534,7 +547,7 @@ if ((options & topt_no_headers) == 0)
           }
         }
 
-      /* Now list the parent of this address */
+      /* Now list the ultimate parent of this address */
 
       for (pp = p; pp->parent != NULL; pp = pp->parent);
       strncpy(buffer + len, pp->orig, 200);
@@ -567,9 +580,9 @@ if ((options & topt_no_headers) == 0)
     if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
     }
 
-  /* Then the message's headers. Don't write any that are flagged as "old"; that
-  means they were rewritten, or are a record of envelope rewriting, or were
-  removed (e.g. Bcc). If remove_headers is not null, skip any headers that
+  /* Then the message's headers. Don't write any that are flagged as "old";
+  that means they were rewritten, or are a record of envelope rewriting, or
+  were removed (e.g. Bcc). If remove_headers is not null, skip any headers that
   match any entries therein. Then check addr->remove_headers too, provided that
   addr is not NULL. */
 
@@ -616,8 +629,31 @@ if ((options & topt_no_headers) == 0)
       if (addr != NULL) list = addr->remove_headers;
       }
 
-    if (include_header && !write_chunk(fd, h->text, h->slen, use_crlf))
-      return FALSE;
+    /* If this header is to be output, try to rewrite it if there are rewriting
+    rules. */
+
+    if (include_header)
+      {
+      if (rewrite_rules != NULL)
+        {
+        void *reset_point = store_get(0);
+        header_line *hh =
+          rewrite_header(h, NULL, NULL, rewrite_rules, rewrite_existflags,
+            FALSE);
+        if (hh != NULL)
+          {
+          if (!write_chunk(fd, hh->text, hh->slen, use_crlf))
+            return FALSE;
+          store_reset(reset_point);
+          continue;     /* With the next header line */
+          }
+        }
+
+      /* Either no rewriting rules, or it didn't get rewritten */
+
+      if (!write_chunk(fd, h->text, h->slen, use_crlf))
+        return FALSE;
+      }
     }
 
   /* Add on any address-specific headers. If there are multiple addresses,
@@ -743,9 +779,10 @@ Returns:       TRUE on success; FALSE (with errno) for any failure
 BOOL
 transport_write_message(address_item *addr, int fd, int options,
   int size_limit, char *add_headers, char *remove_headers, char *check_string,
-  char *escape_string)
+  char *escape_string, rewrite_rule *rewrite_rules, int rewrite_existflags)
 {
 BOOL use_crlf;
+BOOL last_filter_was_NL;
 int rc, len, yield, fd_read, fd_write, save_errno;
 int pfd[2];
 pid_t filter_pid, write_pid;
@@ -755,7 +792,8 @@ the actual work, passing it the incoming fd. */
 
 if (transport_filter_argv == NULL)
   return internal_transport_write_message(addr, fd, options, size_limit,
-    add_headers, remove_headers, check_string, escape_string);
+    add_headers, remove_headers, check_string, escape_string,
+    rewrite_rules, rewrite_existflags);
 
 /* Otherwise the message must be written through a filter. Start up a
 subprocess to run the command. First ensure that our main fd will be closed in
@@ -803,7 +841,8 @@ if ((write_pid = fork()) == 0)
   nl_check_length = nl_escape_length = 0;
   rc = internal_transport_write_message(addr, fd_write,
     (options & ~(topt_use_crlf | topt_end_dot)),
-    size_limit, add_headers, remove_headers, NULL, NULL);
+    size_limit, add_headers, remove_headers, NULL, NULL,
+    rewrite_rules, rewrite_existflags);
   save_errno = errno;
   write(pfd[pipe_write], (void *)&rc, sizeof(BOOL));
   write(pfd[pipe_write], (void *)&save_errno, sizeof(int));
@@ -851,8 +890,10 @@ else nl_check_length = nl_escape_length = 0;
 transport_count = 0;
 chunk_ptr = deliver_out_buffer;
 
-/* Copy the output of the filter */
+/* Copy the output of the filter, remembering if the last character was NL. If
+no data is returned, that counts as "ended with NL". */
 
+last_filter_was_NL = TRUE;
 for (;;)
   {
   if (transport_write_timeout <= 0)  /* Yes, that is "<=" see above */
@@ -881,6 +922,7 @@ for (;;)
     save_errno = errno;
     goto TIDY_UP;
     }
+  last_filter_was_NL = (deliver_in_buffer[len-1] == '\n');
   }
 
 /* A non-zero value for len implies an error */
@@ -888,11 +930,14 @@ for (;;)
 if (len != 0) { save_errno = errno; goto TIDY_UP; }
 
 /* If requested, add a terminating "." line (SMTP output), turning off
-escaping beforehand. */
+escaping beforehand. If the last character from the filter was not NL,
+insert an NL to make the SMTP protocol work. */
 
 nl_check_length = nl_escape_length = 0;
 
-if ((options & topt_end_dot) != 0 && !write_chunk(fd, ".\n", 2, use_crlf))
+if ((options & topt_end_dot) != 0 && (last_filter_was_NL?
+      !write_chunk(fd, ".\n", 2, use_crlf) :
+      !write_chunk(fd, "\n.\n", 3, use_crlf)))
   {
   save_errno = errno;
   goto TIDY_UP;
@@ -1018,13 +1063,8 @@ open_db *dbm_file;
 /* Open the database for this transport */
 
 sprintf(buffer, "wait-%.200s", tpname);
-dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock);
-if (dbm_file == NULL)
-  {
-  log_write(0, LOG_MAIN, "Failed to open %s database: %s", buffer,
-    strerror(errno));
-  return;
-  }
+dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock, TRUE);
+if (dbm_file == NULL) return;
 
 /* Scan the list of hosts for which this message is waiting, and ensure
 that the message id is in each host record for those that have the
@@ -1189,13 +1229,8 @@ if ((local_batch_max > 0 && continue_sequence >= local_batch_max) ||
 /* Open the waiting information database. */
 
 sprintf(buffer, "wait-%.200s", transport_name);
-dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock);
-if (dbm_file == NULL)
-  {
-  log_write(0, LOG_MAIN, "Failed to open %s database: %s", buffer,
-    strerror(errno));
-  return FALSE;
-  }
+dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock, TRUE);
+if (dbm_file == NULL) return FALSE;
 
 /* See if there is a record for this host; if not, there's nothing to do. */
 
@@ -1356,7 +1391,7 @@ int status;
 if ((pid = vfork()) == 0)
   {
   int i;
-  char *argv[17 + MAX_CLMACROS];
+  char *argv[19 + MAX_CLMACROS];
 
   /* Disconnect entirely from the parent process. */
 
@@ -1394,7 +1429,12 @@ if ((pid = vfork()) == 0)
   if (smtp_authenticated) argv[i++] = "-MCA";
   #endif
 
+  #ifdef SUPPORT_TLS
+  if (tls_offered) argv[i++] = "-MCT";
+  #endif
+
   if (smtp_use_size) argv[i++] = "-MCS";
+  if (smtp_use_pipelining) argv[i++] = "-MCP";
 
   if (queue_run_pid != (pid_t)0)
     {
@@ -1483,12 +1523,8 @@ DEBUG(4) debug_printf("transport_check_serialized entered: %s %s\n",
 /* Open and lock the waiting information database. */
 
 sprintf(buffer, "serialize-%.200s", dbname);
-dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock);
-if (dbm_file == NULL)
-  {
-  log_write(0, LOG_MAIN, "Failed to open %s database", buffer);
-  return FALSE;
-  }
+dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock, TRUE);
+if (dbm_file == NULL) return FALSE;
 
 /* See if there is a record for this host or queue run; if there is, we cannot
 proceed with the connection unless the record is very old. */
@@ -1537,13 +1573,8 @@ DEBUG(4) debug_printf("transport_end_serialized entered; %s %s\n",
   dbname, qname);
 
 sprintf(buffer, "serialize-%.200s", dbname);
-dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock);
-if (dbm_file == NULL && errno != ENOENT)
-  {
-  log_write(0, LOG_MAIN, "Failed to open %s database: %s", buffer,
-    strerror(errno));
-  return;
-  }
+dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock, TRUE);
+if (dbm_file == NULL) return;
 
 dbfn_delete(dbm_file, qname);
 dbfn_close(dbm_file);
