@@ -127,6 +127,8 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(appendfile_transport_options_block, quota)) },
   { "quota_filecount",   opt_int,
       (void *)(offsetof(appendfile_transport_options_block, quota_filecount)) },
+  { "quota_is_inclusive", opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, quota_is_inclusive)) },
   {"quota_size_regex",   opt_stringptr,
       (void *)(offsetof(appendfile_transport_options_block, quota_size_regex)) },
   { "quota_warn_message", opt_stringptr | opt_public,
@@ -211,7 +213,8 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   FALSE,          /* maildir_format */
   FALSE,          /* mailstore_format */
   FALSE,          /* mbx_format */
-  FALSE           /* quota_warn_threshold_is_percent */
+  FALSE,          /* quota_warn_threshold_is_percent */
+  TRUE            /* quota_is_inclusive */
 };
 
 
@@ -435,7 +438,7 @@ if (ob->quota_warn_threshold != NULL)
   if (tblock->warn_message == NULL) tblock->warn_message =
     "To: $local_part@$domain\n"
     "Subject: Your mailbox\n\n"
-    "This message is automatically created by mail delivery software.\n\n"
+    "This message is automatically created by mail delivery software (Exim).\n\n"
     "The size of your mailbox has exceeded a warning threshold that is\n"
     "set by the system administrator.\n";
   }
@@ -836,7 +839,9 @@ return OK;
 *              Main entry point                  *
 *************************************************/
 
-/* See local README for general interface details.
+/* See local README for general interface details. This transport always
+returns FALSE, indicating that the status that has been placed in the first
+address should be copied to any other addresses in a batch.
 
 Appendfile delivery is tricky and has led to various security problems in other
 mailers. The logic used here is therefore laid out in some detail. When this
@@ -851,7 +856,7 @@ because in weird configurations not running setuid root there may be a
 difference. In the standard configuration, where setuid() has been used in the
 delivery process, there will be no difference between the uid and the euid.
 
-(1)  If the addr->pfr flag is set and the local_part field starts with '/' then
+(1)  If the af_pfr flag is set and the local_part field starts with '/' then
      this is a delivery to a file after .forward or alias expansion. Otherwise,
      there must be a configured file name or directory name.
 
@@ -989,7 +994,7 @@ forwarding operation and attempting to open it gives EPERM, ENOENT, or EACCES.
 All other failures return DEFER (in addr->transport_return). */
 
 
-void
+BOOL
 appendfile_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address we are working on */
@@ -1034,7 +1039,7 @@ deliver to files specified via .forward or an alias file. If it is, assume
 a directory-style delivery if the last character is '/'. (For address_file
 this shouldn't ever be the case.) */
 
-if (addr->pfr && addr->local_part[0] == '/')
+if (testflag(addr, af_pfr) && addr->local_part[0] == '/')
   {
   path = addr->local_part;
   if (path[(int)strlen(path)-1] == '/') isdirectory = TRUE;
@@ -1057,7 +1062,7 @@ else
     addr->transport_return = PANIC;
     addr->message = string_sprintf("Mandatory file or directory option "
       "missing from %s transport", tblock->name);
-    return;
+    return FALSE;
     }
 
   path = expand_string(fdname);
@@ -1068,30 +1073,9 @@ else
     addr->message = string_sprintf("Expansion of \"%s\" (file or directory "
       "name for %s transport) failed: %s", fdname, tblock->name,
       expand_string_message);
-    return;
+    return FALSE;
     }
 
-  /* If the path contained a reference to $local_part, check that it
-  doesn't attempt to change directory. Note that deliver_localpart may be
-  unset during some deliveries (e.g. batched SMTP). */
-
-  /**** This check is probably not very helpful; people don't usually have
-  slashes in user names, and if they are processing random local parts they
-  can either check explicitly, or just let it fail because the directory
-  doesn't exist.
-
-  if (deliver_localpart != NULL &&
-      strchr(deliver_localpart, '/') != NULL &&
-     (strstr(fdname, "${local_part}") != NULL ||
-      strstr(fdname, "$local_part") != NULL))
-    {
-    addr->message = "appendfile: user name contains /";
-    addr->basic_errno = ERRNO_USERSLASH;
-    addr->special_action = SPECIAL_FREEZE;
-    return;
-    }
-  ****/
-
   /* Check that the expanded name is absolute. This check is needed in case
   it started with an expansion item. If not, the check is done at
   initialization time. */
@@ -1102,7 +1086,7 @@ else
       "\"%s\" is not absolute", path);
     addr->basic_errno = ERRNO_NOTABSOLUTE;
     addr->special_action = SPECIAL_FREEZE;
-    return;
+    return FALSE;
     }
   }
 
@@ -1151,7 +1135,7 @@ if (dont_deliver)
   debug_printf("*** delivery by %s transport bypassed by -N option\n",
     tblock->name);
   addr->transport_return = OK;
-  return;
+  return FALSE;
   }
 
 /* Handle the case of a file name. If the file name is /dev/null, we can save
@@ -1166,7 +1150,7 @@ if (!isdirectory)
   if (strcmp(path, "/dev/null") == 0)
     {
     addr->transport_return = OK;
-    return;
+    return FALSE;
     }
 
   /* The name of the file to be opened, and the file to which the data
@@ -1204,7 +1188,7 @@ if (!isdirectory)
       on the whole file name, because we know the file itself doesn't exist,
       and intermediate directories may also not exist. What we want to know is
       the real path of the longest existing part of the path. That must match
-      the home directory's beginning, which ever is the shorter. */
+      the home directory's beginning, whichever is the shorter. */
 
       #ifndef NO_REALPATH
       if (allow_creation_here && ob->create_file == create_belowhome)
@@ -1223,13 +1207,26 @@ if (!isdirectory)
 
         /* If rp == NULL it means that none of the relevant directories exist.
         This is not a problem here - it means that no symbolic links can exist,
-        which is all we are worried about. */
+        which is all we are worried about. Otherwise, we must compare it
+        against the start of the home directory. However, that may itself
+        contain symbolic links, so we have to "realpath" it as well, if
+        possible. */
 
         if (rp != NULL)
           {
+          char hdbuffer[PATH_MAX+1];
+          char *rph = deliver_home;
           int rlen = (int)strlen(big_buffer);
+
+          rp = realpath(deliver_home, hdbuffer);
+          if (rp != NULL)
+            {
+            rph = hdbuffer;
+            len = (int)strlen(rph);
+            }
+
           if (rlen > len) rlen = len;
-          if (strncmp(deliver_home, big_buffer, rlen) != 0)
+          if (strncmp(rph, big_buffer, rlen) != 0)
             {
             allow_creation_here = FALSE;
             DEBUG(9) debug_printf("Real path \"%s\" does not match \"%s\"\n",
@@ -1257,7 +1254,7 @@ if (!isdirectory)
         string_sprintf("failed to create directories for %s: %s", path,
           strerror(errno));
       DEBUG(9) debug_printf("%s transport: %s\n", tblock->name, path);
-      return;
+      return FALSE;
       }
     *p = '/';
     }
@@ -1291,7 +1288,7 @@ if (!isdirectory)
           addr->transport = tt;
           (tt->info->code)(tt, addr);
           }
-        return;
+        return FALSE;
         }
       }
     }
@@ -1437,8 +1434,10 @@ if (!isdirectory)
           {
           addr->basic_errno = errno;
           addr->message =
-            string_sprintf("creating lock file hitching post %s", hitchname);
-          return;
+            string_sprintf("creating lock file hitching post %s "
+              "(euid=%ld egid=%ld)", hitchname, (long int)geteuid(),
+              (long int)getegid());
+          return FALSE;
           }
         else break;
         }
@@ -1483,7 +1482,7 @@ if (!isdirectory)
       addr->basic_errno = ERRNO_LOCKFAILED;
       addr->message = string_sprintf("failed to lock mailbox %s (lock file)",
         filename);
-      return;
+      return FALSE;
       }
     }
 
@@ -1563,7 +1562,7 @@ if (!isdirectory)
         addr->basic_errno = errno;
         addr->message = string_sprintf("while creating mailbox %s",
           filename);
-        if (addr->pfr && addr->local_part[0] == '/' &&
+        if (testflag(addr, af_pfr) && addr->local_part[0] == '/' &&
             (errno == EPERM || errno == ENOENT || errno == EACCES))
           addr->transport_return = FAIL;
         goto RETURN;
@@ -1942,7 +1941,7 @@ else if (mbformat == mbf_smail)
     {
     addr->basic_errno = errno;
     addr->message = string_sprintf("while creating file %s", filename);
-    return;
+    return FALSE;
     }
 
   /* Why are these here? I should have stuck in a comment. Perhaps it's to
@@ -1991,7 +1990,7 @@ else if (mbformat == mbf_maildir)
     addr->message = string_sprintf("Expansion of \"%s\" (maildir_tag "
       "for %s transport) failed: %s", nametag, tblock->name,
       expand_string_message);
-    return;
+    return FALSE;
     }
 
   /* First ensure that the path we have is a directory; if it does not exist,
@@ -2030,7 +2029,7 @@ else if (mbformat == mbf_maildir)
           mdir);
         addr->basic_errno = ERRNO_NOTDIRECTORY;
         addr->special_action = SPECIAL_FREEZE;
-        return;
+        return FALSE;
         }
 
       /* Try to make if non-existent and configured to do so */
@@ -2043,7 +2042,7 @@ else if (mbformat == mbf_maildir)
           addr->message = string_sprintf("cannot create %s%s", path, mdir);
 	  addr->basic_errno = errno;
 	  addr->special_action = SPECIAL_FREEZE;
-	  return;
+	  return FALSE;
           }
         DEBUG(9) debug_printf("created directory %s%s\n", path, mdir);
         break;   /* out of the race loop */
@@ -2055,7 +2054,7 @@ else if (mbformat == mbf_maildir)
         strerror(errno));
       addr->basic_errno = errno;
       addr->special_action = SPECIAL_FREEZE;
-      return;
+      return FALSE;
       }
 
     /* If we went round the loop 10 times, the directory was flickering in and
@@ -2066,7 +2065,7 @@ else if (mbformat == mbf_maildir)
       addr->message = string_sprintf("existence of %s%s unclear\n", path, mdir);
       addr->basic_errno = errno;
       addr->special_action = SPECIAL_FREEZE;
-      return;
+      return FALSE;
       }
 
     /* First time through the directories loop, cd to the main directory */
@@ -2076,7 +2075,7 @@ else if (mbformat == mbf_maildir)
       addr->message = string_sprintf ("cannot chdir to %s", path);
       addr->basic_errno = errno;
       addr->special_action = SPECIAL_FREEZE;
-      return;
+      return FALSE;
       }
     }
 
@@ -2110,7 +2109,7 @@ else if (mbformat == mbf_maildir)
       addr->message = string_sprintf ("failed to open %s (%d tr%s)",
         filename, i, (i == 1)? "y" : "ies");
       addr->basic_errno = errno;
-      return;
+      return FALSE;
       }
 
     /* Open or stat failed but we haven't tried too many times yet. */
@@ -2155,7 +2154,7 @@ else
     {
     addr->basic_errno = errno;
     addr->message = string_sprintf("while creating file %s", filename);
-    return;
+    return FALSE;
     }
 
   /* Why are these here? Put in because they are present in the non-maildir
@@ -2174,7 +2173,7 @@ else
       "for %s transport) failed", filename, tblock->name);
     close(fd);
     unlink(filename);
-    return;
+    return FALSE;
     }
 
   /* Write the envelope file, then close it. */
@@ -2192,7 +2191,7 @@ else
           expand_string_message);
         fclose(env_file);
         unlink(filename);
-        return;
+        return FALSE;
         }
       }
     else
@@ -2221,7 +2220,7 @@ else
           expand_string_message);
         fclose(env_file);
         unlink(filename);
-        return;
+        return FALSE;
         }
       }
     else
@@ -2237,7 +2236,7 @@ else
     addr->basic_errno = errno;
     addr->message = string_sprintf("while closing %s", filename);
     unlink(filename);
-    return;
+    return FALSE;
     }
 
   DEBUG(9) debug_printf("Envelope file %s written\n", filename);
@@ -2250,7 +2249,7 @@ else
     addr->basic_errno = errno;
     addr->message = string_sprintf("while creating file %s", dataname);
     unlink(filename);
-    return;
+    return FALSE;
     }
 
   /* Why are these here? Put in because they are present in the non-maildir
@@ -2279,8 +2278,10 @@ only if configured. */
 if (isdirectory && (ob->quota_value > 0 || ob->quota_warn_threshold_value > 0))
   {
   const char *error;
+  char *check_path = path;
   int offset;
   pcre *regex = NULL;
+
   if (ob->quota_size_regex != NULL)
     {
     regex = pcre_compile(ob->quota_size_regex, PCRE_COPT, &error,&offset, NULL);
@@ -2291,15 +2292,47 @@ if (isdirectory && (ob->quota_value > 0 || ob->quota_warn_threshold_value > 0))
         ob->quota_size_regex);
       }
     }
-  saved_size = check_dir_size(path, &dircount, regex);
+
+  /* If we are handling a maildir delivery, and the directory contains a file
+  called maildirfolder, this is a maildir++ feature telling us that this is a
+  sub-directory of the real inbox. We should therefore do the quota check on
+  the parent directory. */
+
+  #ifdef SUPPORT_MAILDIR
+  if (mbformat == mbf_maildir)
+    {
+    struct stat statbuf;
+    if (stat("maildirfolder", &statbuf) >= 0)
+      {
+      char *slash = strrchr(path, '/');
+      if (slash != NULL)
+        check_path = string_sprintf("%.*s", slash - path, path);
+      }
+    }
+  #endif
+
+  /* Compute the size of the relevant directory */
+
+  saved_size = check_dir_size(check_path, &dircount, regex);
   }
 
 /* If there is a local quota setting, check that we are not going to exceed it
-with this message. */
+with this message if quota_is_inclusive is set; if it is not set, the check
+is for the mailbox already being over quota (i.e. the current message is not
+included in the check). */
 
 if (ob->quota_value > 0)
   {
-  if (saved_size + message_size > ob->quota_value ||
+  DEBUG(9)
+    {
+    debug_printf("Exim quota = %d old size = %d this message = %d "
+      "(%sincluded)\n", ob->quota_value, saved_size, message_size,
+      ob->quota_is_inclusive? "" : "not ");
+    debug_printf("  file quota = %d count = %d\n", ob->quota_filecount,
+      dircount);
+    }
+  if (saved_size + (ob->quota_is_inclusive? message_size : 0) >
+      ob->quota_value ||
       (ob->quota_filecount > 0 && dircount > ob->quota_filecount))
     {
     yield = DEFER;
@@ -2359,15 +2392,17 @@ if (tblock->local_smtp != local_smtp_off && yield == OK)
   if (tblock->bsmtp_helo &&
       !transport_write_string(fd, "HELO %s%s\n", primary_hostname, cr))
     yield = DEFER;
-  else if (!transport_write_string(fd, "MAIL FROM:<%s>%s\n",
-    ((addr->errors_address != NULL)? addr->errors_address : sender_address),
-      cr)) yield = DEFER;
+  else
+    if (!transport_write_string(fd, "MAIL FROM:<%s>%s\n",
+         ((addr->errors_address != NULL)?
+           addr->errors_address : sender_address), cr))
+      yield = DEFER;
   else
     {
     address_item *a;
     for (a = addr; a != NULL; a = a->next)
       {
-      address_item *b = (a->pfr)? a->parent: a;
+      address_item *b = testflag(a, af_pfr)? a->parent: a;
       if (!transport_write_string(fd, "RCPT TO:<%s@%s>%s\n", b->local_part,
         b->domain, cr))
           { yield = DEFER; break; }
@@ -2399,7 +2434,8 @@ set up at initialization time. */
 if (yield == OK)
   {
   if (!transport_write_message(addr, fd, ob->options, 0, tblock->add_headers,
-      tblock->remove_headers, ob->check_string, ob->escape_string))
+      tblock->remove_headers, ob->check_string, ob->escape_string,
+      tblock->rewrite_rules, tblock->rewrite_existflags))
     yield = DEFER;
   }
 
@@ -2481,13 +2517,16 @@ if (yield != OK)
     addr->more_errno = isdirectory? 0 : time(NULL) - times.actime;
 
   /* Handle system quota excession. Add an explanatory phrase for the error
-  message, since some systems don't have special quota-excession errors. */
+  message, since some systems don't have special quota-excession errors,
+  and on those that do, "quota" doesn't always mean anything to the user. */
 
   if (errno == errno_quota)
     {
     #ifndef EDQUOT
-    addr->message = string_sprintf("probably user's quota exceeded while "
-      "writing to file %s", filename);
+    addr->message = string_sprintf("mailbox is full "
+      "(quota exceeded while writing to file %s)", filename);
+    #else
+    addr->message = string_sprintf("mailbox is full");
     #endif
     DEBUG(9) debug_printf("System quota exceeded for %s%s%s\n",
       dataname,
@@ -2499,8 +2538,8 @@ if (yield != OK)
 
   else if (errno == ERRNO_EXIMQUOTA)
     {
-    addr->message = string_sprintf("MTA-imposed quota exceeded while "
-      "writing to file %s", dataname);
+    addr->message = string_sprintf("mailbox is full "
+      "(MTA-imposed quota exceeded while writing to file %s)", dataname);
     DEBUG(9) debug_printf("Exim quota exceeded for %s%s%s\n",
       dataname,
       isdirectory? "" : ": time since file read = ",
@@ -2710,6 +2749,8 @@ if (isdirectory && filename != NULL)
   unlink(filename);
   if (dataname != filename) unlink(dataname);
   }
+
+return FALSE;
 }
 
 /* End of transport/appendfile.c */
