@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -14,6 +14,7 @@ typedef struct {
   char *suffix;
   char *create_file_string;
   char *quota;
+  char *quota_size_regex;
   char *quota_warn_threshold;
   char *maildir_tag;
   char *mailstore_prefix;
@@ -28,6 +29,7 @@ typedef struct {
   int   dirmode;
   int   lockfile_mode;
   int   lockfile_timeout;
+  int   lock_fcntl_timeout;
   int   lock_retries;
   int   lock_interval;
   int   maildir_retries;
@@ -36,6 +38,7 @@ typedef struct {
   BOOL  allow_fifo;
   BOOL  allow_symlink;
   BOOL  check_group;
+  BOOL  check_owner;
   BOOL  create_directory;
   BOOL  notify_comsat;
   BOOL  require_lockfile;
@@ -54,6 +57,7 @@ typedef struct {
   BOOL  mailstore_format;
   BOOL  mbx_format;
   BOOL  quota_warn_threshold_is_percent;
+  BOOL  quota_is_inclusive;
 } appendfile_transport_options_block;
 
 /* Restricted creation options */
@@ -71,8 +75,7 @@ extern appendfile_transport_options_block appendfile_transport_option_defaults;
 
 /* The main and init entry points for the transport */
 
-extern void appendfile_transport_entry(transport_instance *, address_item *);
-
+extern BOOL appendfile_transport_entry(transport_instance *, address_item *);
 extern void appendfile_transport_init(transport_instance *);
 
 /* End of transports/appendfile.h */
