@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -50,6 +50,8 @@ optionlist autoreply_transport_options[] = {
       (void *)(offsetof(autoreply_transport_options_block, mode)) },
   { "once",              opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, oncelog)) },
+  { "once_file_size",    opt_int,
+      (void *)(offsetof(autoreply_transport_options_block, once_file_size)) },
   { "once_repeat",       opt_time,
       (void *)(offsetof(autoreply_transport_options_block, once_repeat)) },
   { "reply_to",          opt_stringptr,
@@ -87,6 +89,7 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
   NULL,           /* logfile */
   NULL,           /* oncelog */
   0600,           /* mode */
+  0,              /* once_file_size */
   0,              /* once_repeat */
   FALSE,          /* file_expand */
   FALSE,          /* file_optional */
@@ -133,7 +136,7 @@ Arguments:
    s         string to expand
    addr      address that is being worked on
    name      transport name, for error text
-   text      if TRUE, \n is allowed in the expanded string
+   text      if TRUE, don't check for non-printing characters
 
 Returns:     expanded string if expansion succeeds;
              NULL otherwise
@@ -153,15 +156,15 @@ if (ss == NULL)
   return NULL;
   }
 
-for (t = ss; *t != 0; t++)
+if (!text) for (t = ss; *t != 0; t++)
   {
   int c = (uschar)*t;
-  if (!mac_isprint(c) && c != '\t' && (c != '\n' || !text))
+  if (!mac_isprint(c) && c != '\t')
     {
     s = string_printing(s);
     addr->transport_return = FAIL;
     addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
-      "contains non-printing characters", s, name);
+      "contains non-printing character %d", s, name, c);
     return NULL;
     }
   }
@@ -176,24 +179,32 @@ return ss;
 *              Main entry point                  *
 *************************************************/
 
-/* See local README for interface details. */
+/* See local README for interface details. This transport always returns
+FALSE, indicating that the top address has the status for all - though in fact
+this transport can handle only one address at at time anyway. */
 
-void
+BOOL
 autoreply_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address we are working on */
 {
 int fd, pid, rc;
+int cache_fd = -1;
 int log_fd = -1;
+int cache_size = 0;
+int add_size = 0;
+EXIM_DB *dbm_file = NULL;
 BOOL file_expand, return_message;
 char *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
 char *logfile, *oncelog;
+char *cache_buff = NULL;
+char *cache_time = NULL;
 header_line *h;
-EXIM_DB *dbm_file = NULL;
 time_t now = time(NULL);
 time_t once_repeat;
 FILE *f;
 FILE *ff = NULL;
+
 autoreply_transport_options_block *ob =
   (autoreply_transport_options_block *)(tblock->options_block);
 
@@ -267,7 +278,7 @@ else
         (logfile = checkexpand(logfile, addr, tblock->name, FALSE)) == NULL) ||
       (oncelog != NULL &&
         (oncelog = checkexpand(oncelog, addr, tblock->name, FALSE)) == NULL))
-    return;
+    return FALSE;
   }
 
 
@@ -277,79 +288,140 @@ if (dont_deliver)
   {
   debug_printf("*** delivery by %s transport bypassed by -N option\n",
     tblock->name);
-  return;
+  return FALSE;
   }
 
 
-/* If the oncelog field is set, we send want to send only one message to
-the given recipient(s). This works only on the "To" field. If there is no "To"
-field the message is always sent. The "To" field is used as the key in a dbm
-file; if it contains more than one recipient, the effect might not be quite as
-envisaged. */
+/* If the oncelog field is set, we send want to send only one message to the
+given recipient(s). This works only on the "To" field. If there is no "To"
+field, the message is always sent. If the To: field contains more than one
+recipient, the effect might not be quite as envisaged. If once_file_size is
+set, instead of a dbm file, we use a regular file containing a circular buffer
+recipient cache. */
 
 if (oncelog != NULL && to != NULL)
   {
-  EXIM_DATUM key_datum, result_datum;
-  EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode, &dbm_file);
-  if (dbm_file == NULL)
+  time_t then = 0;
+
+  /* Handle fixed-size cache file. */
+
+  if (ob->once_file_size > 0)
     {
-    addr->transport_return = DEFER;
-    addr->message = string_sprintf("Failed to open %s file %s when sending "
-      "message from %s transport: %s", EXIM_DBTYPE, oncelog, tblock->name,
-      strerror(errno));
-    return;
-    }
+    char *p;
+    struct stat statbuf;
+    cache_fd = open(oncelog, O_CREAT|O_RDWR, ob->mode);
 
-  #ifdef EXIM_CLEAR_DATUMSTRUCT
-  memset(&key_datum, 0, sizeof(key_datum));
-  memset(&result_datum, 0, sizeof(result_datum));
-  #endif
-
-  EXIM_DATUM_DATA(key_datum) = to;
-  EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
-
-  if (EXIM_DBGET(dbm_file, key_datum, result_datum))
-    {
-    time_t then;
-    DEBUG(9) debug_printf("message previously sent to %s\n", to);
-
-    /* If the datum size is that of a binary time, we are in the new world
-    where messages are sent periodically. Otherwise the file is an old one,
-    where the datum was filled with a tod_log time, which is assumed to be
-    different in size. For that, only one message is ever sent. This change
-    introduced at Exim 3.00. In a couple of years' time the test on the size
-    can be abolished. */
-
-    if (EXIM_DATUM_SIZE(result_datum) == sizeof(time_t))
+    if (cache_fd < 0 || fstat(cache_fd, &statbuf) != 0)
       {
-      memcpy(&then, EXIM_DATUM_DATA(result_datum), sizeof(time_t));
+      addr->transport_return = DEFER;
+      addr->message = string_sprintf("Failed to %s \"once\" file %s when "
+        "sending message from %s transport: %s",
+        (cache_fd < 0)? "open" : "stat", oncelog, tblock->name,
+          strerror(errno));
+      goto END_OFF;
       }
-    else then = now;
 
-    if (once_repeat <= 0 || now - then < once_repeat)
+    /* Get store in the temporary pool and read the entire file into it. We get
+    an amount of store that is big enough to add the new entry on the end if we
+    need to do that. */
+
+    cache_size = statbuf.st_size;
+    add_size = sizeof(time_t) + (int)strlen(to) + 1;
+    store_pool = POOL_TEMP;
+    cache_buff = store_get(cache_size + add_size);
+    store_pool = POOL_MAIN;
+
+    if (read(cache_fd, cache_buff, cache_size) != cache_size)
       {
-      DEBUG(9) debug_printf("repeat time not reached\n");
-      log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
-      if (log_fd >= 0)
+      addr->transport_return = DEFER;
+      addr->basic_errno = errno;
+      addr->message = "error while reading \"once\" file";
+      goto END_OFF;
+      }
+
+    DEBUG(2) debug_printf("%d bytes read from %s\n", cache_size, oncelog);
+
+    /* Scan the data for this recipient. Each entry in the file starts with
+    a time_t sized time value, followed by the address, followed by a binary
+    zero. If we find a match, put the time into "then", and the place where it
+    was found into "cache_time". Otherwise, "then" is left at zero. */
+
+    p = cache_buff;
+    while (p < cache_buff + cache_size)
+      {
+      char *s = p + sizeof(time_t);
+      char *nextp = s + (int)strlen(s) + 1;
+      if (strcmp(to, s) == 0)
         {
-        char *ptr = log_buffer;
-        sprintf(ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log),
-          to);
-        while(*ptr) ptr++;
-        write(log_fd, log_buffer, ptr - log_buffer);
+        memcpy(&then, p, sizeof(time_t));
+        cache_time = p;
+        break;
         }
-      EXIM_DBCLOSE(dbm_file);
-      return;
-      }
-    else
-      {
-      DEBUG(9) debug_printf("repeat time reached\n");
+      p = nextp;
       }
     }
+
+  /* Use a DBM file for the list of previous recipients. */
+
+  else
+    {
+    EXIM_DATUM key_datum, result_datum;
+    EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode, &dbm_file);
+    if (dbm_file == NULL)
+      {
+      addr->transport_return = DEFER;
+      addr->message = string_sprintf("Failed to open %s file %s when sending "
+        "message from %s transport: %s", EXIM_DBTYPE, oncelog, tblock->name,
+        strerror(errno));
+      goto END_OFF;
+      }
+
+    EXIM_DATUM_INIT(key_datum);        /* Some DBM libraries need datums */
+    EXIM_DATUM_INIT(result_datum);     /* to be cleared */
+    EXIM_DATUM_DATA(key_datum) = to;
+    EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
+
+    if (EXIM_DBGET(dbm_file, key_datum, result_datum))
+      {
+      /* If the datum size is that of a binary time, we are in the new world
+      where messages are sent periodically. Otherwise the file is an old one,
+      where the datum was filled with a tod_log time, which is assumed to be
+      different in size. For that, only one message is ever sent. This change
+      introduced at Exim 3.00. In a couple of years' time the test on the size
+      can be abolished. */
+
+      if (EXIM_DATUM_SIZE(result_datum) == sizeof(time_t))
+        {
+        memcpy(&then, EXIM_DATUM_DATA(result_datum), sizeof(time_t));
+        }
+      else then = now;
+      }
+    }
+
+  /* Either "then" is set zero, if no message has yet been sent, or it
+  is set to the time of the last sending. */
+
+  if (then != 0 && (once_repeat <= 0 || now - then < once_repeat))
+    {
+    DEBUG(9) debug_printf("message previously sent to %s%s\n", to,
+      (once_repeat > 0)? " and repeat time not reached" : "");
+    log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
+    if (log_fd >= 0)
+      {
+      char *ptr = log_buffer;
+      sprintf(ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log), to);
+      while(*ptr) ptr++;
+      write(log_fd, log_buffer, ptr - log_buffer);
+      close(log_fd);
+      }
+    goto END_OFF;
+    }
+
+  DEBUG(9) debug_printf("%s %s\n", (then <= 0)?
+    "no previous message sent to" : "repeat time reached for", to);
   }
 
-
-/* Ensure any requested file is available. */
+/* We are going to send a message. Ensure any requested file is available. */
 
 if (file != NULL)
   {
@@ -359,14 +431,13 @@ if (file != NULL)
     addr->transport_return = DEFER;
     addr->message = string_sprintf("Failed to open file %s when sending "
       "message from %s transport: %s", file, tblock->name, strerror(errno));
-    return;
+    return FALSE;
     }
   }
 
 /* Make a subprocess to send the message */
 
-pid = child_open(mailer_argv, NULL, 077, NULL, NULL, &fd,
-  (debug_file != NULL)? fileno(debug_file) : -1, NULL, FALSE);
+pid = child_open_exim(&fd);
 
 /* Creation of child failed; defer this delivery. */
 
@@ -376,7 +447,7 @@ if (pid < 0)
   addr->message = string_sprintf("Failed to create child process to send "
     "message from %s transport: %s", tblock->name, strerror(errno));
   DEBUG(2) debug_printf("%s\n", addr->message);
-  return;
+  return FALSE;
   }
 
 /* Create the message to be sent - recipients are taken from the headers,
@@ -401,7 +472,7 @@ for (h = header_list; h != NULL; h = h->next)
 if (h != NULL)
   {
   char *s = strchr(h->text, ':') + 1;
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   fprintf(f, "In-Reply-To: %s", s);
   }
 
@@ -461,7 +532,7 @@ if (return_message)
     (tblock->delivery_date_add? topt_add_delivery_date : 0) |
     (tblock->envelope_to_add? topt_add_envelope_to : 0),
     return_size_limit, tblock->add_headers, tblock->remove_headers,
-    NULL, NULL);
+    NULL, NULL, tblock->rewrite_rules, tblock->rewrite_existflags);
   }
 
 /* End the message and wait for the child process to end; no timeout. */
@@ -470,15 +541,47 @@ fclose(f);
 rc = child_close(pid, 0);
 
 /* Update the "sent to" log whatever the yield. This errs on the side of
-missing out a message rather than risking sending more than one. */
+missing out a message rather than risking sending more than one. We either have
+cache_fd set to a fixed size, circular buffer file, or dbm_file set to an open
+DBM file (or neither, if "once" is not set). */
 
-if (dbm_file != NULL)
+/* Update fixed-size cache file. If cache_time is set, we found a previous
+entry; that is the spot into which to put the current time. Otherwise we have
+to add a new record; remove the first one in the file if the file is too big.
+We always rewrite the entire file in a single write operation. This is
+(hopefully) going to be the safest thing because there is no interlocking
+between multiple simultaneous deliveries. */
+
+if (cache_fd >= 0)
+  {
+  char *from = cache_buff;
+  int size = cache_size;
+  (void)lseek(cache_fd, 0, SEEK_SET);
+
+  if (cache_time == NULL)
+    {
+    cache_time = from + size;
+    memcpy(cache_time + sizeof(time_t), to, add_size - sizeof(time_t));
+    size += add_size;
+
+    if (cache_size > 0 && size > ob->once_file_size)
+      {
+      from += sizeof(time_t) + (int)strlen(from + sizeof(time_t)) + 1;
+      size -= (from - cache_buff);
+      }
+    }
+
+  memcpy(cache_time, &now, sizeof(time_t));
+  write(cache_fd, from, size);
+  }
+
+/* Update DBM file */
+
+else if (dbm_file != NULL)
   {
   EXIM_DATUM key_datum, value_datum;
-  #ifdef EXIM_CLEAR_DATUMSTRUCT
-  memset(&key_datum, 0, sizeof(key_datum));
-  memset(&value_datum, 0, sizeof(value_datum));
-  #endif
+  EXIM_DATUM_INIT(key_datum);          /* Some DBM libraries need to have */
+  EXIM_DATUM_INIT(value_datum);        /* cleared datums. */
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
 
@@ -487,10 +590,8 @@ if (dbm_file != NULL)
   can avoid warning messages from the char * systems. */
 
   EXIM_DATUM_DATA(value_datum) = (char *)(&now);
-
   EXIM_DATUM_SIZE(value_datum) = (int)sizeof(time_t);
   EXIM_DBPUT(dbm_file, key_datum, value_datum);
-  EXIM_DBCLOSE(dbm_file);
   }
 
 /* If sending failed, defer to try again - but if once is set the next
@@ -509,7 +610,7 @@ if (rc != 0)
     addr->transport_return = DEFER;
     addr->message = string_sprintf("Failed to send message from %s "
       "transport (%d)", tblock->name, rc);
-    return;
+    goto END_OFF;
     }
   }
 
@@ -524,8 +625,7 @@ writing. */
 
 if (logfile != NULL)
   {
-  if (log_fd < 0)
-    log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
+  int log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
   if (log_fd >= 0)
     {
     char *ptr = log_buffer;
@@ -569,14 +669,27 @@ if (logfile != NULL)
       while(*ptr) ptr++;
       }
     write(log_fd, log_buffer, ptr - log_buffer);
+    close(log_fd);
     }
   else DEBUG(2) debug_printf("Failed to open log file %s for %s "
     "transport: %s\n", logfile, tblock->name, strerror(errno));
   }
 
-if (log_fd >= 0) close(log_fd);
+
+END_OFF:
+if (cache_buff != NULL)
+  {
+  store_pool = POOL_TEMP;
+  store_reset(cache_buff);
+  store_pool = POOL_MAIN;
+  }
+
+if (dbm_file != NULL) EXIM_DBCLOSE(dbm_file);
+if (cache_fd > 0) close(cache_fd);
 
 DEBUG(2) debug_printf("%s transport succeeded\n", tblock->name);
+
+return FALSE;
 }
 
 /* End of transport/autoreply.c */
