@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -20,6 +20,7 @@ typedef struct {
   char *logfile;
   char *oncelog;
   int   mode;
+  off_t once_file_size;
   time_t once_repeat;
   BOOL  file_expand;
   BOOL  file_optional;
@@ -37,8 +38,7 @@ extern autoreply_transport_options_block autoreply_transport_option_defaults;
 
 /* The main and init entry points for the transport */
 
-extern void autoreply_transport_entry(transport_instance *, address_item *);
-
+extern BOOL autoreply_transport_entry(transport_instance *, address_item *);
 extern void autoreply_transport_init(transport_instance *);
 
 /* End of transports/autoreply.h */
