@@ -418,7 +418,13 @@ if (expand_arguments)
     s[offset] = 0;
     }
 
+  /* Allow $recipients in the expansion iff it comes from a system filter */
+
+  enable_dollar_recipients = addr != NULL &&
+    addr->parent != NULL && strcmp(addr->parent->orig, "message filter") == 0;
   argv[2] = expand_string(s);
+  enable_dollar_recipients = FALSE;
+
   if (argv[2] == NULL)
     {
     addr->transport_return = search_find_defer? DEFER :
@@ -428,6 +434,7 @@ if (expand_arguments)
       cmd, tname, expand_string_message);
     return FALSE;
     }
+
   DEBUG(7) debug_printf("shell pipe command after expansion:\n  %s\n", argv[2]);
   }
 else argv[2] = cmd;
@@ -443,9 +450,11 @@ return TRUE;
 *              Main entry point                  *
 *************************************************/
 
-/* See local README for interface details. */
+/* See local README for interface details. This transport always returns FALSE,
+indicating that the status in the first address is the status for all addresses
+in a batch. */
 
-void
+BOOL
 pipe_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address(es) we are working on */
@@ -482,7 +491,7 @@ if (addr->local_part[0] == '|')
   {
   cmd = addr->local_part + 1;
   while (isspace((uschar)*cmd)) cmd++;
-  expand_arguments = addr->expand_pipe;
+  expand_arguments = testflag(addr, af_expand_pipe);
   }
 else
   {
@@ -512,10 +521,10 @@ there is an option to do that. */
 if (ob->use_shell)
   {
   if (!set_up_shell_command(&argv, cmd, expand_arguments, addr, tblock->name))
-    return;
+    return FALSE;
   }
 else if (!set_up_direct_command(&argv, cmd, expand_arguments, addr,
-  tblock->name, ob)) return;
+  tblock->name, ob)) return FALSE;
 
 expand_nmax = -1;   /* Reset */
 
@@ -553,7 +562,7 @@ if (envlist != NULL)
     addr->message = string_sprintf("failed to expand string \"%s\" "
       "for %s transport: %s", ob->environment, tblock->name,
       expand_string_message);
-    return;
+    return FALSE;
     }
   }
 
@@ -575,7 +584,7 @@ if (dont_deliver)
   {
   debug_printf("*** delivery by %s transport bypassed by -N option",
     tblock->name);
-  return;
+  return FALSE;
   }
 
 
@@ -608,7 +617,7 @@ if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd_in, &fd_out,
   addr->message = string_sprintf(
     "Failed to create child process for %s transport: %s", tblock->name,
       strerror(errno));
-  return;
+  return FALSE;
   }
 
 /* Now fork a process to handle the output that comes down the pipe. */
@@ -622,7 +631,7 @@ if ((outpid = fork()) < 0)
       tblock->name);
   close(fd_in);
   close(fd_out);
-  return;
+  return FALSE;
   }
 
 /* This is the code for the output-handling subprocess. Read from the pipe
@@ -714,7 +723,7 @@ if (ob->prefix != NULL)
     addr->transport_return = search_find_defer? DEFER : PANIC;
     addr->message = string_sprintf("Expansion of \"%s\" (prefix for %s "
       "transport) failed: %s", ob->prefix, tblock->name, expand_string_message);
-    return;
+    return FALSE;
     }
   if (!transport_write_block(fd_in, prefix, (int)strlen(prefix)))
     goto END_WRITE;
@@ -723,7 +732,8 @@ if (ob->prefix != NULL)
 /* Now the actual message - the options were set at initialization time */
 
 if (!transport_write_message(addr, fd_in, ob->options, 0, tblock->add_headers,
-  tblock->remove_headers, ob->check_string, ob->escape_string))
+  tblock->remove_headers, ob->check_string, ob->escape_string,
+  tblock->rewrite_rules, tblock->rewrite_existflags))
     goto END_WRITE;
 
 /* Now any configured suffix */
@@ -736,7 +746,7 @@ if (ob->suffix != NULL)
     addr->transport_return = search_find_defer? DEFER : PANIC;
     addr->message = string_sprintf("Expansion of \"%s\" (suffix for %s "
       "transport) failed: %s", ob->suffix, tblock->name, expand_string_message);
-    return;
+    return FALSE;
     }
   if (!transport_write_block(fd_in, suffix, (int)strlen(suffix)))
     goto END_WRITE;
@@ -793,7 +803,7 @@ if (!written_ok)
       addr->message = string_sprintf("Failed repeatedly to write data");
     else
       addr->message = string_sprintf("Error %d", errno);
-    return;
+    return FALSE;
     }
   }
 
@@ -875,7 +885,9 @@ if ((rc = child_close(pid, timeout)) != 0)
 	    can also be used as a catchall message when something
 	    you wanted to do doesn't work, but you don't know why.
 
-  so this is used by the code in child.c to signal execve() failure. */
+  so this is used by the code in child.c to signal execve() failure and
+  other unexpected failures such as setuid() not working - though that won't
+  be the case here because we aren't changing uid. */
 
   else
     {
@@ -964,6 +976,8 @@ while (wait(&rc) >= 0);
 
 DEBUG(2) debug_printf("%s transport yielded %d\n", tblock->name,
   addr->transport_return);
+
+return FALSE;
 }
 
 /* End of transport/pipe.c */
