@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -40,8 +40,7 @@ extern pipe_transport_options_block pipe_transport_option_defaults;
 
 /* The main and init entry points for the transport */
 
-extern void pipe_transport_entry(transport_instance *, address_item *);
-
+extern BOOL pipe_transport_entry(transport_instance *, address_item *);
 extern void pipe_transport_init(transport_instance *);
 
 /* End of transports/pipe.h */
