@@ -37,9 +37,9 @@ static BOOL sender_is_local;
 
 /* This function is called to set up a new address from what was routed or
 directed, so as to include completed domain names, or whatever. If
-local_part_quoted is set, the address is local and the local part was quoted in
-some way, so we have to re-instate the quoting. We use a flag to control this
-to save time doing it on 99.9% of the addresses that don't need it.
+af_local_part_quoted is set, the address is local and the local part was quoted
+in some way, so we have to re-instate the quoting. We use a flag to control
+this to save time doing it on 99.99% of the addresses that don't need it.
 
 Arguments:
   newaddr       points to where to put the new address
@@ -54,7 +54,7 @@ set_new_address(char **newaddress, address_item *addr)
 char *prefix = (addr->prefix == NULL)? "" : addr->prefix;
 char *suffix = (addr->suffix == NULL)? "" : addr->suffix;
 
-if (addr->local_part_quoted)
+if (testflag(addr, af_local_part_quoted))
   {
   int i;
   int size = 64;
@@ -183,14 +183,13 @@ DEBUG(9)
 
 /* Rewrite and report on it. */
 
-if (rewrite_rules != NULL)
+if (global_rewrite_rules != NULL)
   {
   char *old = receiver;
-  receiver = rewrite_address(receiver, is_recipient, FALSE);
-  if (receiver != old)
-    {
-    if (f != NULL && !expn) fprintf(f, "Address rewritten as: %s\n", receiver);
-    }
+  receiver = rewrite_address(receiver, is_recipient, FALSE,
+    global_rewrite_rules, rewrite_existflags);
+  if (receiver != old && f != NULL && !expn)
+    fprintf(f, "Address rewritten as: %s\n", receiver);
   }
 
 /* Set up an initial address structure. */
@@ -202,9 +201,9 @@ addresses. We must also cope with generated pipes and files at the top
 level. (See also the code/comment in deliver.c.) However, it is usually
 the case that the forwardfile director has its verify flag turned off.
 
-The loop is used after directing, however, only when the verify_actions flag is
-set, and this can only be set locally. Remote enquiries just get information
-about the top level address, not anything that it generated.
+The loop is used after directing, however, only when full_info is set, and this
+can only be set locally. Remote enquiries just get information about the top
+level address, not anything that it generated.
 
 In the case of a router discovering that an apparently remote address is in
 fact local, the loop is always re-run. */
@@ -219,26 +218,26 @@ while (addr_new != NULL)
   /* Handle generated pipe, file or reply addresses. We don't get these
   when handling EXPN, as it does only one level of expansion. */
 
-  if (addr->pfr)
+  if (testflag(addr, af_pfr))
     {
     allok = FALSE;
     if (addr->orig[0] == '|')
       {
       if (f != NULL)
         fprintf(f, "%s -> %s %s\n", addr->parent->orig,
-          addr->orig, addr->allow_pipe? "" : "*** forbidden ***");
+          addr->orig, testflag(addr, af_allow_pipe)? "" : "*** forbidden ***");
       }
     else if (addr->orig[0] == '/')
       {
       if (f != NULL)
         fprintf(f, "%s -> %s %s\n", addr->parent->orig,
-          addr->orig, addr->allow_file? "" : "*** forbidden ***");
+          addr->orig, testflag(addr, af_allow_file)? "" : "*** forbidden ***");
       }
     else
       {
       if (f != NULL)
         fprintf(f, "%s -> mail %s %s\n", addr->parent->orig,
-          addr->orig+1, addr->allow_reply? "" : "*** forbidden ***");
+          addr->orig+1, testflag(addr, af_allow_reply)? "" : "*** forbidden ***");
       }
 
     if (addr->basic_errno == ERRNO_BADTRANSPORT && f != NULL)
@@ -264,18 +263,7 @@ while (addr_new != NULL)
 
   if (!deliver_setlocal(addr)) rc = DEFER; else
     {
-    if (local_domain != NULL) *local_domain = addr->local;
-
-    /* If we are handing an EXPN command and this is the original address,
-    and it is not local, we don't proceed any further. (Actually, if it's
-    expn, this will always be the original address, since we don't process
-    generated addresses any more.) */
-
-    if (expn && addr == addr_orig && !addr->local)
-      {
-      fprintf(f, "550 Not a local domain\r\n");
-      return FALSE;
-      }
+    if (local_domain != NULL) *local_domain = testflag(addr, af_local);
 
     /* DEBUG: show what's been done to this address */
 
@@ -284,7 +272,7 @@ while (addr_new != NULL)
       debug_printf("address %s\n", addr->orig);
       debug_printf("  local_part=%s domain=%s\n  %s\n",
         addr->local_part, addr->domain,
-        (addr->local)? "domain is local" : "domain is not local");
+        testflag(addr, af_local)? "domain is local" : "domain is not local");
       }
 
     /* Handle a local address with the directors, or a remote address with the
@@ -292,34 +280,24 @@ while (addr_new != NULL)
 
     verify_forced_errmsg = NULL;
 
-    rc = (addr->local)?
+    rc = testflag(addr, af_local)?
       direct_address(addr, &addr_local, &addr_remote, &addr_new, &addr_succeed,
         verify_type) :
       route_address(addr, &addr_local, &addr_remote, &addr_new, verify_type);
 
+    /* If directing/routing succeeded and set up a list of hosts, and the
+    client host is in sender_verify_hosts_callback and the domain is in
+    sender_verify_callback_domains, do the full call-back verification for a
+    sender. This is expensive. */
 
-    #ifdef NEVER
-    /* This is some embryonic code for doing a verification by calling back to
-    the hosts that an address routed to. It is of course expensive, but apart
-    from that there are potential problems, so I haven't for the moment pursued
-    it. Some big domains (e.g. aol.com) accept anything and check later, so you
-    don't gain anything by this check in any case. If this code is ever
-    required, the option sender_full_verify_domains needs to be created, and
-    really some means of varying the port and the timeout are also required.
-    What should happen when running -bh? Another problem is that ideally, the
-    error from the remote host should be logged, but because it is in the addr
-    structure, it gets lost, since that is local to the verification process.
-    Actually, this is a general problem. */
-
-    /* If directing/routing succeeded and set up a list of hosts, and
-    sender_verify_full_domains is set for this domain, do the full call-back
-    verification for a sender. This is expensive. */
+    /* This is still somewhat experimental code. */
 
     if (rc == OK &&
         addr->host_list != NULL &&
         (options & vopt_is_recipient) == 0 &&
-        sender_full_verify_domains != NULL &&
-        match_isinlist(addr->domain, &sender_full_verify_domains, FALSE,
+        verify_check_host(&sender_verify_hosts_callback, FALSE) &&
+        sender_verify_callback_domains != NULL &&
+        match_isinlist(addr->domain, &sender_verify_callback_domains, FALSE,
           TRUE, NULL))
       {
       host_item *host;
@@ -327,67 +305,90 @@ while (addr_new != NULL)
 
       HDEBUG(1) debug_printf("Attempting full verification using callback\n");
 
-      for (host = addr->host_list; !done && host != NULL; host = host->next)
+      if (host_checking)
         {
-        int fd = smtp_connect(host, 1225, NULL, 30, TRUE);
-        if (fd < 0) continue;
+        HDEBUG(1) debug_printf("... callback omitted when host testing\n");
+        done = TRUE;
+        }
 
-        if (smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-            smtp_write_command(fd, "HELO %s\r\n", primary_hostname) &&
-            smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-            smtp_write_command(fd, "MAIL FROM:<>\r\n") &&
-            smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-            smtp_write_command(fd, "RCPT TO:<%s>\r\n", addr->orig))
+      else for (host = addr->host_list;
+                host != NULL && ! done;
+                host = host->next)
+        {
+        int port = running_in_test_harness? 1225 : 25;
+        smtp_inblock inblock;
+        smtp_outblock outblock;
+        uschar inbuffer[1024];
+        uschar outbuffer[1024];
+
+        /* Set up the buffer for reading SMTP response packets. */
+
+        inblock.buffer = inbuffer;
+        inblock.buffersize = sizeof(inbuffer);
+        inblock.ptr = inbuffer;
+        inblock.ptrend = inbuffer;
+
+        /* Set up the buffer for holding SMTP commands while pipelining */
+
+        outblock.buffer = outbuffer;
+        outblock.buffersize = sizeof(outbuffer);
+        outblock.ptr = outbuffer;
+        outblock.cmd_count = 0;
+
+        inblock.sock = outblock.sock =
+          smtp_connect(host, (int)htons((unsigned short)port), NULL,
+            sender_verify_callback_timeout, TRUE);
+
+        if (inblock.sock < 0) continue;
+
+        if (smtp_read_response(&inblock, US big_buffer, big_buffer_size, '2',
+              sender_verify_callback_timeout) &&
+            smtp_write_command(&outblock, FALSE, "HELO %s\r\n",
+              primary_hostname) &&
+            smtp_read_response(&inblock, US big_buffer, big_buffer_size, '2',
+              sender_verify_callback_timeout) &&
+            smtp_write_command(&outblock, FALSE, "MAIL FROM:<>\r\n") &&
+            smtp_read_response(&inblock, US big_buffer, big_buffer_size, '2',
+              sender_verify_callback_timeout) &&
+            smtp_write_command(&outblock, FALSE, "RCPT TO:<%.1000s>\r\n",
+              addr->orig))
           {
-          if (!smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30))
+          if (smtp_read_response(&inblock, US big_buffer, big_buffer_size, '2',
+              sender_verify_callback_timeout))
             {
-            if (errno == 0 && big_buffer[0] == '5')
+            done = TRUE;                /* Address accepted */
+            }
+          else if (errno == 0)
+            {
+            sender_verify_callback_error =
+              string_sprintf("response from %s [%s] was %s", host->name,
+                host->address, big_buffer);
+            if (big_buffer[0] == '5')   /* Address rejected */
               {
               rc = FAIL;
-              addr->message = string_sprintf("check with host %s failed: %s",
-                host->name, big_buffer);
+              addr->message =
+                string_sprintf("callback check with host %s failed: %s",
+                  host->name, big_buffer);
               done = TRUE;
               }
             }
-          else done = TRUE;
           }
 
-        smtp_write_command(fd, "QUIT\r\n");
-        close(fd);
+        smtp_write_command(&outblock, FALSE, "QUIT\r\n");
+        close(inblock.sock);
         }
 
-      /* Failure to connect to any host, or any other error other than a 5xx
-      response is a temporary error. */
+      /* Failure to connect to any host, or any response other than 2xx or 5xx
+      is a temporary error. */
 
-      if (host == NULL)
+      if (!done)
         {
         rc = DEFER;
         addr->message =
-          string_sprintf("could not connect to MX host for domain %s",
-            addr->domain);
+          string_sprintf("could not complete callback check");
         }
-      }
-    #endif /* NEVER */
-
-    }   /* Routing/directing done, or setlocal() failed */
-
-
-  /* If a director generated a new address as a routing ploy rather than
-  as an aliasing or forwarding type of operation, it returns OK_CONTINUE
-  instead of OK. This means that verification should always inspect the
-  generated addresses, even when short information is required. However,
-  we want to pass back any modified top-level address via newaddress,
-  not any subsequent one. */
-
-  if (rc == OK_CONTINUE)
-    {
-    if (!full_info && newaddress != NULL)
-      {
-      set_new_address(newaddress, addr);
-      newaddress = NULL;
-      }
-    continue;
-    }
+      }   /* Callback */
+    }     /* Routing/directing done, or setlocal() failed */
 
   /* If a remote address turned out to be local after all, set it up for
   reprocessing and restart the loop. If this is a sender and the router
@@ -415,14 +416,22 @@ while (addr_new != NULL)
   if (rc == FAIL)
     {
     allok = FALSE;
-    if (f != NULL) fprintf(f, "%s%s %s:%s%s%s%s%s\n", ko_prefix,
-      addr->orig,
-      address_test_mode? "is undeliverable" : "failed to verify",
-      (addr->basic_errno <= 0)? "" : strerror(addr->basic_errno),
-      (addr->basic_errno <= 0)? "" : ": ",
-      expn? " " : "\n  ",
-      (addr->message != NULL)? addr->message :
-        (addr->basic_errno <= 0)? "unknown error" : "", cr);
+    if (f != NULL)
+      {
+      if (expn && !testflag(addr, af_local))
+        fprintf(f, "550 Not a local domain\r\n");
+      else
+        {
+        fprintf(f, "%s%s %s", ko_prefix, addr->orig,
+          address_test_mode? "is undeliverable" : "failed to verify");
+        if (addr->basic_errno > 0)
+          fprintf(f, ": %s", strerror(addr->basic_errno));
+        if (addr->message != NULL)
+          fprintf(f, ":%s%s", expn? " " : "\n  ", addr->message);
+        fprintf(f, "%s\n", cr);
+        }
+      }
+
     if (!full_info) return FAIL; else yield = FAIL;
     }
 
@@ -447,11 +456,21 @@ while (addr_new != NULL)
     }
 
   /* If we are handling EXPN, we do not want to continue to route beyond
-  the top level. Output what has been generated. */
+  the top level. If the address was not local, give an error. We have to put
+  this test here, after the routing, in case the routing turned it into a
+  local address which then got directed. Otherwise, output what has been
+  generated. */
 
   else if (expn)
     {
     char *ok_prefix = "250-";
+
+    if (addr == addr_orig && !testflag(addr, af_local))
+      {
+      fprintf(f, "550 Not a local domain\r\n");
+      return FALSE;
+      }
+
     if (addr_new == NULL)
       {
       if (addr_local == NULL)
@@ -475,13 +494,28 @@ while (addr_new != NULL)
 
   else
     {
-    if (newaddress != NULL) set_new_address(newaddress, addr);
+    if (newaddress != NULL)
+      {
+      set_new_address(newaddress, addr);
+      newaddress = NULL;                  /* Only at top level */
+      }
 
     /* Handle successful routing or directing when short info wanted. Otherwise
-    continue for other (generated) addresses. When full_info is set, f will not
-    be NULL. */
+    continue for other (generated) addresses.
 
-    if (!full_info)
+    There is a conflict between the use of aliasing as an alternate email
+    address, and as a sort of mailing list. If an alias turns the incoming
+    address into just one address (e.g. J.Caesar->jc44) you may well want to
+    carry on verifying the generated address to ensure it is valid when
+    checking incoming mail. If aliasing generates multiple addresses, you
+    probably don't want to do this. Exim therefore treats the generation of
+    just a single new address as a special case, and continues on to verify the
+    generated address, when the address was directed and the director has
+    one_continue set TRUE (aliasfile and smartuser do, forwardfile does not). */
+
+    if (!full_info &&
+         (addr_new == NULL || addr_new->next != NULL ||
+          addr->director == NULL || !addr->director->one_continue))
       {
       if (f != NULL) fprintf(f, "%s %s\n",
         addr->orig,
@@ -491,14 +525,14 @@ while (addr_new != NULL)
     }
   }
 
-/* Display the full results of the successful directing and routing,
-including any generated addresses. Control gets here only when full_info is
-set, which requires f not to be NULL, and this occurs only when a top-level
-verify is called with the debugging switch on, or as a result of an SMTP EXPN
-command (permitted only for specified hosts/nets). For verification, if the
-debugging value is greater than 1, show everything; for 1 (which is also -v)
-show what "the normal user" might be interested in. For address testing,
-always show everything.
+/* Display the full results of the successful directing and routing, including
+any generated addresses. Control gets here only when full_info is set, which
+requires f not to be NULL, and this occurs only when a top-level verify is
+called with the debugging switch on, or as a result of an SMTP EXPN command
+(permitted only for specified hosts/nets). For verification, if the debugging
+value is greater than 1, show everything; for 1 (which is also -v) show what
+"the normal user" might be interested in. For address testing, always show
+everything.
 
 If there are no local and no remote addresses, and there were no pipes, files,
 or autoreplies, and there were no errors or deferments, the message is to be
@@ -865,7 +899,7 @@ sprintf(buffer, "%s:%.200s", sender_address,
   (sender_host_name != NULL)? sender_host_name :
   (sender_host_address != NULL)? sender_host_address : "");
 
-dbm_file = dbfn_open("reject", O_RDWR, &dbblock);
+dbm_file = dbfn_open("reject", O_RDWR, &dbblock, FALSE);
 reject = (dbm_file == NULL)?
   NULL : dbfn_read_with_length(dbm_file, buffer, &rejectlen);
 
@@ -926,6 +960,8 @@ if (rc == DEFER)
     DEBUG(2) debug_printf("%s verification deferred\n", sender_address);
     *errcode = 451;
     *errmess = "temporarily unable to verify sender address (try again later)";
+    if (sender_verify_callback_error != NULL)
+      *errmess = string_sprintf("%s: %s", *errmess, sender_verify_callback_error);
     return FALSE;
     }
 
@@ -966,6 +1002,8 @@ if (!reject->rejected_mail_from)
   *errcode = 550;
   *errmess = sender_is_local?
     "unknown local-part in sender" : "cannot route to sender";
+  if (sender_verify_callback_error != NULL)
+    *errmess = string_sprintf("%s: %s", *errmess, sender_verify_callback_error);
   reject->rejected_mail_from = TRUE;
   DEBUG(2) debug_printf("%s verification failed after MAIL FROM\n",
     sender_address);
@@ -983,6 +1021,8 @@ else
   refuse_all_rcpts = TRUE;
   *errcode = 250;
   *errmess = "reject all recipients: 3 times bad sender";
+  if (sender_verify_callback_error != NULL)
+    *errmess = string_sprintf("%s: %s", *errmess, sender_verify_callback_error);
   HDEBUG(2) debug_printf("%s verification failed - will reject all recipients\n",
     sender_address);
   yield = TRUE;
@@ -1067,11 +1107,8 @@ switch(sender_ok)
         return FALSE;
         }
 
-      log_write(3, LOG_REJECT, "warning: from%s%s <%.1024s>: can't check for "
-        "valid sender in headers",
-        (sender_fullhost == NULL)? "" : " ",
-        (sender_fullhost == NULL)? "" : host_and_ident(),
-        sender_address);
+      log_write(3, LOG_REJECT, "warning: from%s <%.1024s>: can't check for "
+        "valid sender in headers", host_and_ident(" ", NULL), sender_address);
       break;
 
       case FAIL:
@@ -1087,10 +1124,8 @@ switch(sender_ok)
         return FALSE;
         }
 
-      log_write(3, LOG_REJECT, "warning: from%s%s <%.1024s>: %s",
-        (sender_fullhost == NULL)? "" : " ",
-        (sender_fullhost == NULL)? "" : host_and_ident(),
-        sender_address,
+      log_write(3, LOG_REJECT, "warning: from%s <%.1024s>: %s",
+        host_and_ident(" ", NULL), sender_address,
         (verify_address_parse_error == NULL)?
           "no valid sender in headers" : verify_address_parse_error);
       break;
@@ -1143,6 +1178,9 @@ else
     sender_address);
   }
 
+if (sender_verify_callback_error != NULL)
+  *errmess = string_sprintf("%s: %s", *errmess, sender_verify_callback_error);
+
 /* Set up the key name and open the hints database. O_RDWR (rather than
 O_WRONLY) is needed by Berkeley native DB even when reading only. If the
 database won't open, we can do no more. */
@@ -1151,7 +1189,7 @@ sprintf(buffer, "%s:%.200s", sender_address,
   (sender_host_name != NULL)? sender_host_name :
   (sender_host_address != NULL)? sender_host_address : "");
 
-dbm_file = dbfn_open("reject", O_RDWR|O_CREAT, &dbblock);
+dbm_file = dbfn_open("reject", O_RDWR|O_CREAT, &dbblock, FALSE);
 
 /* Write a new record for this failure. A temporary failure rate of 0.0
 indicates a permanent error. Otherwise initialize to "1 per hour". */
@@ -1426,7 +1464,7 @@ if (*t == 0)
   h.next = NULL;
   h.name = ss;
   h.address = NULL;
-  if (host_find_byname(&h, NULL, FALSE) != HOST_FIND_FAILED)
+  if (host_find_byname(&h, NULL, NULL, FALSE) != HOST_FIND_FAILED)
     {
     host_item *hh;
     for (hh = &h; hh != NULL; hh = hh->next)
